/**
 * Proposition section of Enslavement, Noon Two.
 * @param {SlaveNoonTwoController} controller The Enslavement, Noon Two controller.
 */
function SlaveNoonTwoBadGag(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preGag;
    var subController = ctrl.gag;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new SlaveNoonTwoBadGagRefuse(ctrl);
    var secondNext = new SlaveNoonTwoBadGagOpen(ctrl);

    var firstChoice = SlaveNoonTwoBadGagChoice.REFUSE;
    var secondChoice = SlaveNoonTwoBadGagChoice.OPEN;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.enslavement.noonTwo.gagged = true;
        var name = ctrl.misc.name();
        var declined = ctrl.blackmail.get() === SlaveNoonTwoOfferChoice.DECLINE;
        var toReturn = `[p]
                            "[color=gblue]Well, ${name}, I'm nothing if not a man of my word.[/color]"
                            The wolf ominously states, taking the piece of rope in one hand, and his underwear in the other.
                            You take a${declined ? `nother` : ``} small step backwards, ending up with your back pressed against the far-end of your cell.
                            Trapped between hard, unyielding, splintery wood and a maniacal canine intent on shoving his dirty laundry down your throat.
                            "[color=gblue]Remember: [b]you[/b] chose this path.
                            Not me.[/color]"
                        [/p]
                        [p]
                            Panic wells up inside of you as Fenrir bridges the gap between the two of you.
                            You consider fleeing, but you doubt you'd get far.
                            Even [i]if[/i] you managed to get past the lupine somehow,
                            either him or his gang would [b]hunt you down[/b] a second time.
                            And you don't think they'll make the mistake of trying to [i]sell[/i] you [b]twice[/b].
                        [/p]
                        [p]
                            Fearing for your very life, you stay still as the big bandit holds his disgusting linen underpants in front of your face.
                            He seems to be getting a cruel kick out of letting you take a [b]good look[/b] at the vile thing he's about to force into your mouth.
                            His briefs are visibly worn, small scraps of fur and many thick, grey hairs intertwined with the sloppily stitched-together fabric.
                            They [b]reek[/b] of stale sweat, and there are many bits of [b]filth[/b] bunched up around the leg-holes and the waistband.
                        [/p]
                        [p]
                            The seams are partially [i]discolored[/i] to a revolting, slightly yellowish tint.
                            And that's just what you can [b]see[/b].
                            You can only [b]imagine[/b] how dirty the insides must be.
                            The [b]stench[/b] alone is making your eyes water.
                            He was wearing this for [b]far[/b] longer than yesterday alone, that much is [b]very[/b] clear.
                            For the entire journey, perhaps, if not even longer.
                            You almost begin to [b]retch[/b], but manage to retain your composure through pride,
                            determination and the [b]will[/b] not to show weakness in front of Fenrir.
                        [/p]
                        [p]
                            "[color=gblue]Don't make this worse for yourself. Open your muzzle.[/color]"
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Refuse</button>`;

        var tooltip = global.parser.parse(`Fenrir can't gag you if you don't open your mouth.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Open up</button>`;

        var tooltip = global.parser.parse(`Don't bother to resist.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var SlaveNoonTwoBadGagChoice = {
    REFUSE: 1,
    OPEN: 2
};
