/**
 * Information about the class.
 * @param {ForestController} controller The forest controller.
 * @param {ForestBanditReward} reward The reward section.
 */
function ForestBanditIntervene(controller, reward) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var bargainId = "banditInterveneBargainButton";
    var fightId = "banditInterveneFightButton";
    
    var toReward = reward;
    var toBargain = new ForestBanditBargain(ctrl, toReward);

    var content = ` <div class=text>
                        You step out from behind your cover, and announce your presence by loudly clearing your throat.
                        The bandits seem startled.
                        One of them audibly gasps in surprise.
                        Their combined stupefaction rapidly fades, however, 
                        as they realize you're not some knight in shining armor, but instead a lowly kobold like themselves.
                    </div>
                    <div class="text">
                        The leader addresses you in the deepest, most authorative voice he can muster:
                        "<span class="bold brightred">You! Kobold! What are you doing here?! Go away! This is MY...</span>"
                        He quickly corrects himself.
                        "<span class="bold brightred">...OUR dragon toy! Not yours! Get your own!</span>"
                    </div>
                    <div class=text>
                        You admittedly haven't thought this plan through very well. 
                        There's five of [i]them[/i] and only one of [i]you[/i].
                        Things might go [b]very[/b] wrong for you if you don't pick your next moves carefully.
                    </div>`;

    var rock = `    <div class=text>
                        With one of the kobolds knocked out, you figure your odds of striking a deal or manhandling the situation have improved dramatically.
                        At least you [i]hope[/i] so.
                        You step out from behind your cover, and slowly, non-threateningly approach the situation.
                    </div>
                    <div class=text>
                        The ringleader's visage says it all.
                        His face goes from slightly scared, to confused, to downright [b]angry[/b].
                        At least his knife seems to give the dragoness' throat just a little more leeway.
                        The three remaining underlings breathe a collective sigh of relief, before joining their leader in glaring you down.
                    </div>
                    <div class="text">
                        "<span class="bold brightred">You! Kobold!</span>" 
                        the ringleader finally speaks. 
                        "<span class="bold brightred">What is your problem?! Why you throw rock?! Is MY...</span>"
                        He quickly corrects himself.
                        "<span class="bold brightred">...OUR dragon toy! Not yours! Go away! Get your own!</span>"
                    </div>
                    <div class=text>
                        You admittedly haven't thought this plan through very well. 
                        There's still four of [i]them[/i] and only one of [i]you[/i].
                        Not to mention the fact that you're now completely disarmed.
                        Things might go [b]very[/b] wrong for you if you don't pick your next moves carefully.
                    </div>`;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    function buildRock() {
        var toReturn = rock;
        toReturn = global.parser.parse(toReturn);
        return toReturn;
    }

    function buildRegular() {
        var toReturn = content;
        toReturn = global.parser.parse(toReturn);
        return toReturn;
    }

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var src = ctrl.bandit.intervene.getInterveneSource();
        if (src === ForestBanditInterveneSourceEnum.ROCK) {
            return buildRock();
        }
        return  buildRegular();
    }

    /**
     * Builds and returns the bargain choice button.
     */
    function buildBargainChoice() {
        var disabled = ctrl.bandit.intervene.hasMadeInterveneChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.intervene.getInterveneChoice() == ForestBanditInterveneChoiceEnum.BARGAIN;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${bargainId}" ${disabled}>Bargain</button>`;

        var tooltip = `Perhaps you can reason with the bandits in one way or another.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the intervene choice button.
     */
    function buildFightChoice() {
        var disabled = ctrl.bandit.intervene.hasMadeInterveneChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.intervene.getInterveneChoice() == ForestBanditInterveneChoiceEnum.FIGHT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fightId}" ${disabled}>Fight</button>`;

        var tooltip = `Attempt to take on the entire gang in combat. <span class="todo">UNFINISHED</span>`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You attempt to: </div>`;
        choice += buildBargainChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFightChoice();
        choice += `</div>`;
        return choice;
    }
    
    /**
     * Disables both buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + bargainId), $("#" + fightId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }
    
    /**
     * Adds a listener to the bargain button.
     */
    function registerBargainListener() {
        $("#" + bargainId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bandit.intervene.makeInterveneChoice(ForestBanditInterveneChoiceEnum.BARGAIN);
            loadNextPart(toBargain);
        });
    }

    /**
     * Adds a listener to the fight button.
     */
    function registerFightListener() {
        $("#" + fightId).click(function () {
            /*$(this).addClass("selected");
            disableButtons();
            ctrl.bandit.intervene.makeInterveneChoice(ForestBanditInterveneChoiceEnum.FIGHT);
            loadNextPart(toFight);*/
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "banditIntervene";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.bandit.intervene.hasInterveneSource()) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += toBargain.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.bandit.intervene.hasInterveneSource()) {
            return;
        }

        toBargain.registerListeners();

        if (ctrl.bandit.intervene.hasMadeInterveneChoice()) {
            return;
        }

        registerBargainListener();
        registerFightListener();
    };
}

var ForestBanditInterveneChoiceEnum = {
    BARGAIN: 1,
    FIGHT: 2
};

/**
 * Source for the intervention part of the bandit part.
 */
var ForestBanditInterveneSourceEnum = {
    APPROACH: 1,
    ROCK: 2
};

