/**
 * Relieve choice of the bandit encounter.
 * @param {ForestController} controller The forest controller.
 */
function ForestBanditRelieve(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var eagerId = "banditRelieveEagerButton";
    var reluctantId = "banditRelieveReluctantButton";

    var toSex = new ForestBanditRelieveSex(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var didRock = (ctrl.bandit.getArrivalChoice() === ForestBanditArrivalChoiceEnum.ROCK);
        var toReturn = `[p]
                            You realize that you can't [i]stop[/i] the bandits.
                            They outnumber you, they have [i]actual[/i] weapons, and there's not a doubt in your mind that they know how to use them.
                            You are reminded of an old saying: [i]if you can't beat'em, join'em[/i].
                            That is to say, if you can't prevent the dragoness from getting [i]raped[/i],
                            you might as well [i]ease[/i] her burden by getting some of the kobolds off yourself.
                        [/p]
                        [p]
                            You still need to figure out a way to convince the gang to [i]give up[/i] a dragoness, 
                            one of the most sought after sexual partners, in favor of plain old [b]you[/b].
                            "<span class="bold brightred">Well?! What do you want?!</span>" the ringleader impatiently shouts,
                            yet you don't let him interrupt your train of thoughts.
                            Wait a minute.
                            The leader.
                            Yeah, that might work.
                        [/p]
                        [p]
                            "<span class="bold brightred">What, you mute?! Speak, or I'll gut you!</span>" the knife-wielding kobold threatens.
                            With a devious and bold plan hatched, you finally acknowledge him,
                            opening up with a flattering comment about how [i]great[/i] a job [b]he[/b] did in tackling the dragoness.
                            Hatred quickly melts from his face, and in its place comes an appreciative blush combined with a devilish grin.
                        [/p]
                        [p]
                            Seemingly on the right track, you continue to push your luck.
                            You explain that you witnessed [b]his[/b] [i]grand triumph[/i] from behind the cover of a nearby tree.
                            While [b]he[/b]'s doing an excellent job at keeping the dragoness in check, you mention that there's a small [i]problem[/i] with his current plan.
                            A little oversight, nothing more, you assure the kobold.
                        [/p]
                        [p]
                            Your scheme appears to be working, thus far.
                            You've smooth talked the leader of the gang to the point where he doesn't even ask you [i]what[/i] is wrong with his scheme,
                            he simply waits for you to explain yourself out of your own volition, a marked improvement from the angry 
                            ${didRock ? "knife waving" : "sneering"} you received about a minute ago.
                        [/p]
                        [p]
                            You explain that there's only [i]one[/i] dragoness to split amongst all ${didRock ? "four" : "five"} of them.
                            Hardly a problem, [b]but[/b] [i]you[/i], as a mere spectator to the entire ordeal, believe that wouldn't be very [i]fair[/i].
                            After all, [i]the great leader[/i] did all of the work, assisted by the ${didRock ? "proud man" : "two proud men"} holding down the woman's arms.
                        [/p]
                        [p]
                            While you completely agree that he and his ${didRock ? "finest comrade" : "finest couple of henchmen"} [b]deserve[/b] to reap the spoils of their labour,
                            you personally think that the two kobolds pinning down the dragoness' legs didn't really do all that much.
                            As such, you feel that they don't [i]deserve[/i] a go at one of the [i]hottest[/i], 
                            [i]tightest[/i] [b]most fuckable[/b] things in these entire woods.
                            They [i]do[/i], however, deserve [i]some[/i] compensation for their effort.
                            What a conundrum!
                        [/p]
                        [p]
                            The two bandits you spoke ill of glare at you much like their leader used to.
                            They obviously disagree with just about [b]everything[/b] you're saying.
                            The well-dressed and well-armed kobold, on the contrary, 
                            is eating out of the palm of your hand, nodding agreeably in response to your every word.
                            You've got him hooked.
                            Now to reel him in.
                        [/p]
                        [p]
                            In order to [i]solve[/i] this dilemma, you suggest the following:
                            the [i]great[/i] leader and his right-hand ${didRock ? "man" : "men"} get to [i]abuse[/i] the [b]dragoness whore[/b] for as long as they'd like,
                            while the two [i]inept[/i] members of the gang sate their lust on something -- some[i]one[/i] -- of lesser value.
                            Someone like [b]you[/b].
                        [/p]`;
        if (didRock) {
            toReturn += `[p]
                            Right as it seems you've [b]completely[/b] convinced the ringleader,
                            a sudden thought strikes his otherwise compliment-addled mind.
                            His face scrunches up inquisitively, and his eyes narrow.
                            In a questioning voice, he speaks:
                            "<span class="bold brightred">Why did you throw a rock at us?</span>"
                        [/p]
                        [p]
                            Luckily, you manage to keep your cool.
                            With a smug, toothy smirk you point towards their unconscious comrade, 
                            and reply that [i]he[/i] didn't deserve neither the dragoness [i]nor[/i] you.
                            Your apparent cruelness seems to have struck the leader's fancy.
                            He burst out into hearty laughter, amused sufficiently by your reasoning to [i]overlook[/i] your earlier transgression.
                        [/p]`;
        }
        toReturn += `   [p]
                            Complaints emerge from the two belittled gangmembers, yet they are quickly silenced by their boss.
                            He has made his decision.
                            He'll follow your [i]advice[/i].
                            Reluctantly, the scorned felons let go of the dragoness' legs.
                            Turning away from the scene, their anger-filled eyes fall upon [b]you[/b].
                            As the remaining bandits begin to abuse the woman, the pair of upset outcasts advance in your direction.
                        [/p]
                        [p]
                            To make sure they don't get any [i]funny ideas[/i], you preemptively drop to your knees.
                            You don't intend to lose your virginity to these disgruntled muggers.
                            Mistaking your [i]precaution[/i] for eagerness, they strip out of their meagre clothing mid-march,
                            a pair of fully erect kobold dicks of considerable size coming into view.
                        [/p]`;
        
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the eager choice button.
     */
    function buildEagerChoice() {
        var disabled = ctrl.bandit.intervene.hasMadeRelieveChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.intervene.getRelieveChoice() == ForestBanditRelieveChoice.EAGER;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${eagerId}" ${disabled}>Eagerly</button>`;

        var tooltip = `Fervently jerk and suck the two bandits off. 
                       Maybe you can even touch yourself a little, if you can spare one of your hands for a few minutes.`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 205px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the reluctant choice button.
     */
    function buildReluctantChoice() {
        var disabled = ctrl.bandit.intervene.hasMadeRelieveChoice() ? "disabled" : "";
        var isSelected = ctrl.bandit.intervene.getRelieveChoice() == ForestBanditRelieveChoice.RELUCTANT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${reluctantId}" ${disabled}>Reluctantly</button>`;

        var tooltip = `Hesitantly stroke the two kobolds to orgasm.
                       They aren't expecting you to use your muzzle, are they?`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">How do you service them?</div>`;
        choice += buildEagerChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildReluctantChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all four buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + eagerId), $("#" + reluctantId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the bargain button.
     */
    function registerEagerListener() {
        $("#" + eagerId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bandit.intervene.makeRelieveChoice(ForestBanditRelieveChoice.EAGER);
            loadNextPart(toSex);
        });
    }

    /**
     * Adds a listener to the bargain button.
     */
    function registerReluctantListener() {
        $("#" + reluctantId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bandit.intervene.makeRelieveChoice(ForestBanditRelieveChoice.RELUCTANT);
            loadNextPart(toSex);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "banditRelieve";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.bandit.intervene.getBargainChoice() !== ForestBanditBargainChoice.RELIEVE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        // Build children here.
        part += toSex.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.bandit.intervene.getInterveneChoice() !== ForestBanditInterveneChoiceEnum.BARGAIN) {
            return;
        }

        toSex.registerListeners();

        if (ctrl.bandit.intervene.hasMadeRelieveChoice()) {
            return;
        }

        registerEagerListener();
        registerReluctantListener();
    };
}

var ForestBanditRelieveChoice = {
    EAGER: 1,
    RELUCTANT: 2
};
