/**
 * Hide section of the woods.
 * @param {ForestController} controller The Forest controller.
 */
function ForestWoodsWeatherContinue(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You are stuck in purgatory, between being conscious enough to [b]feel pain[/b],
                            and being too far [b]out of it[/b] to do anything but stare up at the forest canopy.
                            Slowly, but steadily, the bee-venom begins to fade from your system, your hazy mind growing sharper with every passing minute.
                            Your limbs, too, begin to regain their strength, although you doubt they'll [i]ever[/i] [b]fully[/b] return to normal.
                        [/p]
                        [p]
                            About half an hour after falling from the tree, you wearily roll off your aching back onto your relatively unscathed stomach,
                            a breath of cold air running over your body somewhat soothing the [b]burning pain[/b].
                            You begin to pluck stingers from your arms, a great deal of them luckily having fallen off on your way down from the branch.
                            Once your arms are mostly cleared, you toilsomely rise to your feet, your weakened legs barely able to carry your weight.
                        [/p]
                        [p]
                            You cast a final fearful gaze up at the beehive, almost losing your balance in the process.
                            You may not have fully recovered yet, but you [b]really[/b] need to get going.
                            Night will fall soon, and you don't want to be out here when the [b]real[/b] bandits show up.
                            Especially not like this.
                            Let alone while carrying a bundle of gold around your neck.
                            You need to go home.
                        [/p]
                        [p]
                            Step by step, you move your numb legs forwards.
                            It takes a massive amount of concentration just for you to be able to stand upright, let alone walk.
                            With the apitoxin dissipating, your gait swiftly improves from stumbling to limping.
                            It seems as though movement aids your recovery.
                            Mere minutes after you set off, you regain full control of your limbs.
                            While still severely injured, at least you [i]think[/i] you can make it home without collapsing.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.homeSceneName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (!ctrl.preContinue.continued()) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (!ctrl.preContinue.continued()) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}
