/**
 * Barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function BarnRecognize(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var toUnaware = new BarnUnaware(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You try to come up with a suitable insult to hurl Cassidy's way,
                            yet your mind is far too busy trying to come up with a way to escape this darn table to think of anything.
                            Then you remember.
                            You've seen this table before.
                            Well, not exactly [i]this[/i] one, but the very same model.
                        [/p]
                        [p]
                            A few years ago,
                            when you were summoned to the headquarters of the Three-Tailed Rat Clan to [i]explain[/i] your lack of [i]contribution[/i] to [i]the cause[/i],
                            you were [i]introduced[/i] to some of their [i]torture[/i] practices to [b]scare[/b] you into coughing up the coins you owed at the time.
                            Luckily, you were spared any sort of pain yourself,
                            but you were quite well informed what would happen to you should you [b]fail[/b] to pay them back.
                            One of the tools used by the gang to keep whoever they're trying to torment firmly in place,
                            just so happened to be a table like the one you're strapped to now.
                        [/p]
                        [p]
                            You calm down a little, useful memories and a sliver of hope coursing through your mind.
                            "[color=orange]Good. I knew you wouldn't put up too much of a fight.[/color]"
                            Cassidy speaks, mistaking your slight relaxation for some form of acceptance.
                            Like you've already come to terms with what she plans on doing to you.
                            Like you'd [b]ever[/b] willingly agree to this [b]horrible[/b] life she has in store for you.
                        [/p]
                        [p]
                            The canine turns around, facing away from you, her tail wagging excitedly,
                            her rear tauntingly swaying from side to side, as if to add insult to injury.
                            "[color=orange]Anyway, time for your first shot. I'll be [b]right[/b] back. Don't go anywhere![/color]"
                            She giggles at her own joke in a way that only a true [b]psychopath[/b] could,
                            as she begins to walk towards a nearby stash of what you can only presume to be Seki's
                            sold out ${global.kobold.inventory.createItemWithTooltip(new FemPlus())} supply.
                        [/p]
                        [p]
                            With the woman's back turned, you spot the perfect opportunity to make a move.
                            If you recall, the mechanism strapping you to the table is operated via a small lever on the underside of the flat, wooden surface.
                            While your hands and legs may be bound, [i]one[/i] of your appendages is not: your tail!
                            You slither the prehensile body-part underneath the table, blindly searching for the lever.
                        [/p]
                        [p]
                            You swing your tail to the left.
                            Nothing.
                            To the right.
                            Again, nothing!
                            Are you mistaken?
                            Is this a different model?
                            Can you simply not reach far enough?
                            Frustrated and scared, you check the left hand side once more.
                            [b]Nothing![/b]
                            Almost giving up hope, you flail your tail about in random directions, like a wounded animal writhing in pain.
                            Then, suddenly, the scaly appendage hits something!
                            With a soft [b]clink[/b], the cuffs keeping you bound to the table unlock.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "barnRecognize";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.eager.get() !== BarnEagerChoice.RECOGNIZE) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += toUnaware.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.eager.get() !== BarnEagerChoice.RECOGNIZE) {
            return;
        }

        toUnaware.registerListeners();
    };
}
