/**
 * Steal section of the Farmer House scene.
 * @param {BreedingFarmController} controller The Farmer House controller.
 */
function FarmerHouseShedTrident(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var nextId = arguments.callee.name + "ButtonOne";

    var next = new FarmerHouseShedRun(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var toReturn = `[p]
                            You grab the pitchfork leaning against the wall.
                            It's [b]way[/b] too big to be of much use to you.
                            You place it atop the tiny table that holds most of the tools, the wooden handle sticking out over the edge.
                            You need to make this thing smaller somehow, so you can use it as a weapon.
                            An idea forms within your mind.
                        [/p]
                        [p]
                            You take the large, well-used, two-handed saw.
                            Another tool not cut for someone of your stature.
                            You can barely even [b]lift[/b] it due to the two heavy weights attached to either end.
                            You imagine this saw is used for cutting down trees with, but it's the best thing around to shorten the pitchfork with.
                        [/p]
                        [p]
                            Through some fidgeting and a lot of physical exertion to the point of annoyance,
                            you finally manage to get the saw to stand upright near the edge of the table, with the pitchfork's handle underneath its dull blade.
                            You gingerly begin to roll the heavy contraption back and forth as best you can, slowly carving it into the wooden grip below.
                            You need to stop your sawing ever so often, either to adjust the pitchfork to ensure it doesn't fall down, or just to take a breather.
                        [/p]
                        [p]
                            This tool was [b]not[/b] meant to be used  by a kobold.
                            You don't even think it was made for a [i]single[/i] human to employ.
                            The fact that it's more dull than your claws isn't helping.
                            Still, you perservere, deftly sawing away millimetre after millimetre, until finally, you reach the end.
                            As the saw carves slightly into the table, a large part of what was once the pitchfork's handle tumbles down to the ground.
                        [/p]
                        [p]
                            You did it!
                            With one final burst of effort, you quietly lay the saw back down onto table,
                            before taking hold of your brand new ${global.kobold.inventory.createItemWithTooltip(new KoboldTrident())}.
                            You raise the former tool up to inspect it.
                            Far smaller than before.
                            Lighter, and easier for you to use.
                            Perfect.
                            You stab the air a few times to get a feeling for your new weapon.
                            ${weapon ? `You ditch your old ${global.kobold.inventory.createItemWithTooltip(weapon)}.
                            You have found something better.` : `You feel like a spear-wielding knight.`}
                        [/p]
                        [p]
                            You [b]love[/b] it.
                            And you made it yourself!
                            Well, sort of, anyway.
                            The important part is that it's yours now.
                            With the pitchfork-turned-trident in hand, you excitedly leave the shed, ready to face whatever the world has to throw at you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.postshed.continued() ? "disabled" : "";
        var isSelected = ctrl.postshed.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${nextId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + nextId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + nextId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postshed.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.shed.get() !== FarmerHouseShedChoice.TRIDENT) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.postshed.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.shed.get() !== FarmerHouseShedChoice.TRIDENT) {
            return;
        }

        next.registerListeners();

        if (ctrl.postshed.continued()) {
            return;
        }

        registerNextListener();
    };
}
