/**
 * Doggo section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveBasiLeave(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;
    var changeController = ctrl.basiLeave;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var doggoChoice = ctrl.doggo.get();
        var treats = doggoChoice === ctrl.choices.doggo.FEED;
        var act = "petting";
        if (doggoChoice === ctrl.choices.doggo.FEED) {
            act = "feeding";
        }
        if(doggoChoice === ctrl.choices.doggo.FETCH) {
            act = "playing fetch with";
        }
        var toReturn = `[p]
                            After a few more minutes of walking, the two of you reach the entrance to the cave.
                            As you expected: the water was a stale branch of the river!
                            Climbing out of the cavern, you sit your butt down on the riverside dirt, and take a breather.
                            You feel like you've earned one.
                        [/p]
                        [p]
                            Uttering an exhausted sigh, you realize your basilisk buddy didn't follow you out here.
                            Looking back at the cave, you can see the creature staring at you from the shade.
                            It tentatively pokes one of its legs out into the sunlight, but with a loud hiss, quickly retracts the limb once more.
                            You figure it can't stand the sun, or something like that.
                            The beast stares at you longingly, before letting out a sad whine.
                            Hanging its head, it turns around and ventures back into the cave.
                        [p]
                            You consider going after the basilisk for a [b]proper[/b] goodbye, but you figure this way is for the best.
                            You can't stick around this cave forever, and even if you [b]did[/b] wait until nightfall,
                            the guards of Varanar would [b]never[/b] let such a monster into the city, no matter how well-behaved it is.
                            Maybe you'll return one day, with some ${treats ? `more` : ``} tasty treats for it to eat.
                        [/p]
                        [p]
                            Who are you kidding?
                            You can't even properly feed yourself, let alone a monster twice your weight.
                            You hope it'll be okay.
                            If a knight finds this cave, you fear your new friend won't last long.
                            But it's managed to survive until now, as have you.
                            With some luck, the both of you'll be just fine.
                        [/p]
                        [p]
                            The trip into the well was a harrowing experience.
                            You defied death more than once, but meeting the basilisk was all worth it.
                            You've never really owned a [i]pet[/i].
                            You wonder if that [i]hearty[/i] and [i]wholesome[/i] feeling you felt when ${act} the basilisk is how pet-owners feel all the time.
                        [/p]
                        [p]
                            Besides a friend, you've gained a few awesome stories to tell once you've grown old and wise.
                            Stories like the ones Seki used to tell you when you were little.
                            You went through all that trouble to get your mind off of him, and here you are reminiscing once more.
                            You just can't help yourself from clinging to times long gone.
                            Things were so much simpler back then.
                            You miss those days.
                        [/p]
                        [p]
                            You find yourself close to the road to Varanar.
                            In fact, you believe you passed this very place earlier, when heading for the hamlet.
                            You never knew there was a cave here, but then again,
                            you've never really bothered to scour the edge of the river for points of interest.
                        [/p]
                        [p]
                            The sun is slowly starting to set.
                            You drag yourself to your feet.
                            It's time to go.
                            The capital isn't [i]that[/i] far off.
                            If your legs don't give out from underneath you, you might still make it there before nightfall.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(ctrl.nextSceneName, ctrl.nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        ctrl.bgm.stopLoop();
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!ctrl.nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (ctrl.nextSceneReady) {
            registerContinueListener();
        }
    };
}
