/**
 * Stare section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveResistSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.avert;
    var changeController = ctrl.resistSucceed;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.fight;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            [b]No![/b]
                            You can't let this happen!
                            You may have stared into the basilisk's eyes like an [b]idiot[/b], playing right into its cards, but this fight is [b]not[/b] over yet!
                            You begin to focus all of your willpower, all of your energy, [b]everything[/b] you've got on [b]moving[/b] your limbs.
                        [/p]
                        [p]
                            Your hand.
                            That first.
                            If you can move your hand, then you can move [b]anything[/b].
                            You close your eyes.
                            You take deep breaths through your nose.
                            You try to find some inner peace, not an easy task with a huge feral beast drawing near,
                            every scrape of its sharp claws over the stone cavern floor interrupting your meditation.
                            [b]Move, damnit![/b]
                        [/p]
                        ${global.ui.condSucc(`Sufficient sanity, intelligence and endurance`)}
                        [p]
                            Suddenly, something changes.
                            Your body feels [i]off balance[/i].
                            You open your eyes and look down, to find your right hand lifted ever-so slightly above the ground,
                            no longer supporting the rest of your petrified body.
                            You did it!
                            Through sheer [b]determination[/b] alone, you have managed to raise your hand off of the floor!
                        [/p]
                        [p]
                            Hope rising inside of your mind, you watch as the rocky exterior that had overtaken your limb gives way to your natural scales once more.
                            You realize that the petrification is [b]not[/b] a death sentence.
                            It's like a fast-spreading [b]disease[/b].
                            A [b]blight[/b] brought upon you by staring into the monster's oh-so-alluring eyes.
                            But not an uncurable one.
                            It can be [b]fought[/b].
                            It can be [b]beaten[/b].
                            You just need to [b]concentrate[/b]
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(UndergroundCaveAvertChoice.FIGHT);
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
