/**
 * Controller for the Varanar, Afternoon Three scene.
 * @param {string} sceneId The id of the scene.
 * @param {AfternoonThreeParlorController} superCtrl The supercontroller.
 */
function AfternoonThreeParlorController(sceneId, superCtrl) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = superCtrl.nextSceneReady;
    this.nextSceneName = superCtrl.nextSceneName;
    this.nextFolderName = superCtrl.nextSceneName;

    this.misc = superCtrl.misc;
    this.postMassageMisc = new MiscController(id, "postMassageMisc");

    this.arrival = new ChoiceController(id, "parlorArrival");
    this.entrance = new ChoiceController(id, "entrance");
    this.bargain = new ChoiceController(id, "bargain");

    this.accept = new ChangeController(id, "accept", function () {
        if (global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess) {
            global.kobold.inventory.coins = global.kobold.inventory.coins - 20;
        } else {
            global.kobold.inventory.coins = global.kobold.inventory.coins - 40;
        }
        global.kobold.status.mental.increase(3, "Impending massage");
        that.storeInventory();
        that.clearInventory();
    }, function () {
        if (global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess) {
            return global.ui.change.buildNotification([
                new Change("Impending massage", "Sanity", +3),
                new Change("Paid", "Gold", -20)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Impending massage", "Sanity", +3),
                new Change("Paid", "Gold", -40)
            ]);
        }
    });

    this.secondOffer = new ChoiceController(id, "secondOffer");

    this.secondAccept = new ChangeController(id, "secondAccept", function () {
        global.kobold.status.mental.increase(3, "Impending massage");
        that.storeInventory();
        that.clearInventory();
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Impending massage", "Sanity", +3)
        ]);
    });

    this.secondDecline = new ChangeController(id, "secondDecline", function () {
        global.kobold.status.physical.decrease(3, "Face scraped");
        global.kobold.status.mental.decrease(3, "Sudden fight");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Face scraped", "Health", -3),
            new Change("Sudden fight", "Sanity", -3)
        ]);
    });

    this.run = new ChangeController(id, "run", function () {
        global.kobold.status.physical.decrease(6, "Hand whipped");
        global.kobold.status.mental.decrease(5, "Pain");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Hand whipped", "Health", -6),
            new Change("Pain", "Sanity", -5)
        ]);
    });

    this.diveSucceed = new ChangeController(id, "diveSucceed", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(12, "Diving and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Diving", "Stamina", -3),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.dodgeSucceed = new ChangeController(id, "dodgeSucceed", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Dodging and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    function calcTankDamage() {
        var chest = that.misc.shirt();
        var dmg = 8;
        if (chest) {
            dmg -= Math.floor(chest.defence() / 3);
        }
        return Math.max(dmg, 5);
    }

    this.tankSucceed = new ChangeController(id, "tankSucceed", function () {
        global.kobold.status.physical.decrease(calcTankDamage(), "Hit by jar");
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Tanking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Hit by jar", "Health", -calcTankDamage()),
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.duck = new ChangeController(id, "duck", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Ducking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.fight = new ChangeController(id, "fight", function () {
        var weapon = that.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (weaponSave) {
            global.kobold.status.mental.increase(8, "Won fight");
            global.kobold.status.stamina.decrease(8, "Combat");
        } else {
            if (egg) {
                global.kobold.status.physical.decrease(2, "Hit during fight");
                global.kobold.status.mental.increase(6, "Won fight");
                global.kobold.status.stamina.decrease(8, "Combat");
                global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
            } else {
                global.kobold.status.physical.decrease(2, "Hit during fight");
                global.kobold.status.mental.increase(6, "Won fight");
                global.kobold.status.stamina.decrease(8, "Combat");
            }
        }
    }, function () {
        var weapon = that.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (weaponSave) {
            return global.ui.change.buildNotification([
                new Change("Parried blow", "Sanity", +3),
                new Change("Won fight", "Sanity", +5),
                new Change("Fighting", "Stamina", -8)
            ]);
        } else {
            if (egg) {
                return global.ui.change.buildNotification([
                    new Change("Minor injury", "Health", -2),
                    new Change("Saved by armor", "Sanity", +1),
                    new Change("Won fight", "Sanity", +5),
                    new Change("Fighting", "Stamina", -8),
                    new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
                ]);
            } else {
                return global.ui.change.buildNotification([
                    new Change("Minor injury", "Health", -2),
                    new Change("Saved by armor", "Sanity", +1),
                    new Change("Won fight", "Sanity", +5),
                    new Change("Fighting", "Stamina", -8)
                ]);
            }
        }
    });

    this.fightLoot = new ChangeController(id, "fightLoot", function () {
        global.kobold.inventory.coins = global.kobold.inventory.coins + 36;
        global.kobold.inventory.addItem(new aPlusLotion());
        global.kobold.inventory.addItem(new aPlusLotion());
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Loot", "Gold", +36),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new aPlusLotion()), +2)
        ]);
    });

    this.preMassage = new ContinueController(id, "preMassage");
    this.massage = new ChoiceController(id, "massage");

    this.arms = new ChangeController(id, "arms", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Arms strengthened", "Strength", +1)
        ]);
    });

    this.back = new ChangeController(id, "back", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Back loosened", "Endurance", +1)
        ]);
    });

    this.feet = new ChangeController(id, "feet", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Feet rubbed", "Agility", +1)
        ]);
    });

    this.postMassage = new ContinueController(id, "postMassage");

    this.friend = new ChangeController(id, "friend", function () {
        global.kobold.status.mental.increase(5, "Befriending masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Avoided violence", "Sanity", +5),
            new Change("Distrust confirmed", "Sanity", +3),
            new Change("Resolve strengthened", "Sanity", +4),
            new Change("Implied suicide?", "Sanity", -7)
        ]);
    });

    this.foe = new ChangeController(id, "foe", function () {
        global.kobold.status.physical.decrease(3, "Half-completed sleep-grip");
        global.kobold.status.mental.increase(3, "Knocking out masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Half-completed sleep-grip", "Health", -3),
            new Change("Unintended excessive injury", "Sanity", -3),
            new Change("Did not get knocked out", "Sanity", +2),
            new Change("Distrust confirmed", "Sanity", +3),
            new Change("Resolve strengthened", "Sanity", +4),
            new Change("Is she dead?", "Sanity", -3)
        ]);
    });

    this.preKnockout = new ChangeController(id, "preKnockout", function () {
        global.kobold.status.physical.decrease(5, "Sleep-grip");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Sleep-grip", "Health", -5)
        ]);
    });

    this.knockout = new ChangeController(id, "knockout", function () {
        global.kobold.status.physical.increase(2, "Nice nap");
        global.kobold.status.mental.decrease(5, "Tied up");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Decent nap", "Health", +2),
            new Change("Tied up", "Sanity", -2),
            new Change("Rising panic", "Sanity", -3)
        ]);
    });

    this.cut = new ChangeController(id, "cut", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.break = new ChangeController(id, "break", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.escape = new ChangeController(id, "escape", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.preMasseuse = new ContinueController(id, "preMasseuse");

    this.masseuse = new ChoiceController(id, "masseuse");

    this.bluff = new ChangeController(id, "bluff", function () {
        global.kobold.status.mental.increase(3, "Befriended masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Avoided violence", "Sanity", +5),
            new Change("Made a friend", "Sanity", +5),
            new Change("Implied suicide?", "Sanity", -7)
        ]);
    });

    this.trap = new ChangeController(id, "trap", function () {
        global.kobold.status.mental.increase(1, "Trapped masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Outsmarted masseuse", "Sanity", +2),
            new Change("Excessive violence", "Sanity", -3),
            new Change("She deserved it", "Sanity", +2)
        ]);
    });

    this.overpower = new ChangeController(id, "overpower", function () {
        global.kobold.status.mental.increase(1, "Beat masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Overpowered masseuse", "Sanity", +2),
            new Change("Excessive violence", "Sanity", -3),
            new Change("She deserved it", "Sanity", +2)
        ]);
    });

    this.preItemGet = new ContinueController(id, "preItemGet");

    this.itemGet = new ChangeController(id, "itemGet", function () {
        that.restoreInventory();
    }, function () {
        return "";
    });

    this.itemGetFlee = new ChangeController(id, "itemGetFlee", function () {
        global.kobold.status.mental.increase(6, "Escaped massage parlor");
        global.kobold.status.stamina.decrease(9, "Sneaking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Tense sneaking", "Sanity", -2),
            new Change("Close call", "Sanity", -2),
            new Change("Escaped massage parlor", "Sanity", +10),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.itemGetSpare = new ChangeController(id, "itemGetSpare", function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
        }
        global.kobold.status.mental.increase(10, "Knocked out Tychus");
    }, function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            return global.ui.change.buildNotification([
                new Change("Knocked out Tychus", "Sanity", +3),
                new Change("Not a murderer", "Sanity", +3),
                new Change("Vented frustration", "Sanity", +2),
                new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Knocked out Tychus", "Sanity", +3),
                new Change("Not a murderer", "Sanity", +3),
                new Change("Vented frustration", "Sanity", +2)
            ]);
        }
    });

    this.itemGetKill = new ChangeController(id, "itemGetKill", function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
        }
        global.kobold.status.mental.increase(1, "Killed Tychus");
    }, function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            return global.ui.change.buildNotification([
                new Change("Killed Tychus", "Sanity", +3),
                new Change("Murderer", "Sanity", -4),
                new Change("Vented frustration", "Sanity", +2),
                new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Killed Tychus", "Sanity", +3),
                new Change("Murderer", "Sanity", -4),
                new Change("Vented frustration", "Sanity", +2)
            ]);
        }
    });

    this.itemGetFight = new ChoiceController(id, "itemGetFight");

    this.spareFree = new ChangeController(id, "spareFree", function () {
        global.kobold.status.mental.increase(5, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Liberator", "Sanity", +3),
            new Change("Mixed response", "Sanity", -2),
            new Change("Escaped massage parlor", "Sanity", +5),
            new Change("Helped others escape", "Sanity", +2),
            new Change("Disappearing act", "Sanity", -3)
        ]);
    });

    this.secondLoot = new ChangeController(id, "secondLoot", function () {
        global.kobold.inventory.coins = global.kobold.inventory.coins + 36;
        global.kobold.inventory.addItem(new aPlusLotion());
        global.kobold.inventory.addItem(new aPlusLotion());
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Loot", "Gold", +36),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new aPlusLotion()), +2)
        ]);
    });

    this.killFree = new ChangeController(id, "killFree", function () {
        global.kobold.status.mental.increase(3, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Liberator", "Sanity", +3),
            new Change("Upset reaction", "Sanity", -2),
            new Change("Indecent suggestion", "Sanity", +2)
        ]);
    });

    this.killFreeRefuse = new ChangeController(id, "killFreeRefuse", function () {
        global.kobold.status.mental.increase(4, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Moral highground", "Sanity", +2),
            new Change("Escaped massage parlor", "Sanity", +5),
            new Change("Did the right thing?", "Sanity", -3)
        ]);
    });

    this.killFreeAccept = new ChangeController(id, "killFreeAccept", function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            global.kobold.stats.decreaseStat(StatsEnum.STRENGTH, 1);
            global.kobold.status.physical.decrease(15, "Dislocated arm");
            global.kobold.status.mental.decrease(10, "Dislocated arm");
        } else if (back) {
            global.kobold.stats.decreaseStat(StatsEnum.ENDURANCE, 1);
            global.kobold.status.physical.decrease(20, "Dislocated vertebra");
            global.kobold.status.mental.decrease(10, "Dislocated vertebra");
        } else {
            global.kobold.stats.decreaseStat(StatsEnum.AGILITY, 1);
            global.kobold.status.physical.decrease(15, "Dislocated foot");
            global.kobold.status.mental.decrease(10, "Dislocated foot");
        }
    }, function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            return global.ui.change.buildNotification([
                new Change("Arm dislocated", "Health", -15),
                new Change("Immense pain", "Sanity", -10),
                new Change("Arm dislocated", "Strength", -1)
            ]);
        } else if (back) {
            return global.ui.change.buildNotification([
                new Change("Vertebra dislocated", "Health", -20),
                new Change("Immense pain", "Sanity", -10),
                new Change("Vertebra dislocated", "Endurance", -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Foot dislocated", "Health", -15),
                new Change("Immense pain", "Sanity", -10),
                new Change("Foot dislocated", "Agility", -1)
            ]);
        }

    });

    this.killFreeAcceptLeave = new ChangeController(id, "killFreeAcceptLeave", function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            global.kobold.status.physical.increase(5, "Fixing your arm");
        } else if (back) {
            global.kobold.status.physical.increase(5, "Fixing your back");
        } else {
            global.kobold.status.physical.increase(5, "Fixing your foot");
        }
        global.kobold.status.mental.decrease(-3, "Escaped massage parlor");
    }, function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            return global.ui.change.buildNotification([
                new Change("Popped arm back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        } else if (back) {
            return global.ui.change.buildNotification([
                new Change("Popped spine back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Popped foot back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        }

    });

    this.preFail = new ContinueController(id, "preFail");
    this.preContinue = new ContinueController(id, "preContinue");

    this.fail = new ChoiceController(id, "fail");


    this.secondChoice = {
        DECLINE: 1,
        ACCEPT: 2
    };

    this.secondFightChoice = {
        LOOT: 1,
        FREE: 2
    };


    this.fragments = {};
    this.fragments.failChoices = new AfternoonThreeParlorFailChoices(this);
    this.fragments.masseuseFail = new AfternoonThreeParlorMasseuseFail(this);
    this.fragments.ropeFail = new AfternoonThreeParlorRopeFail(this);
    this.fragments.fightFail = new AfternoonThreeParlorFightFailEnd(this);
    this.fragments.bottleFail = new AfternoonThreeParlorBottleFail(this);
    this.fragments.itemGet = new AfternoonThreeParlorItemGet(this);
    this.fragments.masseuse = new AfternoonThreeParlorMasseuse(this);
    this.fragments.postMassage = new AfternoonThreeParlorPostMassage(this);
    this.fragments.massage = new AfternoonThreeParlorMassage(this);
    this.fragments.secondDecline = new AfternoonThreeParlorSecondDecline(this);
    this.fragments.noGoldAccept = new AfternoonThreeParlorNoMoneyAccept(this);
    this.fragments.decline = new AfternoonThreeParlorDecline(this);
    this.fragments.noGold = new AfternoonThreeParlorNoGold(this);
    this.fragments.accept = new AfternoonThreeParlorAccept(this);

    this.bgm = new AudioController(id, "res/audio/varanar/massageparlor.mp3", 120);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Resets the entire scene to the starting point.
     * Note that the main panel will still need to be cleared and reloaded after this.
     */
    this.reset = function () {
        superCtrl.reset();
    };

    /**
     * Clears and reloads the UI.
     */
    this.reload = function () {
        superCtrl.reload();
    };

    var inventoryKey = "parlorInventoryStorage";
    /**
     * Stores the kobold's inventory. Best used before taking it all off.
     */
    this.storeInventory = function () {
        global.kobold.history.setIfNotSet(id, inventoryKey, JSON.parse(JSON.stringify(global.kobold.inventory)));
    };

    /**
     * Clears (i.e. empties) the kobold's inventory.
     */
    this.clearInventory = function () {
        global.kobold.inventory.load(new Inventory());
    };

    /**
     * Restores the kobold's inventory from history. Best used when regaining items.
     */
    this.restoreInventory = function () {
        global.kobold.inventory.load(global.kobold.history.getKey(id, inventoryKey));
    };
}
