/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorAccept(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorEntranceChoice.ACCEPT;
    var reqController = false; // Set manually
    var subController = ctrl.preMassage;
    var changeController = ctrl.accept;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.massage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var bargainSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var clothes = !ctrl.misc.nude();
        var toReturn = `[p]
                            While you remain skeptical, some relaxation sounds [b]lovely[/b] to you.
                            ${bargainSuccess ? `Twenty coins seems a reasonable price, especially for an entire [b]hour[/b].
                            ` : `Fourty coins is a pretty steep price, but you'll surely get a [b]quality[/b] massage for your coin.`}
                            And you'll get to be touched by a fellow kobold, which brings a few perverted thoughts to mind.
                        [/p]
                        [p]
                            Sure!
                            You'll take it!
                            You fetch ${bargainSuccess ? `twenty` : `fourty`} coins from your pouch, and toss them atop Tychus' desk.
                            "[color=purple]Excellent choice. You won't regret it.[/color]"
                            The man rises from his feet, and escorts you to the massage room.
                            The orange kobold stands at attention as her [i]boss[/i] walks in the room.
                            "[color=purple]Toss your belongings in the corner.
                            ${clothes ? `You'll need to be completely naked.` : `Just your coin pouch, I guess.`}[/color]"
                        [/p]
                        [p]
                            You comply with the human's instructions, ${clothes ? `undressing and` : ``}
                            tossing your coin pouch out of the way.
                            You eye your equally nude masseuse up and down.
                            A bit shy, you ask Tychus what her name is.
                            "[color=purple]Just call'er by her color.
                            That's what I do, isn't it, orange?[/color]"
                        [/p]
                        [p]
                            "[color=orange]Y-yes, master![/color]" comes the kobold's almost [i]scared[/i] reply.
                            You disregard her frightened tone, figuring she must be as shy as you are.
                            "[color=purple]You two get along. I'll be back in an hour.[/color]"
                            That being said, the human leaves the room, closing the door behind him, returning to his desk.
                        [/p]
                        [p]
                            In order to somewhat break the ice between the two of you, you approach your masseuse, and kindly compliment her lovely voice.
                            She thanks you, mentioning that she hasn't served another kobold in [b]forever[/b].
                            Well, you doubt a lot of kobolds are able to afford the steep price of a massage.
                            She gives you an uneasy smile, before pointing towards the nearest massage table.
                        [/p]
                        [p]
                            "[color=orange]You can have an arm massage, a back massage, or a footrub, but only [b]one[/b] of those.[/color]"
                            You ask why you can't have two, or perhaps a mix of all three.
                            "[color=orange]You paid for one massage, not multiple.
                            Last time I handed out a freebie, the master threatened to [b]beat[/b] me.
                            He has a bit of a [i]hair trigger[/i], in more than one way![/color]"
                            You frown upon hearing that the human might be [i]abusing[/i] your kind,
                            but you suppose it'd have just been some [i]disciplinary[/i] love-taps.
                        [/p]
                        [p]
                            Instead of dwelling on whether or not the workers here are kept in good condition,
                            you decide to focus on your impending massage.
                            Arms, back or feet.
                            A tough choice, since they all kind of ache in their own separate way.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        reqController = ctrl.entrance;
        if (reqController.get() === AfternoonThreeParlorEntranceChoice.BARTER) {
            reqController = ctrl.bargain;
        }

        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
