/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorSecondDecline(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondChoice.DECLINE;
    var reqController = ctrl.secondOffer;
    var subController = ctrl.secondDecline;
    var changeController = ctrl.secondDecline;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorRun(ctrl);
    var secondNext = new AfternoonThreeParlorFight(ctrl);

    var firstChoice = ctrl.secondChoice.DECLINE;
    var secondChoice = ctrl.secondChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You do [b]not[/b] trust this [b]one bit[/b].
                            You shake your head and the human, and tell him that while you appreciate the offer,
                            you kind of need to [i]get going[/i].
                            Maybe you'll be back some other time, but for now, you have some [i]urgent business[/i] you need to attend.
                        [/p]
                        [p]
                            Your lie does not come out as fluently as you'd hoped for.
                            The human seems [b]very[/b] upset.
                            An angry glare forms on his face, his discolored teeth half-bared.
                            "[color=purple]You [b]DARE[/b] spit my generosity back in my face?![/color]"
                            He rises to his feet, tipping over his wooden desk with a loud grunt,
                            pushing the piece of furniture aside, slamming both of the open doors shut in the process.
                            A large wood chip splintering from the cracking surface only barely misses your face, slightly scraping your cheek.
                        [/p]
                        [p]
                            The human reveals that the thing he was clutching underneath the now-tipped table was a large, lengthy, steel-tipped bullwhip.
                            Like the kind a slaver would use, only less meant to [i]discipline[/i], and more geared to [b]hurt[/b], [b]maim[/b] or even [b]kill[/b].
                            "[color=purple]Get the [b]fuck[/b] in the back room [b]right now[/b], kobold.
                            Don't make me [b]hurt[/b] you.[/color]"
                            Tychus loudly cracks the whip, showing that he means business.
                            As if his expression of pure, unbridled rage hadn't already convinced you.
                        [/p]
                        [p]
                            You cautiously take a few steps backwards, adrenaline flooding your system.
                            Like you figured: he's used to getting his way,
                            and you not playing along with his [i]plan[/i] has propelled the human into a blind, wild rampage.
                            You didn't figure him for the [i]violent[/i] type, however.
                            Must be bipolar or something.
                        [/p]
                        [p]
                            While Tychus doesn't look very healthy, he's still twice your size, and his weapon looks like it could seriously hurt you.
                            Still, you get the feeling that you [i]don't[/i] want to take orders from him,
                            given that he went from trying to sell you a massage to wanting to [b]flay[/b] you in a matter of seconds.
                            Your safest bet would be to back off slowly, and hightail out of the building the way you came.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Run</button>`;

        var tooltip = global.parser.parse(`A man his size could easily end you. Run!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var success = ctrl.misc.weapon();
        var disabled = subController.has() || !success ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`If this human oaf wants a fight, a fight he shall get!`);
        if (!success) {
            tooltip = global.parser.parse(`You have no weapon.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}


var AfternoonThreeParlorSecondDeclineChoice = {
    RUN: 1,
    FIGHT: 2
};
