/**
 * Fight section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFightSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.fight;
    var changeController = ctrl.fight;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.fightFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFought = true;
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorKilled = true;
        var weapon = ctrl.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var blunt = weapon.blunt();
        var stab = weapon.stab();
        var slash = weapon.slash();
        var canThrow = weapon.throw();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var egg = weapon.id === ItemEnum.ROTTEN_EGG;
        var armorSave = ctrl.misc.totalArmor() > 6;
        var shirt = ctrl.misc.shirt();
        var shirtPopup = global.kobold.inventory.createItemWithTooltip(shirt);
        var shirtArmor = (shirt && shirt.defence()) || 0;
        var pants = ctrl.misc.pants();
        var pantsPopup = global.kobold.inventory.createItemWithTooltip(pants);
        var pantsArmor = (pants && pants.defence()) || 0;
        var pantsBest = pantsArmor > shirtArmor;
        var toReturn = `[p]
                            You never asked for this, but if it's a fight the human wants, a fight he'll get.
                            He may be bigger than you, he may be [b]stronger[/b] than you, and he may have a fancy long-reaching weapon,
                            but you are [b]far[/b] more determined to leave this place alive than the sickly human is to capture you.
                            Heck, he looks like he's halfway to hell already.
                            Time to deliver him to where he belongs.
                        [/p]
                        [p]
                            
                            The leather cord snags you from the side,
                            snapping around your body, the steel tip [b]digging[/b] into your back,
                            burying in between two ligaments of your spine.
                        [/p]`;

        if (weaponSave) {
            toReturn += `${global.ui.condSucc(`Attack + defence greater than 5`)}
                        [p]
                            You brandish your ${weaponPopup}, and drop into a combat stance.
                            The two of you circle around one another like predatory animals.
                            Then, Tychus makes the first move.
                            He swings his whip in a wide horizontal arc.
                            Luckily, you have a strong weapon built for combat.
                            You quickly move to parry the blow, and,
                            to your own surprise, you actually manage to do so,
                            striking the metal tip of the whip away!
                            Wow!
                            Is that how knights do it?
                        [/p]`;
        } else {
            toReturn += `${global.ui.condFail(`Attack + defence below 6`)}
                        [p]
                            You brandish your ${weaponPopup}, and drop into a combat stance.
                            The two of you circle around one another like predatory animals.
                            Then, Tychus makes the first move.
                            He swings his whip in a wide horizontal arc.
                            Despite your best efforts, you [b]fail[/b] to block the attack.
                        [/p]
                        [p]
                            ${global.ui.condSucc(`Armor greater than 6`)}
                        [/p]
                        [p]
                            ${pantsBest ? `The leather cord snags your leg from the side,
                            wrapping around the limb, the steel tip [b]smacking[/b] against your kneecap!
                            Luckily, your ${pantsPopup} are sturdy enough to somewhat shield you from the blow, preventing the worst of the damage.
                            ` : `The leather cord snags you from the side,
                            snapping around your body, the steel tip [b]digging[/b] into your back!
                            Luckily, your ${shirtPopup} is sturdy enough to somewhat shield you from the blow, preventing the worst of the damage.`}
                        [/p]`;
        }

        toReturn += `   [p]
                            Tychus is visibly confused.
                            He probably expected that attack to take you down.
                            He tosses his whip to the side, and prepares his hands to grab you instead.
                            "[color=purple]I'll [b]squeeze[/b] the fight out of you![/color]"
                            He clearly intents to choke or bearhug you, but as the human advances,
                            he stumbles and falls, tripping over his own feet, tumbling face-first to the ground!
                        [/p]
                        [p]
                            Unlucky bastard.
                            You capitalize on his mistake, breaching the gap between the two of you.
                            ${egg ? `You throw your ${weaponPopup}, hitting Tychus straight in the eye!
                            Both the rotten egg and the man's eyeball [b]splatter[/b] open into a foul mixture of eye-goop and reeking,
                            long-overdue eggwhite.` : blunt ? `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]bashing[/b] his skull with all of your might.` : stab ? `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]stabbing[/b] it into his skull with all of your might.` : `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]slicing[/b] his skull with all of your might, rending his flesh.`}
                            The human [b]screams[/b] in pain!
                            ${egg ? `` : `You retract your weapon, the damage done.`}
                        [/p]
                        [p]
                            With his last ounce of energy, Tychus glares up at you.
                            The hateful look on his face has given way to a fearful, frightened gaze.
                            He stammers out the final few concoctions of his pain-shocked mind.
                            "[color=purple]What...are...you?[/color]"
                            Instead of bothering to reply, you lift up your foot, and [b]stomp[/b] his head into the floor,
                            knocking the man out, hopefully [b]for good[/b].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
