/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetFlee(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorItemGetChoice.FLEE;
    var reqController = ctrl.itemGet;
    var changeController = ctrl.itemGetFlee;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorRan = true;
        var him = ctrl.misc.baseGender() === GenderEnum.MALE;
        var toReturn = `[p]
                            You need to get out of here.
                            The sooner, the better.
                            Tychus is distracted by his notes.
                            He didn't notice you entering the building, so he won't notice you leaving until you're already out the door.
                            At least that's the plan.
                        [/p]
                        [p]
                            You slowly slip into the lobby, and begin to move towards the front door as silently as you possibly can.
                            There is little in the room to hide yourself behind,
                            but the human is too engrossed in his reading to realize that his latest [i]acquisition[/i] hasn't been restrained.
                        [/p]
                        [p]
                            Halfway to the exit, a gust of wind suddenly blows shut the door through which you came.
                            Tychus's attention is grabbed, but he does not lift his head from his notes.
                            "[color=purple]Back to the break room, Orange.
                            I'll go check up on our [i]guest[/i] after I finish reading.[/color]"
                            Thank goodness, he thinks you're the masseuse!
                        [/p]
                        [p]
                            Without answering his half-minded mumbling, you continue to walk towards the front door.
                            Once you've finally reached the doorway to freedom, the human, sensing something wrong,
                            lifts his head his up, and realizes that you're [b]not[/b] his treacherous pet.
                            "[color=purple]Where the [b]hell[/b] do you think you're going?![/color]"
                            He reaches underneath his desk, and brandishes a steel-tipped whip, but it's already too late.
                            You open the door, and escape into the streets,
                            closing it behind you just in time to catch Tychus' first blow, his battle-whip practically [b]tearing[/b] the door in half!
                        [/p]
                        [p]
                            You sprint away, vanishing into the slums, bolting off in a random direction.
                            A few minutes of rapid jogging later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            That was close.
                            [b]Too[/b] close.
                            You could have ended up a slave, or worse!
                            And all that because you wanted a simple massage.
                            Then again, nothing is ever really [i]simple[/i] for a kobold like yourself.
                            Everything is complicated because of your lack of rights and everyone hating you and all.
                            It's too bad that you couldn't help the other kobolds under Tychus' tyrannical reign,
                            but out here in Varanar, it's every lizard for ${him ? `him` : `her`}self.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your conscience will be more at ease after a good night's rest.
                            And if it's not, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}
