/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetSpare(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.itemGetFight;
    var changeController = ctrl.itemGetSpare;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorSpareFree(ctrl);
    var secondNext = new AfternoonThreeParlorItemGetLoot(ctrl);

    var firstChoice = ctrl.secondFightChoice.FREE;
    var secondChoice = ctrl.secondFightChoice.LOOT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFought = true;
        var weapon = ctrl.misc.weapon();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        var toReturn = `[p]
                            Tychus' reign of terror needs to end.
                            And what more noble thing is there than to slay the wicked?
                            Surely, this is what a knight would do, in your situation!
                            The human is distracted by his notes.
                            You can sneak up behind him, and then end his miserable existence!
                            At least that's the plan.
                            It's not the most chivalrous way to fight evil,
                            but you guess a little [i]improvisation[/i] is okay when you're half the size and a fourth of the bulk of a regular knight.
                        [/p]
                        [p]
                            You slowly slip into the lobby, and begin to move towards Tychus' desk as silently as you possibly can.
                            There is little in the room to hide yourself behind, so you lower your frame as much as you possibly can,
                            attempting to use the desk itself as cover.
                            The human is too engrossed in his reading to realize that his latest [i]acquisition[/i] hasn't been restrained.
                            And with some luck, he won't understand what's going on until it's too late.
                        [/p]
                        [p]
                            You creep all the way up to the desk, where you quietly take a deep breath, your body concealed by the large piece of wooden furniture.
                            Your nerves steeled, you maneuver around to the rear of the seated, sickly bald man.
                            He still hasn't noticed you.
                            ${weapon ? `You ready your ${weaponPopup}.` : `Lacking a weapon, you grab a hefty jar from a nearby rack.`}
                            It's time to end this.
                        [/p]
                        `;

        if (egg) {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            smacking your ${weaponPopup} against the back of his head!
                            The egg shatters, and its disgusting, goopy insides ooze down the human's head.
                            Your strike ineffective, you quickly follow up by grabbing a jar from a nearby rack,
                            and [b]breaking[/b] it atop his very [b]skull[/b], which splits open with a nasty [b]crack[/b],
                            audible even over the noise of shattering ceramic!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        } else if (weapon) {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            brutally [b]bashing[/b] the back of Tychus' head,
                            your weapon [b]burying[/b] into his very [b]skull[/b], which splits open with a nasty [b]crack[/b]!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        } else {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            brutally [b]bashing[/b] the back of Tychus' head with the jar,
                            your temporary weapon [b]breaking[/b] atop his very [b]skull[/b], which splits open with a nasty [b]crack[/b],
                            audible even over the noise of shattering ceramic!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        }

        toReturn += `   [p]
                            A puddle of blood oozes out from Tychus' head.
                            You are breathing heavily.
                            The deed is done.
                            He's only knocked unconscious, but that's good enough for you.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Free</button>`;

        var tooltip = global.parser.parse(`Release your fellow kobolds from oppression.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Loot</button>`;

        var tooltip = global.parser.parse(`Steal whatever you can before Tychus wakes up!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}
