/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageFoe(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.foe;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var feet = ctrl.massage.get() === AfternoonThreeParlorMassageChoice.FEET;
        var knowsSeki = global.kobold.history.shared.seki.knowsDying;
        var toReturn = `[p]
                            As you continue to bask in the afterglow of the massage,
                            the masseuse ${feet ? `climbs up on the table, and` : ``} places her hand on the back of your neck.
                            A little extra aftercare?
                            You're not complaining.
                        [/p]
                        ${global.ui.condFail(`Charisma below 7`)}
                        ${global.ui.condSucc(`Intelligence greater than 6`)}
                        [p]
                            She presses two of her fingers against your scales, and begins to apply pressure.
                            Wait a minute.
                            You know this grip.
                            The Serpent King's Sedative,
                            named after its usage in the famous assassination of the previous dragon emperor.
                            Seki${knowsSeki ? `, your old alchemist fence and friend,` : ``} told you about it!
                            He showed you once, even.
                        [/p]
                        [p]
                            Your masseuse is trying to knock you out!
                            You can already feel your limbs growing heavy.
                            But it's not too late!
                            Using your hands, you crudely [b]shove[/b] Orange off the table.
                            She clearly was not expecting any resistance.
                            Taken aback, you easily, manage to force her off.
                        [/p]
                        [p]
                            Instead of taking you by surprise and knocking you out, your masseuse is the one who ends up passing out.
                            She falls backwards off the table with a surprised yelp, landing square on her head with a loud [b]crack[/b].
                            While the table isn't particularly high, her landing was [b]not[/b] a gentle one.
                            The kobold is knocked out cold.
                            Blood oozes from her cracked head.
                            She's still breathing, but she's going to have one [b]hell[/b] of a headache when she wakes up.
                            [i]If[/i] she wakes up.
                        [/p]
                        [p]
                            Good riddance!
                            She tried to disable you.
                            But [b]why[/b]?
                            Something clicks inside of your head.
                            All of the pieces finally fit together.
                            But of course!
                            How could you have missed this?
                            You [b]knew[/b] there was something wrong about this place!
                        [/p]
                        [p]
                            Those kobolds you saw earlier.
                            The red one, the green one and the blue one.
                            They must be [b]slaves[/b].
                            [b]Forced[/b] to work for Tychus.
                            Orange was trying to knock you out so you could be added to the [i]collection[/i].
                        [/p]
                        [p]
                            How [b]terrible[/b]!
                            You have to get out of here.
                            If you can save the others in the process, all the better.
                            But your survival is paramount.
                            If just to tell the tale of what happened here.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
