/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageFriend(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.friend;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var feet = ctrl.massage.get() === AfternoonThreeParlorMassageChoice.FEET;
        var toReturn = `[p]
                            As you continue to bask in the afterglow of the massage,
                            the masseuse ${feet ? `climbs up on the table, and` : ``} places her hand on the back of your neck.
                            A little extra aftercare?
                            You're not complaining.
                        [/p]
                        ${global.ui.condSucc(`Charisma greater than 6`)}
                        [p]
                            She presses two of her fingers against your scales, but then abruptly pulls back.
                            Thinking something's the matter, you cast a glance back at her.
                            She looks conflicted, and kind of sad, like she's struggling to make a tough decision.
                            As you kindly ask what the matter is, she bursts out into tears.
                        [/p]
                        [p]
                            "[color=orange]I can't do it! I [b]can't[/b]![/color]"
                            Rolling onto your back, and coming to a seat on the table, you tell the masseuse to calm down.
                            What can't she do?
                            Did you do something wrong?
                            "[color=orange]You don't understand!
                            The others...I was the first.
                            He bought me off a slaver.
                            Trained me to do [b]horrible[/b] things.
                            I-I--[/color]"
                            She tears up once more.
                        [/p]
                        [p]
                            You place a hand on Orange's shoulder, and tell her it's okay.
                            She lets out a snicker, and then takes a deep breath.
                            "[color=orange]The other three, they were just like you.
                            They came in here for a massage.
                            Then I knocked them out, like Tychus told me to.
                            But I can't do it.
                            Not anymore.[/color]"
                        [/p]
                        [p]
                            That [b]bastard[/b].
                            You never really liked him.
                            "[color=orange]You have to go.
                            Take your things and leave.
                            Don't worry about me.
                            I died a long time ago, at least on the inside.[/color]"
                        [/p]
                        [p]
                            Before you manage to comfort the kobold further, she hops off the table, and bolts towards the back room.
                            Opening the door, Orange utters a few final words.
                            "[color=orange]Good luck. And thank you.[/color]"
                            Then, she steps inside, closing the door behind her, audibly locking it from the other side.
                            And that is the last you ever see of her.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}
