/**
 * Controller for the Varanar, Noon Three scene.
 * @param {string} sceneId The id of the scene.
 */
function NoonThreeStoreController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Controller for the store.
     * @param {string} key The key to use for the store.
     */
    function BuyController(key) {
        var that = this;
        ContinueController.call(this, id, key);

        var shopItemKey = key + "Items";
        var shopGoldKey = key + "Gold";
        var initialShopItems = [
            new ItemAmount(1, new HalfScythe()),
            new ItemAmount(1, new Mace()),
            new ItemAmount(1, new ShortSword()),
            new ItemAmount(1, new GreatSword()),
            new ItemAmount(1, new LinenShirt()),
            new ItemAmount(1, new LinenPants()),
            new ItemAmount(1, new HatScarf()),
            new ItemAmount(1, new KoboldSocks()),
            new ItemAmount(1, new WoodenSandals()),
            new ItemAmount(1, new Rope()),
            new ItemAmount(1, new SmallMirror()),
            new ItemAmount(2, new Lockpick()),
            new ItemAmount(2, new BeefUp()),
            new ItemAmount(2, new LiquidLuck()),
            new ItemAmount(2, new MassRedu()),
            new ItemAmount(2, new EnduGain()),
            new ItemAmount(2, new CouragePotion()),
            new ItemAmount(1, new IntelligencePotion()),
            new ItemAmount(1, new FemPlus()),
            new ItemAmount(1, new mPlus()),
            new ItemAmount(1, new mTo()),
            new ItemAmount(1, new FemTo()),
            new ItemAmount(2, new Plum()),
            new ItemAmount(3, new Carrot()),
            new ItemAmount(1, new Fish()),
            new ItemAmount(1, new Apple())
        ];

        var sellModKey = key + "SellMod";
        var initialSellMod = 0.4;
        var buyModKey = key + "BuyMod";
        var initialBuyMod = 1.2;

        /**
         * Gets the current sell mod from history.
         * @returns {number} The sell mod.
         */
        function getSellMod() {
            global.kobold.history.setIfNotSet(id, sellModKey, initialSellMod);
            return global.kobold.history.getKey(id, sellModKey);
        }

        /**
         * Alters the sell mod.
         * @param {number} amount The amount to increase or decrease the sellmod with.
         */
        this.alterSellMod = function (amount) {
            var currentSellMod = getSellMod();
            currentSellMod += amount;
            global.kobold.history.setKey(id, sellModKey, currentSellMod);
        };

        /**
         * Gets the current buy mod from history.
         * @returns {number} The buy mod.
         */
        function getBuyMod() {
            global.kobold.history.setIfNotSet(id, buyModKey, initialBuyMod);
            return global.kobold.history.getKey(id, buyModKey);
        }

        /**
         * Alters the buy mod.
         * @param {number} amount The amount to increase or decrease the buymod with.
         */
        this.alterBuyMod = function (amount) {
            var currentBuyMod = getBuyMod();
            currentBuyMod += amount;
            global.kobold.history.setKey(id, buyModKey, currentBuyMod);
        };

        function sellListener(item) {
            var itemAmounts = that.shopItems();
            var added = false;
            for (var i = 0; i < itemAmounts.length && !added; i++) {
                if (itemAmounts[i].item.id === item.id) {
                    itemAmounts[i].amount++;
                    added = true;
                }
            }
            if (!added) {
                itemAmounts.push(new ItemAmount(1, item));
            }
            that.updateCoins();
            that.updateShopItems(itemAmounts);
        }

        function buyListener(location) {
            var oldCoins = that.coins();
            that.updateCoins();
            var newCoins = that.coins();
            var itemAmounts = that.shopItems();
            itemAmounts[location].amount--;
            that.updateShopItems(itemAmounts);
            global.kobold.history.shared.varanar.dayThree.noon.moneySpent += oldCoins - newCoins;
        }

        this.getGenerator = function () {
            return new MerchantInventoryGenerator("RolandeStore", that.shopItems(), that.coins(), getSellMod(), getBuyMod(), that.continued(), sellListener, buyListener);
        };
        this.coins = function () {
            global.kobold.history.setIfNotSet(id, shopGoldKey, global.kobold.inventory.coins);
            return global.kobold.history.getKey(id, shopGoldKey);
        };
        this.updateCoins = function () {
            global.kobold.history.setKey(id, shopGoldKey, global.kobold.inventory.coins);
        };
        this.shopItems = function () {
            global.kobold.history.setIfNotSet(id, shopItemKey, initialShopItems);
            var jsonItems = global.kobold.history.getKey(id, shopItemKey);
            var items = global.kobold.inventory.constructItemAmountListFromMemory(jsonItems);
            return items;
        };
        this.updateShopItems = function (newItems) {
            global.kobold.history.setKey(id, shopItemKey, newItems);
        };
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = true;
    this.nextSceneName = "AfternoonThree";
    this.nextFolderName = "Varanar/DayThree";

    this.misc = new MiscController(id, "misc");
    this.postShopMisc = new MiscController(id, "postShopMisc");

    this.arrival = new ContinueController(id, "arrival");

    this.proposal = new ChangeController(id, "proposal", function () {
        global.kobold.status.mental.decrease(5, "Indecent proposal");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Indecent proposal", "Sanity", -5)
        ]);
    });

    this.decline = new ChangeController(id, "decline", function () {
        global.kobold.status.mental.increase(8, "Moral highground");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Moral highground", "Sanity", +8)
        ]);
    });

    this.handjobAttitude = new ChoiceController(id, "handjobAttitude");
    this.blowjobMoney = new ChoiceController(id, "blowjobMoney");
    this.blowjobAttitude = new ChoiceController(id, "blowjobAttitude");

    this.blowjobDecline = new ChangeController(id, "blowjobDecline", function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        global.kobold.status.mental.decrease(5, "Prostitution");
        if (hjEager) {
            global.kobold.status.mental.increase(2, "Enjoyable handjob");
        } else {
            global.kobold.status.mental.decrease(2, "Frustrating handjob");
        }
        global.kobold.status.mental.decrease(5, "Degrading facial");
        global.kobold.status.stamina.decrease(4, "Handjob");
        global.kobold.status.exertion.increase(5, "Handjob");
        global.kobold.status.exertion.increase(4, "Forced facial");
    }, function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var changes = [
            new Change("Prostitution", "Sanity", -5)
        ];
        if (hjEager) {
            changes.push(new Change("Enjoyable handjob", "Sanity", +2));
        } else {
            changes.push(new Change("Frustrating handjob", "Sanity", -2));
        }
        changes.push(new Change("Degrading facial", "Sanity", -5));
        changes.push(new Change("Handjob", "Stamina", -4));
        changes.push(new Change("Handjob", "Sexual exhaustion", +5, true));
        changes.push(new Change("Forced facial", "Sexual exhaustion", +4, true));
        return global.ui.change.buildNotification(changes);
    });

    this.blowjobEnd = new ChangeController(id, "blowjobEnd", function () {
        var paid = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY ? 50 : 30;
        var eager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        if (eager) {
            that.store.alterBuyMod(-0.1);
            that.store.alterSellMod(+0.1);
        } else {
            that.store.alterBuyMod(-0.05);
            that.store.alterSellMod(+0.05);
        }
        global.kobold.inventory.coins += paid;
        global.kobold.history.shared.varanar.dayThree.noon.moneyReceived = paid;
        global.kobold.status.mental.increase(10, "Prostitution payment");
    }, function () {
        var paid = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY ? 50 : 30;
        return global.ui.change.buildNotification([
            new Change("Payday", "Sanity", +10),
            new Change("Paid", "Coins", +paid)
        ]);
    });

    this.sexMoney = new ChoiceController(id, "sexMoney");
    this.sexAttitude = new ChoiceController(id, "sexAttitude");

    this.sexDecline = new ChangeController(id, "sexDecline", function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        global.kobold.status.physical.decrease(4, "Suffocation");
        global.kobold.status.mental.decrease(5, "Prostitution");
        if (hjEager) {
            global.kobold.status.mental.increase(2, "Enjoyable handjob");
        } else {
            global.kobold.status.mental.decrease(2, "Frustrating handjob");
        }
        if (bjEager) {
            global.kobold.status.mental.increase(2, "Arousing blowjob");
        } else {
            global.kobold.status.mental.decrease(2, "Reluctant blowjob");
        }
        global.kobold.status.mental.decrease(5, "Suffocating deepthroating");
        global.kobold.status.stamina.decrease(4, "Handjob");
        global.kobold.status.stamina.decrease(6, "Blowjob");
        global.kobold.status.stamina.decrease(4, "Suffocation");
        global.kobold.status.exertion.increase(5, "Handjob");
        global.kobold.status.exertion.increase(10, "Blowjob");
        global.kobold.status.exertion.increase(5, "Suffocating deepthroating");
    }, function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var changes = [
            new Change("Suffocation", "Health", -4),
            new Change("Prostitution", "Sanity", -5)
        ];
        if (hjEager) {
            changes.push(new Change("Enjoyable handjob", "Sanity", +2));
        } else {
            changes.push(new Change("Frustrating handjob", "Sanity", -2));
        }
        if (bjEager) {
            changes.push(new Change("Arousing blowjob", "Sanity", +2));
        } else {
            changes.push(new Change("Reluctant blowjob", "Sanity", -2));
        }
        changes.push(new Change("Suffocating deepthroating", "Sanity", -5));
        changes.push(new Change("Handjob", "Stamina", -4));
        changes.push(new Change("Blowjob", "Stamina", -6));
        changes.push(new Change("Suffocation", "Stamina", -4));
        changes.push(new Change("Handjob", "Sexual exhaustion", +5, true));
        changes.push(new Change("Blowjob", "Sexual exhaustion", +10, true));
        changes.push(new Change("Suffocating deepthroating", "Sexual exhaustion", +5, true));
        return global.ui.change.buildNotification(changes);
    });

    this.sexEnd = new ChangeController(id, "sexEnd", function () {
        var paid = 30;
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var hjMoney = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjMoney = that.blowjobMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        if (hjMoney) {
            paid += 20;
        }
        if (bjMoney) {
            paid += 20;
        }
        if (hjEager && bjEager) {
            that.store.alterBuyMod(-0.15);
            that.store.alterSellMod(+0.15);
        } else if (hjEager || bjEager) {
            that.store.alterBuyMod(-0.1);
            that.store.alterSellMod(+0.1);
        } else {
            that.store.alterBuyMod(-0.05);
            that.store.alterSellMod(+0.05);
        }
        global.kobold.inventory.coins += paid;
        global.kobold.history.shared.varanar.dayThree.noon.moneyReceived = paid;
        global.kobold.status.mental.increase(10, "Prostitution payment");
    }, function () {
        var paid = 30;
        var hjMoney = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var bjMoney = that.blowjobMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        if (hjMoney) {
            paid += 20;
        }
        if (bjMoney) {
            paid += 20;
        }
        return global.ui.change.buildNotification([
            new Change("Payday", "Sanity", +10),
            new Change("Paid", "Coins", +paid)
        ]);
    });

    this.sexReluctant = new ChangeController(id, "sexReluctant", function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        global.kobold.status.physical.decrease(6, "Stretched and scorched");
        global.kobold.status.mental.decrease(5, "Prostitution");
        if (hjEager) {
            global.kobold.status.mental.increase(2, "Enjoyable handjob");
        } else {
            global.kobold.status.mental.decrease(2, "Frustrating handjob");
        }
        if (bjEager) {
            global.kobold.status.mental.increase(2, "Arousing blowjob");
        } else {
            global.kobold.status.mental.decrease(2, "Reluctant blowjob");
        }
        global.kobold.status.mental.decrease(10, "Unwilling sex");
        global.kobold.status.stamina.decrease(4, "Handjob");
        global.kobold.status.stamina.decrease(6, "Blowjob");
        global.kobold.status.stamina.decrease(5, "Screaming for help");
        global.kobold.status.exertion.increase(5, "Handjob");
        global.kobold.status.exertion.increase(10, "Blowjob");
        global.kobold.status.exertion.increase(20, "Unwilling sex");
    }, function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var changes = [
            new Change("Stretched", "Health", -3),
            new Change("Scorched", "Health", -3),
            new Change("Prostitution", "Sanity", -5)
        ];
        if (hjEager) {
            changes.push(new Change("Enjoyable handjob", "Sanity", +2));
        } else {
            changes.push(new Change("Frustrating handjob", "Sanity", -2));
        }
        if (bjEager) {
            changes.push(new Change("Arousing blowjob", "Sanity", +2));
        } else {
            changes.push(new Change("Reluctant blowjob", "Sanity", -2));
        }
        changes.push(new Change("Unwilling sex", "Sanity", -10));
        changes.push(new Change("Handjob", "Stamina", -4));
        changes.push(new Change("Blowjob", "Stamina", -6));
        changes.push(new Change("Screaming for help", "Stamina", -5));
        changes.push(new Change("Handjob", "Sexual exhaustion", +5, true));
        changes.push(new Change("Blowjob", "Sexual exhaustion", +10, true));
        changes.push(new Change("Unwilling sex", "Sexual exhaustion", +20, true));
        return global.ui.change.buildNotification(changes);
    });

    this.sexEager = new ChangeController(id, "sexEager", function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        global.kobold.status.physical.decrease(6, "Stretched and scorched");
        global.kobold.status.mental.decrease(5, "Prostitution");
        if (hjEager) {
            global.kobold.status.mental.increase(2, "Enjoyable handjob");
        } else {
            global.kobold.status.mental.decrease(2, "Frustrating handjob");
        }
        if (bjEager) {
            global.kobold.status.mental.increase(2, "Arousing blowjob");
        } else {
            global.kobold.status.mental.decrease(2, "Reluctant blowjob");
        }
        global.kobold.status.mental.increase(15, "Heavenly fucking");
        global.kobold.status.stamina.decrease(4, "Handjob");
        global.kobold.status.stamina.decrease(6, "Blowjob");
        global.kobold.status.stamina.decrease(3, "Riding cock");
        global.kobold.status.stamina.decrease(2, "Screaming for more");
        global.kobold.status.exertion.increase(5, "Handjob");
        global.kobold.status.exertion.increase(10, "Blowjob");
        global.kobold.status.exertion.increase(20, "Incredible sex");
    }, function () {
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var changes = [
            new Change("Stretched", "Health", -3),
            new Change("Scorched", "Health", -3),
            new Change("Prostitution", "Sanity", -5)
        ];
        if (hjEager) {
            changes.push(new Change("Enjoyable handjob", "Sanity", +2));
        } else {
            changes.push(new Change("Frustrating handjob", "Sanity", -2));
        }
        if (bjEager) {
            changes.push(new Change("Arousing blowjob", "Sanity", +2));
        } else {
            changes.push(new Change("Reluctant blowjob", "Sanity", -2));
        }
        changes.push(new Change("Heavenly fucking", "Sanity", +15));
        changes.push(new Change("Handjob", "Stamina", -4));
        changes.push(new Change("Blowjob", "Stamina", -6));
        changes.push(new Change("Riding cock", "Stamina", -3));
        changes.push(new Change("Screaming for more", "Stamina", -2));
        changes.push(new Change("Handjob", "Sexual exhaustion", +5, true));
        changes.push(new Change("Blowjob", "Sexual exhaustion", +10, true));
        changes.push(new Change("Incredible sex", "Sexual exhaustion", +20, true));
        return global.ui.change.buildNotification(changes);
    });

    this.sexFinale = new ChangeController(id, "sexFinale", function () {
        var paid = 30;
        var hjEager = that.handjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var hjMoney = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var bjEager = that.blowjobAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var bjMoney = that.blowjobMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var sexEager = that.sexAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        var sexMoney = that.sexMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        if (hjMoney) {
            paid += 20;
        }
        if (bjMoney) {
            paid += 20;
        }
        if (sexMoney) {
            paid += 30;
        }
        if (hjEager) {
            that.store.alterBuyMod(-0.05);
            that.store.alterSellMod(+0.05);
        }
        if (bjEager) {
            that.store.alterBuyMod(-0.05);
            that.store.alterSellMod(+0.05);
        }
        if (sexEager) {
            that.store.alterBuyMod(-0.05);
            that.store.alterSellMod(+0.05);
        }
        that.store.alterBuyMod(-0.05);
        that.store.alterSellMod(+0.05);
        global.kobold.inventory.coins += paid;
        global.kobold.history.shared.varanar.dayThree.noon.moneyReceived = paid;
        global.kobold.status.physical.increase(2, "Soothing spunk");
        if (sexEager) {
            global.kobold.status.mental.decrease(4, "Didn't get to cum");
        }
        global.kobold.status.mental.increase(10, "Prostitution payment");
    }, function () {
        var paid = 30;
        var hjMoney = that.proposal.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var bjMoney = that.blowjobMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var sexMoney = that.sexMoney.get() === that.NoonThreeStoreMoneyChoice.MOREMONEY;
        var sexEager = that.sexAttitude.get() === that.NoonThreeStoreAttitudeChoice.EAGER;
        if (hjMoney) {
            paid += 20;
        }
        if (bjMoney) {
            paid += 20;
        }
        if (sexMoney) {
            paid += 30;
        }
        if (sexEager) {
            return global.ui.change.buildNotification([
                new Change("Soothing spunk", "Health", +2),
                new Change("Didn't get to cum", "Sanity", -4),
                new Change("Payday", "Sanity", +10),
                new Change("Paid", "Coins", +paid)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Soothing spunk", "Health", +2),
                new Change("Payday", "Sanity", +10),
                new Change("Paid", "Coins", +paid)
            ]);
        }
    });

    this.return = new ChangeController(id, "return", function () {
        global.kobold.inventory.coins += 50;
        var weapon = that.misc.weapon();
        if (weapon) {
            global.kobold.inventory.addItem(weapon);
        } else {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
            global.kobold.status.mental.decrease(5, "Half refunded weapon");
        }
    }, function () {
        var weapon = that.misc.weapon();
        var greatsword = that.postShopMisc.weapon();
        if (weapon) {
            return global.ui.change.buildNotification([
                new Change("Refund", "Coins", +50),
                new Change("Returned weapon", global.kobold.inventory.createItemWithTooltip(greatsword, true), -1),
                new Change("Old weapon", global.kobold.inventory.createItemWithTooltip(weapon, true), +1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Peeved", "Sanity", -5),
                new Change("Refund", "Coins", +50),
                new Change("Returned weapon", global.kobold.inventory.createItemWithTooltip(greatsword, true), -1)
            ]);
        }

    });

    this.preStore = new ContinueController(id, "preStore");
    this.store = new BuyController("store");
    this.preContinue = new ContinueController(id, "preContinue");


    this.NoonThreeStoreMoneyChoice = {
        DECLINE: 1,
        ACCEPT: 2,
        MOREMONEY: 3
    };

    this.NoonThreeStoreAttitudeChoice = {
        RELUCTANT: 1,
        EAGER: 2
    };


    this.fragments = {};
    this.fragments.store = new NoonThreeStoreShop(this);
    this.fragments.sexFinale = new NoonThreeStoreSexFinale(this);
    this.fragments.sexChoices = new NoonThreeStoreSexChoices(this);
    this.fragments.blowjobEndChoices = new NoonThreeStoreBlowjobEndChoices(this);
    this.fragments.blowjobChoices = new NoonThreeStoreBlowjobChoices(this);
    this.fragments.handjobEndChoices = new NoonThreeStoreHandjobEndChoices(this);
    this.fragments.handjobChoices = new NoonThreeStoreHandjobChoices(this);

    this.bgm = new AudioController(id, "res/audio/varanar/noonthreestore.mp3", 100);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

}
