/**
 * End section of Varanar, Noon Three.
 * @param {VaranarSoupKitchenController} controller The Varanar, Noon Three controller.
 */
function VaranarSoupKitchenHalfApple(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;
    var changeController = ctrl.halfApple;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Denied your meal, and kicked back out onto the cold streets, you decide to go back to your [i]original[/i] plan.
                            You scrounge the town square, crawling through the dirt looking for anything left to eat.
                            Almost [b]everything[/b] has been picked clean already, probably by [i]other kobolds[/i].
                            Right as you were about to give up hope completely, you find a single half-eaten apple concealed underneath a dropped handkerchief.
                        [/p]
                        [p]
                            You pick up the day-old, half-eaten fruit.
                            No worms, no bird-pecking marks, a bit of grime, but you wipe most of it off using the handkerchief.
                            Better than nothing, and you've had [b]far[/b] worse to eat in the past.
                            You feel like you [b]need[/b] to eat something, or you'll pass out.
                            Here goes nothing.
                        [/p]
                        [p]
                            You take a first bite into the apple.
                            It's still quite [i]hard[/i], having retained much of its its firmness overnight due to the sub-zero temperatures out.
                            While the taste of [i]dirt[/i] is [b]very[/b] prevalent, underneath lies a satisfying, rather sweet treat.
                            Savoring the delightful flavor, you gobble down the rest of your unsanitary snack.
                            While not much, your midday meal gives you the energy boost you needed to keep on truckin'.
                        [/p]
                        [p]
                            Your hunger [i]somewhat[/i] sated, you decide to try your luck elsewhere.
                            Perhaps the grand avenue will hold some interesting sights to see.
                            Everything was already closed last night, but you're certain there'll be at least one store worth browsing.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}
