/**
 * End section of Varanar, Noon Three.
 * @param {VaranarSoupKitchenController} controller The Varanar, Noon Three controller.
 */
function VaranarSoupKitchenSteal(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = VaranarSoupKitchenServedChoice.STEAL;
    var reqController = ctrl.serve;
    var changeController = ctrl.steal;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.noon.stole = true;
        var toReturn = `[p]
                            While you appreciate the sheep's hospitality,
                            he made a [b]very[/b] big made mistake.
                            He left what you presume to be one of his most valuable possessions in the same room as a kobold, without any supervision whatsoever.
                            What did he [b]think[/b] was going to happen?
                            That you were going to sit here and eat this old broth while being [b]tempted[/b] by the promise of riches and gold?
                            What an [b]idiot[/b].
                        [/p]
                        [p]
                            The caprine doesn't have any excuse.
                            He's been working with your kind for [b]years[/b].
                            He should have known what would happen if he left his wedding ring alone with you.
                            Heck, you're surprised the fat man has managed to hold onto it for this long.
                            It being stolen at one point or another was inevitable.
                            You're just helping the natural order of things along.
                        [/p]
                        [p]
                            Your decision rationalized, you reach out for the small bag.
                            Opening it up, you discover a shiny ${global.kobold.inventory.createItemWithTooltip(new GoldRing())} within.
                            It looks brand new, despite obviously being rather old.
                            The sheep must have kept it in good condition ever since his wife died.
                            This must be the only thing he has to remember her by, especially now that the soup kitchen has closed down.
                            Almost kind of sad, but you really [b]need[/b] the money.
                            Rather the caprine upset, than you [b]dead[/b].
                            Your loot in hand, you get up off your chair,
                            sneak towards the door you came in through, and quietly exit the building.
                        [/p]
                        [p]
                            Back onto the cold streets, you immediately [b]bolt[/b] towards the grand avenue,
                            to hide within the masses of people on the streets, should the fat man come looking for you after he figures out what you took.
                            Adrenaline somewhat suppresses your hunger, making you forget all about your rumbling stomach.
                            With a golden ring as your haul, you can buy [b]proper[/b] food later.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}
