

// SCENE RiversideMarsh

// scenes\Deepwoods\RiversideMarsh\upstream\RiversideMarshUpstream.js

/**
 * Information about the class.
 */
function RiversideMarshUpstream() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END scenes\Deepwoods\RiversideMarsh\upstream\RiversideMarshUpstream.js

// scenes\Deepwoods\RiversideMarsh\downstream\RiversideMarshDownstream.js

/**
 * Downstream section of the Riverside Marsh.
 * @param {RiversideMarshController} controller The Riverside Marsh controller.
 */
function RiversideMarshDownstream(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = RiversideMarshArrivalChoice.DOWNSTREAM;
    var reqController = ctrl.arrival;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You're saved!
                            You thought you were never going to get out of this damp [b]hellhole[/b].
                            With renewed determination, you walk alongside the river's edge back towards the habitable part of the forest.
                        [/p]
                        [p]
                            Navigating the riverbank is a lot easier than making your way through the deepwoods.
                            There may be a bunch of treacherous rocks in your way, but without the forest roof to block it,
                            the moon's radiating light illuminates your path rather nicely.
                            You can actually [i]see[/i] where you're going, which allows you to make a lot more progress than your earlier clumsy stumbling.
                        [/p]
                        [p]
                            In less than an hour, you find yourself back in the civilized side of the woods.
                            Recognizing your surroundings, you split off from the riverside, and head straight towards your home.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart("Home", "");
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.downstream.change();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\Deepwoods\RiversideMarsh\downstream\RiversideMarshDownstream.js

// scenes\Deepwoods\RiversideMarsh\arrival\RiversideMarshArrival.js

/**
 * Arrival section of the Riverside Marsh.
 * @param {RiversideMarshController} controller The Riverside Marsh controller.
 */
function RiversideMarshArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.arrival;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new RiversideMarshDownstream(ctrl);
    var secondNext = new RiversideMarshUpstream(ctrl);

    var firstChoice = RiversideMarshArrivalChoice.DOWNSTREAM;
    var secondChoice = RiversideMarshArrivalChoice.UPSTREAM;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Traversing the deepwoods in near complete pitch-blackness isn't an easy task.
                            Luckily, your superb nightvision helps you more than a little, amplifying what little light manages to shine through the dense canopy.
                            Despite not being completely [i]blind[/i], you are in constant danger of tripping and falling.
                            Your feet are repeatedly getting caught on roots of varying shapes and sizes,
                            and when you finally find a place to plop down one of your scaly paws,
                            you still have barely visible low-hanging branches to worry about.
                            Flexible as you are, however, you succeed in making some form of progress.
                        [/p]
                        [p]
                            After nearly half an hour of slow and cautious trudging through the darkness, the ground beneath your feet begins to soften.
                            With every step, the soil [i]dampens[/i] further, and before long,
                            the previously solid dirt you were walking on is replaced by thick, sticky and slippery [b]mud[/b].
                            Coming to a halt for a moment, you become aware of a faint, continuous splashing in the distance.
                            Could you be nearing the river?
                        [/p]
                        [p]
                            Emboldened by the premise of a landmark you recognize, you press onwards.
                            After just a couple of minutes more of struggling to pass the terrain, the tightly packed trees give way to a clearing.
                            You were right!
                            It's the river!
                            You find yourself on top of the riverbank, gazing out onto the slowly streaming waterway, sparkling stars reflected by the water's surface.
                        [/p]
                        [p]
                            While you still have no idea where exactly you are, you know that Varanar is downstream from your home.
                            That, in turn, must mean that you are currently somewhere upstream.
                            Closer to the river's source than you've ever been.
                            If you follow the waterflow, it should lead you straight back to parts of the forest that you know.
                            Or you could head the opposite way, further away from civilization.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Downstream</button>`;

        var tooltip = global.parser.parse(`You need to get back home as soon as you can.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        disabled = "disabled";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Upstream</button>`;

        var tooltip = global.parser.parse(`Your adventure has barely even begun. Why turn back now? <span class="todo">Unavailable.</span>`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You go:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("forest_night");
        if (!subController.has()) {
            ctrl.riverAmbience.playSettingsDependent();
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.arrival.change();
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        //part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        //secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        //registerSecondListener();
    };
}

var RiversideMarshArrivalChoice = {
    DOWNSTREAM: 1,
    UPSTREAM: 2
};

// END scenes\Deepwoods\RiversideMarsh\arrival\RiversideMarshArrival.js

// scenes\Deepwoods\RiversideMarsh\RiversideMarshController.js

/**
 * Controller for the Caravan Confinement scene.
 * @param {string} sceneId The id of the scene.
 */
function RiversideMarshController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = false;
    this.nextSceneName = "HamletAuction";
    this.nextFolderName = "Enslavement";

    this.misc = new MiscController(id, "misc");

    this.arrival = new ChangeController(id, "arrival", function () {
        global.kobold.status.mental.increase(10, "Finding a landmark");
        global.kobold.status.stamina.decrease(5, "Rough terrain traversal");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Landmark", "Sanity", +5),
            new Change("A way out?", "Sanity", +5),
            new Change("Traversing rough terrain", "Stamina", -5)
        ]);
    });

    this.downstream = new ChangeController(id, "downstream", function () {
        global.kobold.status.mental.increase(5, "Going home");
        global.kobold.status.stamina.decrease(5, "Walking on tired feet");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Going home", "Sanity", +5),
            new Change("More hiking", "Stamina", -5)
        ]);
    });

    this.preContinue = new ContinueController(id, "preContinue");

    this.riverAmbience = new AudioController(id, "res/audio/effects/river.mp3", 100);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

}

// END scenes\Deepwoods\RiversideMarsh\RiversideMarshController.js

// scenes\Deepwoods\RiversideMarsh\RiversideMarsh.js

/**
 * First night in the deepwoods.
 */
function RiversideMarsh() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var sceneId = "RiversideMarsh";
    var sceneName = "Deepwoods, Riverside";
    var sceneSubfolder = "Deepwoods";
    var ctrl = new RiversideMarshController(sceneId);
    var arrival = new RiversideMarshArrival(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets all the required global variables such as scene id, scene name,
     * scene stack and theme.
     */
    function setGlobals() {
        global.currentScene = that.id;
        global.currentSceneName = that.name;
        global.currentSceneFolder = that.subfolder ? that.subfolder : "";
        global.kobold.history.addToSceneStack(that.id);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = global.ui.buildGenericHeader();
        tab += global.parser.parse(`[t]Deepwoods[/t]`);
        tab += global.parser.parse(`[st]Riverside[/st]`);
        tab += arrival.build();
        tab += global.ui.buildGenericFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = sceneId;
    this.name = sceneName;
    this.subfolder = sceneSubfolder;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this scene.
     */
    this.load = function () {
        setGlobals();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        arrival.registerListeners();
        ctrl.autosave();
    };
}

global.sceneLoader.queueRegister(function () {
    var scene = new RiversideMarsh();
    global.sceneLoader.registerScene(scene.id, scene);
});

// END scenes\Deepwoods\RiversideMarsh\RiversideMarsh.js



// END SCENE RiversideMarsh

