

// SCENE AfternoonThree

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailFlee.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFailFlee(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorFailChoicesChoice.FLEE;
    var reqController = ctrl.fail;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You may be beaten, but you are not yet broken.
                            Thinking you spy an opportunity for escape, you run into the lobby as fast as your malnourished legs can carry you.
                            Your mad dash for freedom is cut short less than a dozen steps in,
                            your pace not nearly as quick as you'd hoped for,
                            your body far too battered to burst into a sprint.
                        [/p]
                        [p]
                            Tychus [b]grabs[/b] you by the back of your neck, painfully lifting you off the ground.
                            "[color=purple]You want to go outside? Then let's go.[/color]"
                            The human carries you out of the building, using your body to push open the wooden door,
                            a few splinters agonizingly burying into your lash-marks.
                        [/p]
                        [p]
                            Outside, you can barely see.
                            The rain hits your scales.
                            It reminds you of how things used to be, back home in the forest.
                            A small smile forms on your tortured face.
                            Is this heaven?
                            Did you finally die?
                        [/p]
                        [p]
                            No.
                            Not yet.
                            You are thrown to the ground.
                            "[color=purple]You were trouble right from the start.
                            I shouldn't have wasted my time on you.[/color]"
                            That said, Tychus lifts up his heavy boot, and [b]stomps[/b] down on your leg.
                            A loud, sickening [b]snap[/b] reverberates across the empty slum-streets,
                            louder than the splashing of the raindrops on the rooftops and the dirt road.
                            Gut-wrenching pain, a thousand times worse than the whippings makes every passing moment a living hell.
                        [/p]
                        [p]
                            You [b]beg[/b] for mercy, but it's too late.
                            The human has made his decision.
                            A second time, his boot comes down, your other leg broken in the same fashion as the first.
                            You throw up what little contents you stomach has, the pain too much to bear.
                            Accidentally, while trying to look Tychus in the eyes, you gaze up at the sky, bright enough to [b]scorch[/b] your retinas.
                        [/p]
                        [p]
                            A sharp [b]kick[/b] to your already hurting chest snaps you out of your near-delirious state.
                            "[color=purple][b]CRAWL, slave.[/b]
                            And if you [b]dare[/b] stop before you're out of my sight,
                            then I'll break [b]every[/b] last bone in your body before allowing you to [b]die[/b].[/color]"
                        [/p]
                        [p]
                            Half-dead already, you dig your claws into the dirt, and you begin to crawl.
                            Blinded, crippled, in so much pain you can barely breathe, you crawl through the cold, muddy dirt.
                            You don't know where you're going.
                            You don't care.
                            As long as it's away from [b]him[/b].
                            You leave a trail of blood in your wake, quickly washed away by the rain.
                            You cough up even more blood.
                            Something inside you broke.
                        [/p]
                        [p]
                            You need help.
                            You whimper, beg and plead, but nobody is around to hear you.
                            Your arms are getting tired.
                            You can't feel your tail.
                            You lower your head into the mud.
                            Your heavy breathing slows down to a crawl.
                            And then it stops, forever more.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Gutter end.
                                [/b]
                            [/c]
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bgm.stopLoop();
            ctrl.reset();
            ctrl.reload();
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailFlee.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailSuccumb.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFailSuccumb(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorFailChoicesChoice.SUCCUMB;
    var reqController = ctrl.fail;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Threatening to beat you to [b]death[/b] if you don't comply,
                            Tychus forces your face into the bent-over bear's sweaty behind,
                            forcing your muzzle in between his furred buttcheeks.
                            Your face is coated in perspiration, the only fluid you've felt in [b]days[/b].
                            "[color=purple][b]LICK![/b][/color]" comes the simple command,
                            one you would [b]never[/b] have followed in your [i]previous[/i] life.
                            But it's amazing how much a kobold can change in a month of [b]torture[/b] and [b]imprisonment[/b].
                        [/p]
                        [p]
                            You don't want to get whipped.
                            Not anymore.
                            You want to be like those other kobolds.
                            The ones you saw at the very beginning of all of this.
                            They weren't scarred from head to toe.
                            They looked content.
                            Well-fed, even.
                            You'd [b]kill[/b] for a slice of bread with more [b]crust[/b] than [b]mold[/b].
                            If you play along, maybe you'll get rewarded.
                        [/p]
                        [p]
                            Your mind broken, your slender tongue snakes from your muzzle,
                            flicking out at the big ursine's rear much like Tychus' whip cracked along your chest the day before.
                            Your mouth is [b]flooded[/b] with filth, but being used to eating mostly mold by now,
                            it doesn't even faze you.
                            The bear seems pleased.
                            So you keep licking.
                            Lap after lap after lap, cleaning bits of dirt and grime from the fur on his ass.
                        [/p]
                        [p]
                            A few minutes later, you are pulled back, only to be forced into his armpit, to repeat the process.
                            Most of that afternoon, you spend exploring your ursine client's body with your tongue, cleaning him from head to toe,
                            all under Tychus' vigilant watch.
                            Finally, your gargantuan task comes to an end, as you dig out a particularly [b]nasty[/b] bit of filth from the big bear's belly button.
                            Your face is coated in sweat and dirt, your stomach is filled with much of the same, and you have forsaken your last bit of pride,
                            but at least you're not getting whipped tonight.
                        [/p]
                        [p]
                            Before throwing you back into your isolation cell, your human master gives you a few threatening words.
                            "[color=purple]Tomorrow, I won't be there to hold your head the entire time.
                            Tonguebathe your clients on your own, slave.
                            One complaint, and I'll do [b]far[/b] worse than just whipping you.[/color]"
                        [/p]
                        [p]
                            From the next day onwards, your new life begins.
                            You lick dirty fur, suckle grime from underneath untrimmed toenails, get used as a footmat by a dragon or two, eat equine ass,
                            tongue canine sheaths to clean the cockfilth from within, use your entire scaly body to wipe a well-built hippo clean of mud,
                            and even serve a feral every now and again.
                        [/p]
                        [p]
                            Not a week after you've accepted your place, you are let ouf ot your [i]hole[/i],
                            allowed to reside with the rest of the massagists.
                            They shun and look down on you, calling you a [b]whore[/b] and a [b]harlot[/b], but you're just doing what Tychus wants you to do.
                            At night, you sometimes get held down and beaten up by your fellow slaves, but never bad enough to leave any permanent marks.
                            It hurts, but not nearly as much as the whipping did.
                            Your twisted mind considers it a sign of friendship.
                            Every rookie went through this, right?
                        [/p]
                        [p]
                            Wrong.
                            A sixth kobold joins, and promptly joins the rest in abusing and ostracizing you.
                            But still, despite not fitting in with the other very well, your skillful tongue-work draws quite a crowd to the Kobold's Den.
                            Before long, the others are pulled from their massaging duties, forced to perform the same degrading acts you are.
                            This makes them like you [b]even less[/b].
                            Ontop of serving customers during the day, you are forced to clean your [i]roommates[/i] at night.
                        [/p]
                        [p]
                            The new one, especially, is a rowdy one.
                            He always takes things one step further.
                            One of the females makes you lick her slit clean of dirt,
                            he makes you suck him off.
                            Your tailhole gets fingered for a [i]prank[/i],
                            he climbs ontop of you, rams himself inside of your ass, and fucks the shit out of you.
                            The others beat you up as they have countless of times before,
                            he picks up a [b]rock[/b] he smuggled inside, and [b]bashes[/b] you with it until you scream so loud you wake up Tychus himself,
                            who promptly proceeds to [b]whip[/b] you.
                        [/p]
                        [p]
                            How much longer can you endure this abuse?
                            You don't know.
                            Does it matter?
                            At least you're no longer confined to your cell.
                            That place was [b]hell[/b].
                            But after the whipping, your master decides you're no longer fit to stay with the rest.
                            He only has one other place for you.
                            Back into the hole you go.
                            Into hell once again.
                        [/p]
                        [p]
                            From that point onwards,
                            your only relief from crushingly oppressive darkness are the sparse few moments in between being pulled from your cell,
                            and being forced into yet another client's rear.
                        [/p]
                        [p]
                            [c]
                                [b]
                                    Masseuse end.
                                [/b]
                            [/c]
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" style="margin-left: auto; position: relative; float: right;" id="${continueId}">`;
        choice += `&lt;&lt Revert`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.bgm.stopLoop();
            ctrl.reset();
            ctrl.reload();
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        registerContinueListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailSuccumb.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailChoices.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFailChoices(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.fail;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorFailFlee(ctrl);
    var secondNext = new AfternoonThreeParlorFailSuccumb(ctrl);

    var firstChoice = AfternoonThreeParlorFailChoicesChoice.FLEE;
    var secondChoice = AfternoonThreeParlorFailChoicesChoice.SUCCUMB;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = ``;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Flee</button>`;

        var tooltip = global.parser.parse(`You have to get out of here, no matter the cost!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Succumb</button>`;

        var tooltip = global.parser.parse(`Get used to your new life at the massage parlor.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var AfternoonThreeParlorFailChoicesChoice = {
    FLEE: 1,
    SUCCUMB: 2
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\choices\AfternoonThreeParlorFailChoices.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorFightFailEnd.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFightFailEnd(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preFail;
    var subController = false;
    var changeController = false;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.failChoices;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Tychus carries you past the massaging area, into a dark and shady back room,
                            where he unceremoniously [b]slams[/b] you down ontop of a hard, wooden table.
                            The bald man keeps you pinned with one hand,
                            while using the other to tie your limbs to the legs of the table, employing a length of rope.
                            You clothing is ripped from your body, discarded to the side.
                            Although still somewhat [i]out of it[/i], you are awake enough to realize that you are about to have a [b]very[/b] bad time.
                        [/p]
                        [p]
                            The rage that had befallen the human during your fight seems to have vanished completely.
                            A disconcertingly grim look is plastered across his pale face.
                            He reaches for another whip laying about, this time [i]without[/i] steel tip.
                            "[color=purple]All you had to do was accept the [b]damn[/b] massage.[/color]"
                            You attempt to respond, to perhaps smooth-talk your way out of this situation still,
                            but before you manage to utter even a single word, Tychus cracks his whip,
                            knocking the air out of your lungs by lashing you across the chest!
                        [/p]
                        [p]
                            You let out a coughing, pained grunt.
                            "[color=purple]This could have been so much [b]EASIER[/b]![/color]"
                            Another whiplash, this time smacking your soft and tender belly,
                            making you [b]scream[/b] out in [b]agony[/b].
                            Tears flow from your eyes, dripping to the ground below.
                            "[color=purple]Now you're [b]mine[/b]. And I intend to train you well.[/color]"
                        [/p]
                        [p]
                            That being said, the whipping begins properly.
                            Your weary body withers strike after strike of the grown man's powerful whipping.
                            You lose count somewhere after ten.
                            Your mind shuts down, escaping from the pain.
                            By the end of the brutal disciplinary session, your chest is a bloody mess of lacerations,
                            some stretching all the way from your waist up to your shoulders.
                        [/p]
                        [p]
                            You are thrown in a dark room, which is promptly locked.
                            Hours go by.
                            Then days.
                            You are barely fed.
                            Sometimes, instead of a moldy piece of bread, you get taken to the [i]punishment room[/i] for more whipping, to keep your injuries fresh.
                            For the better part of a month, your existence is a mixture of agony, wishing you were dead and thinking you already are.
                            The worst part is that the human still hasn't told you what he wants from you.
                            Or if he did, you can't remember.
                        [/p]
                        [p]
                            That changes one day, when instead of being whipped again, you are brought to the massaging room.
                            You can barely stand.
                            It's raining outside, but the meagre amount of sunlight beaming in through the window still hurts your eyes,
                            which are now used to pitch-black darkness and nothing more.
                            A big, burly anthro bear is laying atop one of the massage tables.
                            You are given one simple order: work his back.
                        [/p]
                        [p]
                            You don't know how to give a massage.
                            You meekly climb atop the table, and then atop the ursine.
                            You ooze some massage oil onto your palms, and try to rub his back as best you can.
                            Your best is not good enough.
                            The bear is everything but satisfied.
                            After half an hour of wasting his time, he pushes you off and complains about your performance to your [i]master[/i].
                            You fear more whipping, but that is not the case.
                            Instead, the human offers his client a new, special kind of service.
                            A tonguebath.
                            [i]Oh no[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("hamlet_shop");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorFightFailEnd.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorBottleFail.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBottleFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preFail;
    var subController = false;
    var changeController = false;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.failChoices;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Tychus carries you past the massaging area, into a dark and shady back room,
                            where he unceremoniously [b]slams[/b] you down ontop of a hard, wooden table.
                            The bald man keeps you pinned with one hand,
                            while using the other to tie your limbs to the legs of the table, employing a length of rope.
                            You clothing is ripped from your body, discarded to the side.
                            Although still somewhat [i]out of it[/i], you are awake enough to realize that you are about to have a [b]very[/b] bad time.
                        [/p]
                        [p]
                            The rage that had befallen the human during your fight seems to have vanished completely.
                            A disconcertingly grim look is plastered across his pale face.
                            He reaches for another whip laying about, this time [i]without[/i] steel tip.
                            "[color=purple]All you had to do was accept the [b]damn[/b] massage.[/color]"
                            You attempt to respond, to perhaps smooth-talk your way out of this situation still,
                            but before you manage to utter even a single word, Tychus cracks his whip,
                            knocking the air out of your lungs by lashing you across the chest!
                        [/p]
                        [p]
                            You let out a coughing, pained grunt.
                            "[color=purple]This could have been so much [b]EASIER[/b]![/color]"
                            Another whiplash, this time smacking your soft and tender belly,
                            making you [b]scream[/b] out in [b]agony[/b].
                            Tears flow from your eyes, dripping to the ground below.
                            "[color=purple]Now you're [b]mine[/b]. And I intend to train you well.[/color]"
                        [/p]
                        [p]
                            That being said, the whipping begins properly.
                            Your weary body withers strike after strike of the grown man's powerful whipping.
                            You lose count somewhere after ten.
                            Your mind shuts down, escaping from the pain.
                            By the end of the brutal disciplinary session, your chest is a bloody mess of lacerations,
                            some stretching all the way from your waist up to your shoulders.
                        [/p]
                        [p]
                            You are thrown in a dark room, which is promptly locked.
                            Hours go by.
                            Then days.
                            You are barely fed.
                            Sometimes, instead of a moldy piece of bread, you get taken to the [i]punishment room[/i] for more whipping, to keep your injuries fresh.
                            For the better part of a month, your existence is a mixture of agony, wishing you were dead and thinking you already are.
                            The worst part is that the human still hasn't told you what he wants from you.
                            Or if he did, you can't remember.
                        [/p]
                        [p]
                            That changes one day, when instead of being whipped again, you are brought to the massaging room.
                            You can barely stand.
                            It's raining outside, but the meagre amount of sunlight beaming in through the window still hurts your eyes,
                            which are now used to pitch-black darkness and nothing more.
                            A big, burly anthro bear is laying atop one of the massage tables.
                            You are given one simple order: work his back.
                        [/p]
                        [p]
                            You don't know how to give a massage.
                            You meekly climb atop the table, and then atop the ursine.
                            You ooze some massage oil onto your palms, and try to rub his back as best you can.
                            Your best is not good enough.
                            The bear is everything but satisfied.
                            After half an hour of wasting his time, he pushes you off and complains about your performance to your [i]master[/i].
                            You fear more whipping, but that is not the case.
                            Instead, the human offers his client a new, special kind of service.
                            A tonguebath.
                            [i]Oh no[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("hamlet_shop");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorBottleFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorRopeFail.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorRopeFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preFail;
    var subController = false;
    var changeController = false;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.failChoices;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            Suddenly, the door bursts open, and Tychus walks into the room.
                            A disconcertingly grim look is plastered across his pale face.
                            He reaches for a whip laying about, like the kind a slave-owner would use.
                            "[color=purple]All you had to do was ask for a job.[/color]"
                            You attempt to respond, to perhaps smooth-talk your way out of this situation,
                            but before you manage to utter even a single word, Tychus cracks his whip,
                            knocking the air out of your lungs by lashing you across the chest!
                        [/p]
                        [p]
                            You let out a coughing, pained grunt.
                            "[color=purple]This could have been so much [b]EASIER[/b]![/color]"
                            Another whiplash, this time smacking your soft and tender belly,
                            making you [b]scream[/b] out in [b]agony[/b].
                            Tears flow from your eyes, dripping to the ground below.
                            "[color=purple]But it doesn't matter.
                            You're [b]mine[/b] now.
                            And I intend to train you well.[/color]"
                        [/p]
                        [p]
                            That being said, the whipping begins properly.
                            Your massaged body withers strike after strike of the grown man's powerful whipping.
                            You lose count somewhere after ten.
                            Your mind shuts down, escaping from the pain.
                            By the end of the brutal disciplinary session, your chest is a bloody mess of lacerations,
                            some stretching all the way from your waist up to your shoulders.
                            What did you do to deserve this?
                            All you wanted was a massage.
                        [/p]
                        [p]
                            You are thrown in a dark room, which is promptly locked.
                            Hours go by.
                            Then days.
                            You are barely fed.
                            Sometimes, instead of a moldy piece of bread, you get taken to the [i]punishment room[/i] for more whipping, to keep your injuries fresh.
                            For the better part of a month, your existence is a mixture of agony, wishing you were dead and thinking you already are.
                        [/p]
                        [p]
                            Then, one day, instead of being whipped again, you are brought to the massaging room.
                            You can barely stand.
                            It's raining outside, but the meagre amount of sunlight beaming in through the window still hurts your eyes,
                            which are now used to pitch-black darkness and nothing more.
                            A big, burly anthro bear is laying atop one of the massage tables.
                            You are given one simple order: work his back.
                        [/p]
                        [p]
                            You don't know how to give a massage.
                            You meekly climb atop the table, and then atop the ursine.
                            You ooze some massage oil onto your palms, and try to rub his back as best you can.
                            Your best is not good enough.
                            The bear is everything but satisfied.
                            After half an hour of wasting his time, he pushes you off and complains about your performance to your [i]master[/i].
                            You fear more whipping, but that is not the case.
                            Instead, the human offers his client a new, special kind of service.
                            A tonguebath.
                            [i]Oh no[/i].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("hamlet_shop");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorRopeFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorMasseuseFail.js

/**
 * Fail section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preFail;
    var subController = false;
    var changeController = false;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.failChoices;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var broke = ctrl.knockout.get() === AfternoonThreeParlorKnockoutAwakenChoice.BREAK;
        var toReturn = `[p]
                            You wake up a while later, tied to the table you escaped from once more.
                            ${broke ? `The leg you broke off must have been jury-rigged back in place.` : ``}
                            Tychus is in the room.
                            A disconcertingly grim look is plastered across his pale face.
                            He reaches for a whip laying about, like the kind a slave-owner would use.
                            "[color=purple]Awake already?[/color]"
                            You attempt to respond, to perhaps smooth-talk your way out of this situation,
                            but before you manage to utter even a single word, Tychus cracks his whip,
                            knocking the air out of your lungs by lashing you across the chest!
                        [/p]
                        [p]
                            You let out a coughing, pained grunt.
                            "[color=purple]This could have been so much [b]EASIER[/b]![/color]"
                            Another whiplash, this time smacking your soft and tender belly,
                            making you [b]scream[/b] out in [b]agony[/b].
                            Tears flow from your eyes, dripping to the ground below.
                            "[color=purple]You shouldn't have tried to escape.
                            But it doesn't matter.
                            You're [b]mine[/b] now.
                            And I intend to train you well.[/color]"
                        [/p]
                        [p]
                            That being said, the whipping begins properly.
                            Your massaged body withers strike after strike of the grown man's powerful whipping.
                            You lose count somewhere after ten.
                            Your mind shuts down, escaping from the pain.
                            By the end of the brutal disciplinary session, your chest is a bloody mess of lacerations,
                            some stretching all the way from your waist up to your shoulders.
                            What did you do to deserve this?
                            All you wanted was a massage.
                        [/p]
                        [p]
                            You are thrown in a dark room, which is promptly locked.
                            Hours go by.
                            Then days.
                            You are barely fed.
                            Sometimes, instead of a moldy piece of bread, you get taken to the [i]punishment room[/i] for more whipping, to keep your injuries fresh.
                            For the better part of a month, your existence is a mixture of agony, wishing you were dead and thinking you already are.
                        [/p]
                        [p]
                            Then, one day, instead of being whipped again, you are brought to the massaging room.
                            You can barely stand.
                            It's raining outside, but the meagre amount of sunlight beaming in through the window still hurts your eyes,
                            which are now used to pitch-black darkness and nothing more.
                            A big, burly anthro bear is laying atop one of the massage tables.
                            You are given one simple order: work his back.
                        [/p]
                        [p]
                            You don't know how to give a massage.
                            You meekly climb atop the table, and then atop the ursine.
                            You ooze some massage oil onto your palms, and try to rub his back as best you can.
                            Your best is not good enough.
                            The bear is everything but satisfied.
                            After half an hour of wasting his time, he pushes you off and complains about your performance to your [i]master[/i].
                            You fear more whipping, but that is not the case.
                            Instead, the human offers his client a new, special kind of service.
                            A tonguebath.
                            [i]Oh no[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("hamlet_shop");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\fail\AfternoonThreeParlorMasseuseFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeAcceptLeave.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFreeAcceptLeave(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preContinue;
    var changeController = ctrl.killFreeAcceptLeave;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        var alreadyDislocated = global.kobold.history.shared.well.basiliskFingerDislocated;
        var toReturn = `[p]
                            You wake up not too long later, still on the floor.
                            Red is gone.
                            The other two are gone.
                            Even Tychus' corpse is missing.
                            Your ${arms ? `arm` : back ? `back` : `foot`} hurts.
                            [b]Everything[/b] hurts.
                        [/p]`;

        if (arms) {
            toReturn += `[p]
                            Your arm is still fucked up.
                            Dislocated,
                            bending the wrong way${alreadyDislocated ? `, like your finger did in the well` : ``}.
                            There's only one way to fix this.
                            And you aren't looking forward to it.
                        [/p]
                        [p]
                            You pick up a convenient, small piece of wood laying about, and place it in your muzzle, to bite down on.
                            You'll need it.
                            This is going to hurt.
                            With your one functioning arm, you grab onto your limp-hanging wrist.
                            Taking a deep breath, you [b]pull[/b].
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Once again, you [b]scream[/b] out in pain,
                            completely reduced to tears as your bones [b]painfully[/b] re-arrange themselves back into position.
                            Your teeth dig into the stick in your mouth, you wildly [b]bang[/b] your one functioning hand against the wooden floor,
                            you kick and flail your legs about, and you cry your eyeballs out.
                            It's [b]too much[/b] pain!
                        [/p]
                        [p]
                            Your shallow, ragged breathing slowly turns into deep, soothing breaths as the massive spike of tormenting trauma begins to die down,
                            leaving you a sobbing green mess of scales and tears.
                            You just wanted to help them.
                            And this is how they repaid you.
                        [/p]
                        [p]
                            You spit out the piece of wood, and through your teary eyes,
                            you are glad to see that you have regained full motion over your relocated arm.
                            Moving it still hurts, but you're certain that will improve over time.
                            Maybe you'd have been better off [b]not[/b] trying to be a hero, but damnit, you feel like you did the right thing.
                            You'd do it again, if given a chance, apart from maybe trusting Red.
                        [/p]
                        [p]
                            You need to get out of here.
                            It's time to leave.
                            You don't care where Tychus went.
                            You don't care where the kobolds went.
                            You don't [b]ever[/b] want to see that red one again.
                            And you don't [b]ever[/b] want to experience [b]any[/b] kind of pain like this again.
                        [/p]
                        [p]
                            You scramble to your feet,
                            and waddle out the door, never to return to this [b]damnable[/b] place.
                            Still clutching your arm, you walk off the pain,
                            headed for the mercantile district.
                        [/p]
                        [p]
                            Arriving at your destination, the sun is slowly starting to set.
                            The lower it gets, the more distant your agony seems.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe tomorrow, you can find some more [b]grateful[/b] kobolds to [i]liberate[/i].
                        [/p]`;
        } else if (back) {
            toReturn += `[p]
                            Your back is still fucked up.
                            A vertebra is dislocated,
                            jutting out from the rest of your spine${alreadyDislocated ? `, like your finger was in the well, but worse` : ``}.
                            There's only one way to fix this.
                            And you aren't looking forward to it.
                        [/p]
                        [p]
                            You pick up a convenient, small piece of wood laying about, and place it in your muzzle, to bite down on.
                            You'll need it.
                            This is going to hurt.
                            With one arm, you reach for your back, and place your palm atop the protruding bit of bone.
                            Taking a deep breath, you [b]ram[/b] it back into place.
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Once again, you [b]scream[/b] out in pain,
                            completely reduced to tears as your spine [b]painfully[/b] re-arranges itself back into proper position.
                            Your teeth dig into the stick in your mouth, you wildly [b]bang[/b] your balled-up hands against the wooden floor,
                            and you cry your eyeballs out.
                            It's [b]too much[/b] pain!
                        [/p]
                        [p]
                            Your shallow, ragged breathing slowly turns into deep, soothing breaths as the massive spike of tormenting trauma begins to die down,
                            leaving you a sobbing green mess of scales and tears.
                            You just wanted to help them.
                            And this is how they repaid you.
                        [/p]
                        [p]
                            You spit out the piece of wood, and through your teary eyes,
                            you are glad to see that you have regained full motion over your legs.
                            Moving them is still a bit iffy, but you're certain that will improve over time.
                            Maybe you'd have been better off [b]not[/b] trying to be a hero, but damnit, you feel like you did the right thing.
                            You'd do it again, if given a chance, apart from maybe trusting Red.
                        [/p]
                        [p]
                            You need to get out of here.
                            It's time to leave.
                            You don't care where Tychus went.
                            You don't care where the kobolds went.
                            You don't [b]ever[/b] want to see that red one again.
                            And you don't [b]ever[/b] want to experience [b]any[/b] kind of pain like this again.
                        [/p]
                        [p]
                            You climb to your unsteady feet,
                            and waddle out the door, never to return to this [b]damnable[/b] place.
                            Using one of your arms to keep your back straightened, you walk off the pain,
                            headed for the mercantile district.
                        [/p]
                        [p]
                            Arriving at your destination, the sun is slowly starting to set.
                            The lower it gets, the more distant your agony seems.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe tomorrow, you can find some more [b]grateful[/b] kobolds to [i]liberate[/i].
                        [/p]`;
        } else {
            toReturn += `[p]
                            Your foot is still fucked up.
                            Dislocated,
                            twisted the wrong way around${alreadyDislocated ? `, like your finger was in the well` : ``}.
                            There's only one way to fix this.
                            And you aren't looking forward to it.
                        [/p]
                        [p]
                            You pick up a convenient, small piece of wood laying about, and place it in your muzzle, to bite down on.
                            You'll need it.
                            This is going to hurt.
                            With both of your hands, your reach for your limp foot, firmly grasping onto it.
                            Taking a deep breath, you [b]twist[/b] it back into place.
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Once again, you [b]scream[/b] out in pain,
                            completely reduced to tears as your bones [b]painfully[/b] re-arrange themselves back into position.
                            Your teeth dig into the stick in your mouth, you wildly [b]bang[/b] your balled-up hands against the wooden floor,
                            you kick and flail your one good leg about, and you cry your eyeballs out.
                            It's [b]too much[/b] pain!
                        [/p]
                        [p]
                            Your shallow, ragged breathing slowly turns into deep, soothing breaths as the massive spike of tormenting trauma begins to die down,
                            leaving you a sobbing green mess of scales and tears.
                            You just wanted to help them.
                            And this is how they repaid you.
                        [/p]
                        [p]
                            You spit out the piece of wood, and through your teary eyes,
                            you are glad to see that you have regained full motion over your foot.
                            Wriggling your toes about still hurts, but you're certain that will improve over time.
                            Maybe you'd have been better off [b]not[/b] trying to be a hero, but damnit, you feel like you did the right thing.
                            You'd do it again, if given a chance, apart from maybe trusting Red.
                        [/p]
                        [p]
                            You need to get out of here.
                            It's time to leave.
                            You don't care where Tychus went.
                            You don't care where the kobolds went.
                            You don't [b]ever[/b] want to see that red one again.
                            And you don't [b]ever[/b] want to experience [b]any[/b] kind of pain like this again.
                        [/p]
                        [p]
                            You climb to your unsteady feet, leaning most of your weight on the foot that [i]wasn't[/i] dislocated,
                            and you limp out the door, never to return to this [b]damnable[/b] place.
                            As you walk off the pain,
                            headed for the mercantile district,
                            your gait gradually improves,
                            as you begin to use your hurting foot more and more.
                        [/p]
                        [p]
                            Arriving at your destination, the sun is slowly starting to set.
                            The lower it gets, the more distant your agony seems.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe tomorrow, you can find some more [b]grateful[/b] kobolds to [i]liberate[/i].
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeAcceptLeave.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeAccept.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFreeAccept(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorKillFreeChoice.ACCEPT;
    var reqController = ctrl.killFree;
    var subController = ctrl.preContinue;
    var changeController = ctrl.killFreeAccept;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new AfternoonThreeParlorFreeAcceptLeave(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorInjured = true;
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        var toReturn = `[p]
                            Some [i]aftercare[/i], huh?
                            You shoot the red kobold a lecherous smile.
                            Sure, you'd [b]love[/b] that.
                            You give her a suave wink.
                        [/p]`;

        if (arms) {
            toReturn += `[p]
                            "[color=bred]Very well then.
                            Stick out your arm, and close your eyes.[/color]"
                            Wait, she intends to do this [i]here[/i]?
                            In front of the others?
                            Oh.
                            [b]Oh.[/b]
                            She wants to give you some [i]actual[/i] massage-aftercare.
                            You thought she wanted to --
                            Judging by her unamused stare, you wisely drop your thinking out loud.
                            Instead, you comply with her wishes, closing your eyes and sticking one of your arms.
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Immense [b]pain[/b] floods your mind, so much of it that you can't help but [b]scream[/b] out in agony.
                            Opening your eyes, you witness your arm bending [b]the wrong way[/b]!
                            "[color=bred]He was [b]MINE[/b] to kill![/color]"
                            You clutch your elbow, feeling woozy, your sight growing dark.
                            "[color=bred]You [b]ROBBED[/b] me of the chance to earn my honor back![/color]"
                        [/p]
                        [p]
                            As if to add insult to injury, Red harshly slaps you across the face.
                            You barely even feel your cheek sting, the overbearing torment permeating from your dislocated arm fully eclipsing it.
                            Your mind can't handle this.
                            Your legs feel weak.
                            You collapse to the floor, and pass out.
                        [/p]`;
        } else if (back) {
            toReturn += `[p]
                            "[color=bred]Very well then.
                            Turn around, and close your eyes.[/color]"
                            Wait, she intends to do this [i]here[/i]?
                            In front of the others?
                            Oh.
                            [b]Oh.[/b]
                            She wants to give you some [i]actual[/i] massage-aftercare.
                            You thought she wanted to --
                            Judging by her unamused stare, you wisely drop your thinking out loud.
                            Instead, you comply with her wishes, closing your eyes and turning your back to her.
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Immense [b]pain[/b] floods your mind, so much of it that you can't help but [b]scream[/b] out in agony.
                            She [b]karate-chopped[/b] you straight in the [b]spine[/b]!
                            You can't feel [b]anything[/b] below your waist!
                            You collapse to the ground, your unresponsive legs no longer able to carry you.
                            "[color=bred]He was [b]MINE[/b] to kill![/color]"
                            You feel around your back.
                            There's a [b]lump[/b] around halfway up your spine.
                            "[color=bred]You [b]ROBBED[/b] me of the chance to earn my honor back![/color]"
                        [/p]
                        [p]
                            As if to add insult to injury, Red harshly [b]kicks[/b] your legs, but you can't even feel them.
                            They're completely numb.
                            Your mind can't handle this.
                            Your sight goes black.
                            You pass out on the floor.
                        [/p]`;
        } else {
            toReturn += `[p]
                            "[color=bred]Very well then.
                            Stick out your foot, and close your eyes.[/color]"
                            Wait, she intends to do this [i]here[/i]?
                            In front of the others?
                            Oh.
                            [b]Oh.[/b]
                            She wants to give you some [i]actual[/i] massage-aftercare.
                            You thought she wanted to --
                            Judging by her unamused stare, you wisely drop your thinking out loud.
                            Instead, you comply with her wishes, closing your eyes and sticking out your foot.
                        [/p]
                        [p]
                            [big][b]CRACK[/b][/big]
                        [/p]
                        [p]
                            Immense [b]pain[/b] floods your mind, so much of it that you can't help but [b]scream[/b] out in agony.
                            In the flash of an eye, she stepped on your foot to hold it down,
                            and then [b]threw[/b] you to the ground sideways, twisting and [b]dislocating[/b] your foot!
                            Opening your eyes, you can see it stand crooked.
                            Your very [b]bones[/b] grind against one another as you attempt to [b]move[/b] it!
                            "[color=bred]He was [b]MINE[/b] to kill![/color]"
                            It [b]hurts[/b]!
                            "[color=bred]You [b]ROBBED[/b] me of the chance to earn my honor back![/color]"
                        [/p]
                        [p]
                            As if to add insult to injury, Red harshly [b]kicks[/b] your stomach, but the pain is [b]nothing[/b] compared to your foot.
                            Your mind can't handle this.
                            Your sight goes black.
                            You pass out on the floor.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("varanar_indoor_knockout");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeAccept.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeRefuse.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFreeRefuse(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorKillFreeChoice.REFUSE;
    var reqController = ctrl.killFree;
    var changeController = ctrl.killFreeRefuse;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You kindly decline her inappropriate advances,
                            thinking that perhaps she was [i]trained[/i] to offer such services,
                            and agreeing would be kind of [i]taking advantage[/i] of Red.
                            She shrugs.
                            "[color=bred]Well, I don't have much else to offer my [i]gratitude[/i] with.
                            Are you sure you don't want to...?[/color]"
                            You resolutely shake your head no.
                            The kobold sighs.
                        [/p]
                        [p]
                            "[color=bred]Fine, but you'd best get going.
                            Before Green goes [b]mental[/b] and starts attacking you or something.[/color]"
                            Given how the male kobold hasn't calmed down one bit, you don't doubt that that is a [b]very[/b] real possibility.
                            Receiving not so much as a [i]thank you[/i],
                            you walk towards the front door, and exit the building.
                            You're confident those three'll do fine without you.
                            They just need a while to recover.
                            They'll be back to their old selves in no-time.
                            And if they're not, Red seemed clever enough to keep them out of trouble.
                        [/p]
                        [p]
                            Closing the door behind you, you are left with mixed feelings.
                            The way those three reacted to seeing their dead master, perhaps they would have been better off [i]not[/i] being rescued?
                            Maybe they [b]liked[/b] it there.
                            Did you end up doing something bad instead of something good?
                            If being a knight involves complex situations like these, maybe you're better off remaining a thief.
                        [/p]
                        [p]
                            Oh well, at least you got to [b]smite[/b] an [b]evildoer[/b].
                            But now, you'd better get out of here.
                            [b]Before[/b] Varanar's guards get involved in this entire ordeal.
                            You walk towards the mercantile district.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe tomorrow, you can find some more [b]grateful[/b] kobolds to free.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\free\AfternoonThreeParlorFreeRefuse.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorKillFree.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorKillFree(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondFightChoice.FREE;
    var reqController = ctrl.itemGetFight;
    var subController = ctrl.killFree;
    var changeController = ctrl.killFree;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorFreeRefuse(ctrl);
    var secondNext = new AfternoonThreeParlorFreeAccept(ctrl);

    var firstChoice = AfternoonThreeParlorKillFreeChoice.REFUSE;
    var secondChoice = AfternoonThreeParlorKillFreeChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFreed = true;
        var toReturn = `[p]
                            After a minute of indulging in the rare feeling of [b]pride[/b],
                            your mind settles down from the adrenaline rush that fueled your vicious attack.
                            You did it!
                            You beat the insane man that tried to enslave you!
                            Now to free your fellow kobolds from his apalling rule.
                        [/p]
                        [p]
                            You walk over to the door leading to the [i]break room[/i],
                            and triumphantly swing it open, drawing the attention of the three slaves inside.
                            "[color=blue]Yes, mast-[/color]" the blue one speaks, realizing you aren't the human halfway through her sentence.
                            "[color=blue]Huh? Who are you? A client?[/color]"
                            The green one looks past you, and notices Tychus laying on the floor.
                        [/p]
                        [p]
                            "[color=green]Master?![/color]"
                            Before even offering you a chance to explain what's going on,
                            he rushes past you, crudely pushing you out of the way to check up on the human.
                            The blue one follows suit, kneeling besides Tychus' body.
                            The red one doesn't seem to care all too much.
                            She casually saunters out of the break room, and casually leans against a wall, observing the display.
                        [/p]
                        [p]
                            "[color=green]Master, [b]no[/b]!
                            You [b]KILLED[/b] him![/color]" the green kobold laments hysterically,
                            his loud crying and plentiful tears stressing his apparent [i]fondness[/i] of Tychus.
                            The blue one tried to soothe him, wrapping an arm around him, pulling him close, letting him cry on her shoulder.
                            "[color=blue]Shhh, Green.
                            It's okay.
                            Everything's going to be fine.
                            Focus on your breathing, okay?
                            Like I taught you.[/color]"
                        [/p]
                        [p]
                            While consoling her peer, Blue shoots daggers at you with her eyes, glaring at you with a hateful look on her face.
                            You [i]did[/i] cause her friend to have a [b]mental breakdown[/b], but you thought you were doing the right thing!
                            You [b]freed[/b] them!
                            Interrupting your guilt-trip, Red taps your shoulder, and whispers to you:
                            "[color=bred]Old bastard finally got what he deserved, huh?
                            Good job.
                            Orange gave you a massage, right?
                            How about I give you some [i]aftercare[/i]?[/color]"
                            You like the sound of that.
                            You like the sound of that [b]a lot[/b].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Refuse</button>`;

        var tooltip = global.parser.parse(`Freeing them is a reward in itself.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 140px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`You could use a little [i]extra[/i], for your troubles.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var AfternoonThreeParlorKillFreeChoice = {
    REFUSE: 1,
    ACCEPT: 2
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorKillFree.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorItemGetLoot.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondFightChoice.LOOT;
    var reqController = ctrl.itemGetFight;
    var changeController = ctrl.secondLoot;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var killed = global.kobold.history.shared.varanar.dayThree.afternoon.parlorKilled;
        var toReturn = `[p]
                            After a minute of indulging in the rare feeling of [b]pride[/b],
                            your mind settles down from the adrenaline rush that fueled your vicious attack.
                            You did it!
                            You beat the insane man that tried to enslave you!
                            You've done a favor to the city${killed ? `` : `, assuming he'll remember the lesson you taught him when he wakes up`}.
                        [/p]
                        [p]
                            Then again, the [i]law[/i] might not see it that way.
                            You [b]assaulted[/b] ${killed ? `and even [b]killed[/b]` : ``} a human.
                            He's a prick, and he totally deserved what happened to him, but he has rights, and you don't.
                            While you would love to stay and spread the good news to your peers,
                            someone might have [b]heard[/b] the struggle that took place.
                        [/p]
                        [p]
                            It'd be best if you took whatever you can and got the hell out of here [i]before[/i] anyone comes to investigate.
                            You bolt around the room, investigating a few points of interest, swiftly and superficially searching some cupboards and racks.
                            A tiny haul.
                            A bit of ${global.kobold.inventory.createItemWithTooltip(new aPlusLotion())} and some coins.
                            Not a lot, but more than nothing!
                            With your loot in hand, you bolt out of the door, and make yourself scarce from the slums.
                            A few minutes later, you find yourself in the mercantile district.
                            You're fairly certain nobody saw you leave the massage parlor, and you don't think anyone followed you, so you must be in the clear!
                        [/p]
                        [p]
                            You did a [b]good[/b] thing today.
                            Well, about as good as ${killed ? `killing someone` : `knocking someone out` } can be.
                            Anyway, the sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe you can [b]smite[/b] some more [b]evildoers[/b] tomorrow!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorItemGetLoot.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorSpareFree.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorSpareFree(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondFightChoice.FREE;
    var reqController = ctrl.itemGetFight;
    var changeController = ctrl.spareFree;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFreed = true;
        var toReturn = `[p]
                            After a minute of indulging in the rare feeling of [b]pride[/b],
                            your mind settles down from the adrenaline rush that fueled your vicious attack.
                            You did it!
                            You beat the insane man that tried to enslave you!
                            Now to free your fellow kobolds from his apalling rule.
                        [/p]
                        [p]
                            You walk over to the door leading to the [i]break room[/i],
                            and triumphantly swing it open, drawing the attention of the three slaves inside.
                            "[color=blue]Yes, mast-[/color]" the blue one speaks, realizing you aren't the human halfway through her sentence.
                            "[color=blue]Huh? Who are you? A client?[/color]"
                            The green one looks past you, and notices Tychus laying on the floor.
                        [/p]
                        [p]
                            "[color=green]Master?![/color]"
                            Before even offering you a chance to explain what's going on,
                            he rushes past you, crudely pushing you out of the way to check up on the human.
                            The blue one follows suit, kneeling besides Tychus' body.
                            The red one doesn't seem to care all too much.
                            She casually saunters out of the break room, and casually leans against a wall, observing the display.
                        [/p]
                        [p]
                            "[color=green]Master, [b]no[/b]!
                            You [b]KILLED[/b] him![/color]" the green kobold laments hysterically,
                            his loud crying and plentiful tears stressing his apparent [i]fondness[/i] of Tychus.
                            The blue one, luckily, is a bit less [i]indoctrinated[/i], and a bit more [b]reasonable[/b].
                            She presses her hand against her master's chest, and notices the human is still breathing.
                            "[color=blue]Shhh, Green.
                            It's okay, the master's still alive.
                            Look, he's breathing.
                            See?[/color]"
                        [/p]
                        [p]
                            It's like a sparkle of hope lights up in the green kobold's eyes,
                            his Stockholm-syndrome affected mind overjoyous that the human is still alive.
                            He curls up next to Tychus' body, and begins disturbingly rocking back and forth, calming himself down.
                            "[color=green]Master's still alive.[/color]"
                            "[color=blue]That's right, Green.
                            The master'll be fine.
                            I'm going to talk to the client now, okay?[/color]"
                            The only response Blue gets is a slight nod, the other kobold barely even acknowledging her existence,
                            too focused on the person that kept him in captivity.
                        [/p]
                        [p]
                            Rising to her feet, the blue kobold walks over to you, addressing you in a hushed voice, to shield Green from the truth.
                            "[color=blue]Thank goodness.
                            I thought I'd be trapped here forever.
                            Are you with the guild?
                            Did they send you to rescue me?[/color]"
                            Guild?
                            You're not really part of any guild, no.
                            "[color=blue]Oh.
                            Doesn't matter.
                            Thank you regardless.
                            Please excuse Green, he's been here for a [b]long[/b] time.
                            After months of [b]whipping[/b] and [b]torture[/b], your brain kind of starts to go [i]haywire[/i].
                            And I think he was sort of [i]special[/i] to begin with.
                            I hope you understand.[/color]"
                        [/p]
                        [p]
                            You can't imagine what that would be like, but you [i]get it[/i].
                            You ask Blue what she thinks you should do next.
                            She seems far more versed in [i]handling[/i] her [b]emotional[/b] friend than you are,
                            and you'd hate to have to leave someone behind because they [i]think[/i] they like it here.
                            "[color=blue]Don't worry.
                            I'll take him and Red somewhere safe.[/color]"
                            You ask if they have [i]real[/i] names, instead of y'know...[i]colors[/i].
                            "[color=blue]The others, I don't know.
                            They never told, I never asked.
                            Me?
                            My name actually [i]was[/i] Blue.
                            [i]Before[/i] coming to this [b]accursed[/b] place, that is.[/color]"
                        [/p]
                        [p]
                            Oh.
                            [i]Oh.[/i]
                            You didn't mean to offend or anything.
                            "[color=blue]It's fine.
                            Look, it'd be better if we [i]weren't[/i] seen coming out of this building together.
                            How about you get going,
                            and I'll be out with the rest in a minute or two, alright?[/color]"
                            You agree with the blue-scaled kobold's plan.
                            She asks if you've seen Orange around, but you half-heartedly tell her not to worry about Orange.
                            Shrugging, she thanks you one final time, and then you head for the front door, looking both ways to ensure your [i]privacy[/i].
                            Nobody's there.
                        [/p]
                        [p]
                            You cast one final gaze back at the three kobolds.
                            Red seems to understand what's going on, and is pilfering some items from the lobby, preparing to leave.
                            Blue's talking to Green, trying to get him on his feet, lying and saying they're going for a [i]walk[/i].
                            Whatever it takes, you suppose.
                        [/p]
                        [p]
                            You stride out into the empty streets, and wait around the corner.
                            A few minutes later, as she promised, Blue and the other two emerge from the massage parlor.
                            They head the opposite way you went, and disappear into the first side-street.
                            You jog after them, but as you turn the same corner they did, you find not a single trace of the group of three.
                        [/p]
                        [p]
                            Odd.
                            There aren't really any other ways to go.
                            In fact, you think this street is a [b]dead end[/b].
                            Weird.
                            Very weird.
                            Almost like [b]magic[/b].
                            You wonder what that [i]guild[/i] was that Blue spoke of.
                        [/p]
                        [p]
                            Oh well, you'd better get out of here yourself.
                            [b]Before[/b] Tychus wakes up to find his slaves [i]liberated[/i].
                            You walk towards the mercantile district.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe tomorrow, you can set some more kobolds free.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\postfight\AfternoonThreeParlorSpareFree.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetSpare.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetSpare(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.itemGetFight;
    var changeController = ctrl.itemGetSpare;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorSpareFree(ctrl);
    var secondNext = new AfternoonThreeParlorItemGetLoot(ctrl);

    var firstChoice = ctrl.secondFightChoice.FREE;
    var secondChoice = ctrl.secondFightChoice.LOOT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFought = true;
        var weapon = ctrl.misc.weapon();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        var toReturn = `[p]
                            Tychus' reign of terror needs to end.
                            And what more noble thing is there than to slay the wicked?
                            Surely, this is what a knight would do, in your situation!
                            The human is distracted by his notes.
                            You can sneak up behind him, and then end his miserable existence!
                            At least that's the plan.
                            It's not the most chivalrous way to fight evil,
                            but you guess a little [i]improvisation[/i] is okay when you're half the size and a fourth of the bulk of a regular knight.
                        [/p]
                        [p]
                            You slowly slip into the lobby, and begin to move towards Tychus' desk as silently as you possibly can.
                            There is little in the room to hide yourself behind, so you lower your frame as much as you possibly can,
                            attempting to use the desk itself as cover.
                            The human is too engrossed in his reading to realize that his latest [i]acquisition[/i] hasn't been restrained.
                            And with some luck, he won't understand what's going on until it's too late.
                        [/p]
                        [p]
                            You creep all the way up to the desk, where you quietly take a deep breath, your body concealed by the large piece of wooden furniture.
                            Your nerves steeled, you maneuver around to the rear of the seated, sickly bald man.
                            He still hasn't noticed you.
                            ${weapon ? `You ready your ${weaponPopup}.` : `Lacking a weapon, you grab a hefty jar from a nearby rack.`}
                            It's time to end this.
                        [/p]
                        `;

        if (egg) {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            smacking your ${weaponPopup} against the back of his head!
                            The egg shatters, and its disgusting, goopy insides ooze down the human's head.
                            Your strike ineffective, you quickly follow up by grabbing a jar from a nearby rack,
                            and [b]breaking[/b] it atop his very [b]skull[/b], which splits open with a nasty [b]crack[/b],
                            audible even over the noise of shattering ceramic!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        } else if (weapon) {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            brutally [b]bashing[/b] the back of Tychus' head,
                            your weapon [b]burying[/b] into his very [b]skull[/b], which splits open with a nasty [b]crack[/b]!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        } else {
            toReturn += `[p]
                            With a loud grunt, you [b]lash[/b] out,
                            brutally [b]bashing[/b] the back of Tychus' head with the jar,
                            your temporary weapon [b]breaking[/b] atop his very [b]skull[/b], which splits open with a nasty [b]crack[/b],
                            audible even over the noise of shattering ceramic!
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor,
                            completely lifeless, apart from his chest moving up and down ever so slightly,
                            signalling that his flame hasn't been [i]completely[/i] snuffed out yet.
                        [/p]`;
        }

        toReturn += `   [p]
                            A puddle of blood oozes out from Tychus' head.
                            You are breathing heavily.
                            The deed is done.
                            He's only knocked unconscious, but that's good enough for you.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Free</button>`;

        var tooltip = global.parser.parse(`Release your fellow kobolds from oppression.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Loot</button>`;

        var tooltip = global.parser.parse(`Steal whatever you can before Tychus wakes up!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetSpare.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetKill.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetKill(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.itemGetFight;
    var changeController = ctrl.itemGetKill;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorItemGetLoot(ctrl);
    var secondNext = new AfternoonThreeParlorKillFree(ctrl);

    var firstChoice = ctrl.secondFightChoice.LOOT;
    var secondChoice = ctrl.secondFightChoice.FREE;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFought = true;
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorKilled = true;
        var weapon = ctrl.misc.weapon();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var slash = weapon.slash();
        var toReturn = `[p]
                            Tychus' reign of terror needs to end.
                            And what more noble thing is there than to slay the wicked?
                            Surely, this is what a knight would do, in your situation!
                            The human is distracted by his notes.
                            You can sneak up behind him, and then end his miserable existence!
                            At least that's the plan.
                            It's not the most chivalrous way to fight evil,
                            but you guess a little [i]improvisation[/i] is okay when you're half the size and a fourth of the bulk of a regular knight.
                        [/p]
                        [p]
                            You slowly slip into the lobby, and begin to move towards Tychus' desk as silently as you possibly can.
                            There is little in the room to hide yourself behind, so you lower your frame as much as you possibly can,
                            attempting to use the desk itself as cover.
                            The human is too engrossed in his reading to realize that his latest [i]acquisition[/i] hasn't been restrained.
                            And with some luck, he won't understand what's going on until it's too late.
                        [/p]
                        [p]
                            You creep all the way up to the desk, where you quietly take a deep breath, your body concealed by the large piece of wooden furniture.
                            Your nerves steeled, you maneuver around to the rear of the seated, sickly bald man.
                            He still hasn't noticed you.
                            You ready your ${weaponPopup}.
                            It's time to end this.
                        [/p]
                        [p]
                            With a loud grunt, you [b]lash[/b] out,
                            ${slash ? `brutally [b]slashing[/b] the back of Tychus' neck,
                            your weapon digging deep enough to sever his very spinal cord!` : `brutally [b]stabbing[/b] the back of Tychus' bald head,
                            your weapon digging deep enough to sink into his very brain.`}
                            Startled, you take a small step backwards, and drop into a combat stance,
                            as his body lets out a few final spasms,
                            before falling out of his chair, and dropping to the floor, completely lifeless.
                        [/p]
                        [p]
                            A puddle of blood oozes out from Tychus' corpse.
                            You are breathing heavily.
                            The deed is done.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Loot</button>`;

        var tooltip = global.parser.parse(`Time to reap the spoils of war.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 125px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Free</button>`;

        var tooltip = global.parser.parse(`Release your fellow kobolds from oppression.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 140px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetKill.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetFight.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetFight(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorItemGetChoice.FIGHT;
    var reqController = ctrl.itemGet;

    var toFail = new AfternoonThreeParlorItemGetKill(ctrl);
    var toSuccess = new AfternoonThreeParlorItemGetSpare(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var weapon = ctrl.misc.weapon();
        return !weapon || weapon.blunt() || weapon.id === ItemEnum.ROTTEN_EGG;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\fight\AfternoonThreeParlorItemGetFight.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\AfternoonThreeParlorItemGetFlee.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGetFlee(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorItemGetChoice.FLEE;
    var reqController = ctrl.itemGet;
    var changeController = ctrl.itemGetFlee;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorRan = true;
        var him = ctrl.misc.baseGender() === GenderEnum.MALE;
        var toReturn = `[p]
                            You need to get out of here.
                            The sooner, the better.
                            Tychus is distracted by his notes.
                            He didn't notice you entering the building, so he won't notice you leaving until you're already out the door.
                            At least that's the plan.
                        [/p]
                        [p]
                            You slowly slip into the lobby, and begin to move towards the front door as silently as you possibly can.
                            There is little in the room to hide yourself behind,
                            but the human is too engrossed in his reading to realize that his latest [i]acquisition[/i] hasn't been restrained.
                        [/p]
                        [p]
                            Halfway to the exit, a gust of wind suddenly blows shut the door through which you came.
                            Tychus's attention is grabbed, but he does not lift his head from his notes.
                            "[color=purple]Back to the break room, Orange.
                            I'll go check up on our [i]guest[/i] after I finish reading.[/color]"
                            Thank goodness, he thinks you're the masseuse!
                        [/p]
                        [p]
                            Without answering his half-minded mumbling, you continue to walk towards the front door.
                            Once you've finally reached the doorway to freedom, the human, sensing something wrong,
                            lifts his head his up, and realizes that you're [b]not[/b] his treacherous pet.
                            "[color=purple]Where the [b]hell[/b] do you think you're going?![/color]"
                            He reaches underneath his desk, and brandishes a steel-tipped whip, but it's already too late.
                            You open the door, and escape into the streets,
                            closing it behind you just in time to catch Tychus' first blow, his battle-whip practically [b]tearing[/b] the door in half!
                        [/p]
                        [p]
                            You sprint away, vanishing into the slums, bolting off in a random direction.
                            A few minutes of rapid jogging later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            That was close.
                            [b]Too[/b] close.
                            You could have ended up a slave, or worse!
                            And all that because you wanted a simple massage.
                            Then again, nothing is ever really [i]simple[/i] for a kobold like yourself.
                            Everything is complicated because of your lack of rights and everyone hating you and all.
                            It's too bad that you couldn't help the other kobolds under Tychus' tyrannical reign,
                            but out here in Varanar, it's every lizard for ${him ? `him` : `her`}self.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your conscience will be more at ease after a good night's rest.
                            And if it's not, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\AfternoonThreeParlorItemGetFlee.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\AfternoonThreeParlorItemGet.js

/**
 * Item get section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorItemGet(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preItemGet;
    var subController = ctrl.itemGet;
    var changeController = ctrl.itemGet;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorItemGetFlee(ctrl);
    var secondNext = new AfternoonThreeParlorItemGetFight(ctrl);

    var firstChoice = AfternoonThreeParlorItemGetChoice.FLEE;
    var secondChoice = AfternoonThreeParlorItemGetChoice.FIGHT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var clothes = !ctrl.misc.nude();
        var weapon = ctrl.misc.weapon();
        var anything = ctrl.misc.hasAnything();
        var tiedUp = ctrl.knockout.get();
        var toReturn = `[p]
                            In the corner of the massage room, you find your discarded effects.
                            ${tiedUp ? `Tychus probably hasn't gotten around to taking what is yours yet.` : ``}
                            You collect your things.
                            Your coin pouch.
                            ${clothes ? `Your clothes, which you put on.` : ``}
                            ${weapon ? `Your ${global.kobold.inventory.createItemWithTooltip(weapon)}.` : ``}
                            ${anything ? `Your belongings.` : ``}
                            ${clothes || weapon || anything ? `And that's pretty much everything.` : `And you didn't really come in with anything else.`}
                        [/p]
                        [p]
                            Geared up once more, you walk over to the door to the lobby.
                            Silently opening it, you observe the human sitting behind his desk, engrossed in notes once more.
                            You can either try to sneak by him towards the exit,
                            or you can attempt to get behind him and [b]attack[/b] him.
                            Whatever you do, you better act fast, before he finishes reading.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Flee</button>`;

        var tooltip = global.parser.parse(`Sneak past the human without him even noticing.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`He deserves a big whallop to the back of the head.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

var AfternoonThreeParlorItemGetChoice = {
    FLEE: 1,
    FIGHT: 2
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\itemget\AfternoonThreeParlorItemGet.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrapSucceed.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseTrapSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.trap;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.itemGet;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            If you play your cards right, you might be able to knock her out [i]before[/i] she can alarm Tychus.
                            Hm, your best course of action would be to lure her towards you, away from the human.
                            Since she'll be coming to you, it might be wise to place a [b]trap[/b].
                        [/p]
                        [p]
                            You grab some of the rope you were tied down with,
                            and re-string it in front of the door, around ankle-height.
                            If Orange comes through here, she'll trip over the wire, and that's when you'll get her good!
                            Your trap in place, you quietly close the door once more.
                            Then, you set your plan in motion, by loudly [b]knocking[/b] on the table,
                            pretending to have recently woken up.
                        [/p]
                        ${global.ui.condSucc(`Intelligence greater than 5`)}
                        [p]
                            A few moments later, the door swings open.
                            "[color=orange]Rise and shine, swee-[/color]"
                            The kobold's sentence is cut short as she trips over your devious trap, falling flat on her face!
                            You take advantage of the situation, rapidly approaching her, and crudely [b]stomping[/b] on the back of Orange's head,
                            [b]crushing[/b] her face between the bottom of your foot and the hard, wooden floor.
                        [/p]
                        [p]
                            A loud [b]crack[/b] resonates throughout the room, and the masseuse's body goes limp.
                            As you remove your foot from her head, you notice a puddle of blood forming underneath her downed frame.
                            Good.
                            Serves her right for trying to [i]capture[/i] you.
                            She knocked you out, and now you've returned the favor.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrapSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrapFail.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseTrapFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuseFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            If you play your cards right, you might be able to knock her out [i]before[/i] she can alarm Tychus.
                            Hm, your best course of action would be to lure her towards you, away from the human.
                            Since she'll be coming to you, it might be wise to place a [b]trap[/b].
                        [/p]
                        [p]
                            You grab some of the rope you were tied down with,
                            and re-string it in front of the door, around ankle-height.
                            If Orange comes through here, she'll trip over the wire, and that's when you'll get her good!
                            Your trap in place, you quietly close the door once more.
                            Then, you set your plan in motion, by loudly [b]knocking[/b] on the table,
                            pretending to have recently woken up.
                        [/p]
                        ${global.ui.condFail(`Intelligence below 6`)}
                        [p]
                            A few moments later, the door opens, but not fully.
                            In a fit of stupidity, you tied the rope to the door itself, blocking it, preventing it from opening all the way.
                            Orange, alerted to your trap, easily steps over the cord you strung.
                            You kind of hadn't planned for this.
                            Dumbstruck, you find nothing better to do than to wave hello to your masseuse.
                        [/p]
                        [p]
                            She doesn't seem to happy to see you.
                            In fact, she punches you straight in the face, knocking you down on your ass.
                            Her punch is quickly followed by a kick to your face,
                            and instead of falling for your trap, the masseuse knocks you out cold.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrapFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrap.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseTrap(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMasseuseChoice.TRAP;
    var reqController = ctrl.masseuse;

    var toFail = new AfternoonThreeParlorMasseuseTrapFail(ctrl);
    var toSuccess = new AfternoonThreeParlorMasseuseTrapSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.intelligence() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\trap\AfternoonThreeParlorMasseuseTrap.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpowerSucceed.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseOverpowerSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.overpower;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.itemGet;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            If you play your cards right, you might be able to knock her out [i]before[/i] she can alarm Tychus.
                            Plus, you kind of want to [b]hurt[/b] the kobold after what she did to you.
                        [/p]
                        [p]
                            It's settled, then.
                            You'll take her down.
                            You creep forwards, sneaking behind Orange.
                            She's finished sweeping, and is now dusting off a few racks.
                            You approach her, unseen and unheard.
                            Then, striking from the shadows, you grab the back of her head, and [b]ram[/b] it forwards,
                            [b]bashing[/b] her forehead into the wall!
                        [/p]
                        ${global.ui.condSucc(`Strength greater than 5`)}
                        [p]
                            The orange-scaled kobold's body instantly goes limp, slumping down to the ground.
                            Her head is bleeding profusely.
                            Good.
                            Serves her right for trying to [i]capture[/i] you.
                            She knocked you out, and now you've returned the favor.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpowerSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpowerFail.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseOverpowerFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuseFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            If you play your cards right, you might be able to knock her out [i]before[/i] she can alarm Tychus.
                            Plus, you kind of want to [b]hurt[/b] the kobold after what she did to you.
                        [/p]
                        [p]
                            It's settled, then.
                            You'll take her down.
                            You creep forwards, sneaking behind Orange.
                            She's finished sweeping, and is now dusting off a few racks.
                            You approach her, unseen and unheard.
                            Then, striking from the shadows, you grab the back of her head, and [b]ram[/b] it forwards,
                            [b]bashing[/b] her forehead into the wall!
                        [/p]
                        ${global.ui.condFail(`Strength below 6`)}
                        [p]
                            Your sudden strike was perhaps not as powerful as you intended for it to be.
                            While you've succeeded in hurting Orange,
                            you didn't quite manage to [i]knock her out[/i].
                            Enraged, the masseuse turns around, and before you even know what's going on,
                            she crudely [b]socks you[/b] in the muzzle, beating you so hard your fall down on your ass.
                            Her punch is quickly followed by a kick to your face,
                            and instead of hers, [b]your[/b] lights are the ones that go out.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpowerFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpower.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseOverpower(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMasseuseChoice.OVERPOWER;
    var reqController = ctrl.masseuse;

    var toFail = new AfternoonThreeParlorMasseuseOverpowerFail(ctrl);
    var toSuccess = new AfternoonThreeParlorMasseuseOverpowerSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.strength() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\overpower\AfternoonThreeParlorMasseuseOverpower.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluffSucceed.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseBluffSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.bluff;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.itemGet;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            But if you [i]surprise[/i] her, she might scream for help.
                            Hm, your best course of action would be to talk her down, but first you need to make your presence clear somehow.
                            You think you have an idea that [i]might[/i] work.
                        [/p]
                        [p]
                            You creep forwards, sneaking behind Orange.
                            She's finished sweeping, and is now dusting off a few racks.
                            You approach her, unseen and unheard.
                            Then, striking from the shadows, you grab her muzzle from behind, silencing her while keeping her in place.
                            She instantly struggles against your grasp.
                            You tell her to stay calm, that you're not going to hurt her.
                            You just want to talk.
                        [/p]
                        ${global.ui.condSucc(`Charisma greater than 5`)}
                        [p]
                            Orange slightly calms down, ceasing her attempts to shake you off.
                            You tell her that you're going to let go of her mouth now, but she has to promise not to scream.
                            Removing your hand from her muzzle, you're glad to see that she [b]doesn't[/b] rat you out to Tychus.
                            At least not right away.
                        [/p]
                        [p] 
                            You ask the masseuse why she knocked you out.
                            What's going on here?
                            Does Tychus treat all of his customers this way?
                            She speaks, in a hushed tone, as to not alert her [i]master[/i].
                            "[color=orange]No, only the kobolds.
                            I was the first.
                            The other three were like you, once.
                            They came in here for a massage, and they never left.[/color]"
                        [/p]
                        [p]
                            But why?
                            You mean, you can see why Tychus would prefer slave labor over actually having to [i]pay[/i] his employees,
                            but why does she play along with all of this?
                            What's in it for her?
                            Orange masterfully breaks your hold on her, swinging you around,
                            turning the entire situation on its head, pressing [b]you[/b] against the wall, pinning your throat with her lower arm.
                            She stares into your eyes with a hateful glare.
                        [/p]
                        [p]
                            While obviously upset, her voice remains somewhat even and level.
                            "[color=orange]Do you think I [b]want[/b] to be a part of this?
                            He was going to [b]kill me[/b] if I didn't do his bidding.
                            Heck, he'll kill us [b]both[/b] if he sees me talking to you.[/color]"
                            Orange lets go of your throat, and takes a few steps back.
                            Tears start to form at the corners of her hate-filled eyes.
                        [/p]
                        [p]
                            "[color=orange]I'm sorry, for what it's worth.
                            If not me, he'd have bought another slave to do his dirty work.
                            I couldn't stop it, not even by letting him kill me.
                            So I did the only thing I could.
                            I served.[/color]"
                        [/p]
                        [p]
                            You say you understand.
                            She did what she had to.
                            But you can't stay here.
                            And neither can she.
                            She needs to come with you.
                            You'll escape, together.
                            Maybe you can even come back for the others, later.
                            You extend your hand, hoping she'll take it, hoping she'll see reason.
                        [/p]
                        [p]
                            But she doesn't.
                            Orange shakes her head no.
                            Her tears drop to the ground.
                            She slowly starts to back away, towards the room you emerged from.
                            "[color=orange]It's too late for me.
                            The things I've done can [b]never[/b] be forgiven.
                            I've ruined more lives than my own.
                            I can't live with the guilt.
                            Not anymore.[/color]"
                        [/p]
                        [p]
                            You tell her that it'll be okay.
                            [b]You[/b] forgive her.
                            You're sure the others will, too.
                            You can help her.
                            It's not too late for her to redeem herself!
                            It's [b]never[/b] too late!
                            But your words are wasted.
                            She doesn't listen.
                            She's reached the entrance to the backroom.
                            She slips inside, but doesn't close the door quite yet.
                        [/p]
                        [p]
                            Orange gives you a few final words of advice:
                            "[color=orange]Run.
                            Run and [b]never[/b] look back.
                            Save yourself.
                            There's nothing left worth fighting for here.
                            I'm sorry for everything.
                            I won't be a burden anymore.
                            Thank you.
                            Goodbye.[/color]"
                        [/p]
                        [p]
                            With those words said, she closes the door, audibly locking it behind her.
                            And that is the last you ever see of Orange.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluffSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluffFail.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseBluffFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuseFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            She hasn't noticed you yet.
                            That gives you the element of surprise.
                            But if you [i]surprise[/i] her, she might scream for help.
                            Hm, your best course of action would be to talk her down, but first you need to make your presence clear somehow.
                            You think you have an idea that [i]might[/i] work.
                        [/p]
                        [p]
                            You creep forwards, sneaking behind Orange.
                            She's finished sweeping, and is now dusting off a few racks.
                            You approach her, unseen and unheard.
                            Then, striking from the shadows, you grab her muzzle from behind, silencing her while keeping her in place.
                            She instantly struggles against your grasp.
                            You tell her to stay calm, that you're not going to hurt her.
                            You just want to talk.
                        [/p]
                        ${global.ui.condFail(`Charisma below 6`)}
                        [p]
                            Orange is too startled to settle down.
                            She crudely [b]bites[/b] your hand, throwing off your hold on her.
                            The kobold turns around at an incredible pace.
                            Catching you off-guard, she violently [b]punches[/b] your muzzle, hard enough to knock you down on your ass.
                            Her punch is quickly followed by a kick to your face,
                            and instead of talking to you, the masseuse knocks you out cold.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluffFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluff.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuseBluff(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMasseuseChoice.BLUFF;
    var reqController = ctrl.masseuse;

    var toFail = new AfternoonThreeParlorMasseuseBluffFail(ctrl);
    var toSuccess = new AfternoonThreeParlorMasseuseBluffSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.charisma() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\bluff\AfternoonThreeParlorMasseuseBluff.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\AfternoonThreeParlorMasseuse.js

/**
 * Masseuse section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMasseuse(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preMasseuse;
    var subController = ctrl.masseuse;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new AfternoonThreeParlorMasseuseBluff(ctrl);
    var secondNext = new AfternoonThreeParlorMasseuseTrap(ctrl);
    var thirdNext = new AfternoonThreeParlorMasseuseOverpower(ctrl);

    var firstChoice = AfternoonThreeParlorMasseuseChoice.BLUFF;
    var secondChoice = AfternoonThreeParlorMasseuseChoice.TRAP;
    var thirdChoice = AfternoonThreeParlorMasseuseChoice.OVERPOWER;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = ``;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Reason</button>`;

        var tooltip = global.parser.parse(`Use your words. A fellow kobold might listen.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Trap</button>`;

        var tooltip = global.parser.parse(`Quickly set a trap for the woman.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 135px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Overpower</button>`;

        var tooltip = global.parser.parse(`Sometimes the best approach is the most direct.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 195px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

var AfternoonThreeParlorMasseuseChoice = {
    BLUFF: 1,
    TRAP: 2,
    OVERPOWER: 3
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\masseuse\AfternoonThreeParlorMasseuse.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopesSucceed.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorCutRopesSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preMasseuse;
    var changeController = ctrl.cut;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuse;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            These ropes don't look so tough.
                            You reckon you can cut them using your sharp claws.
                            Wriggling around your hand, you inch your restraints upwards ever so slightly.
                            Now if only you can reach your wrist with your claws, you'll be able to slice the rope and get out of here!
                        [/p]
                        ${global.ui.condSucc(`Agility greater than 5`)}
                        [p]
                            You expertly curl your wrist,
                            angling it in such a way that the claw at the end of the longest of your four fingers can reach the rope, with half an inch to spare!
                            Perfect!
                            Your superior agility saves the day once more.
                            But you're not out of the woods yet.
                            You still need to [i]sever[/i] the rope.
                        [/p]
                        ${global.ui.condSucc(`Strength greater than 5`)}
                        [p]
                            You frantically begin to grind your sharp claw against the cord,
                            but this kind of rope in particular seems a lot more [b]firm[/b] than the regular stuff.
                            Like the kind used at the riverside docks, to tie boats to the piers.
                            While at first, your claw seems ineffective against the thick fibers of the strong rope,
                            after about a minute of trying, you appear to have made a gash!
                        [/p]
                        [p]
                            Perfect!
                            You keep on cutting until you reach about halfway through the rope, several strands breaking off.
                            With the cord weakend, you stop slicing, and instead firmly [b]yank[/b] it,
                            the rope breaking off around where you slice it, freeing your hand!
                        [/p]
                        [p]
                            With one arm free, you have no trouble escaping from the rest of your bondage.
                            You rid yourself of the last bit of rope still dangling off your wrist, and sneak over to one of the two doors.
                            Opening it, you discover that the room you were in was adjacent to the massage room.
                            Orange is tidying up the place, sweeping the floor, probably removing any trace that were ever even here.
                            As if anyone would come looking for [i]you[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopesSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopesFail.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorCutRopesFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.ropeFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var agiReq = ctrl.postMassageMisc.agility() > 5;
        var toReturn = `[p]
                            These ropes don't look so tough.
                            You reckon you can cut them using your sharp claws.
                            Wriggling around your hand, you inch your restraints upwards ever so slightly.
                            Now if only you can reach your wrist with your claws, you'll be able to slice the rope and get out of here!
                        [/p]`;

        if (!agiReq) {
            toReturn += `${global.ui.condFail(`Agility below 6`)}
                        [p]
                            You try and you try, but not matter how hard you strain your wrist,
                            you fail to get your claws anywhere [b]near[/b] the rope!
                            If only your fingers were a little bit more nimble, you'd easily be able to reach!
                            After a good five minutes of trying, an intense pain builds up in your stressed joints.
                            You decide to cease your attempts, for now, lest you pop a strained muscle or something.
                        [/p]`;
        } else {
            toReturn += `${global.ui.condSucc(`Agility greater than 5`)}
                        [p]
                            You expertly curl your wrist,
                            angling it in such a way that the claw at the end of the longest of your four fingers can reach the rope, with half an inch to spare!
                            Perfect!
                            Your superior agility saves the day once more.
                            But you're not out of the woods yet.
                            You still need to [i]sever[/i] the rope.
                        [/p]
                        ${global.ui.condFail(`Strength below 6`)}
                        [p]
                            You frantically begin to grind your sharp claw against the cord,
                            but this kind of rope in particular seems a lot more [b]firm[/b] than the regular stuff.
                            Like the kind used at the riverside docks, to tie boats to the piers.
                            Your claw, while sharp, isn't nearly sharp enough to cut through the thick fibers of the strong rope.
                            Perhaps if you had a little bit more elbow grease to back up your fingerwork, you'd book more progress.
                        [/p]
                        [p]
                            After a good five minutes of trying, you decide to relent.
                            The rope's not even scuffed.
                            Looks like you'll have to find a different way to get out of here.
                        [/p]`;
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopesFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopes.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorCutRopes(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorKnockoutAwakenChoice.CUT;
    var reqController = ctrl.knockout;

    var toFail = new AfternoonThreeParlorCutRopesFail(ctrl);
    var toSuccess = new AfternoonThreeParlorCutRopesSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.strength() > 5 && ctrl.postMassageMisc.agility() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\cut\AfternoonThreeParlorCutRopes.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopesSucceed.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorEscapeRopesSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preMasseuse;
    var changeController = ctrl.escape;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuse;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var footMassage = global.kobold.history.shared.varanar.dayThree.afternoon.parlorFootMassage;
        var toReturn = `[p]
                            These ropes aren't so tightly tied.
                            You reckon you can slip free without needing to pry the cord itself apart!
                            Wriggling your wrist about, you begin to try to rid yourself of your bindings.
                        [/p]
                        ${global.ui.condSucc(`Agility greater than 6`)}
                        [p]
                            You try tugging, twisting, pulling, sliding and contorting your hand in all manners possible,
                            but you simply can not escape the thick rope binding your wrists.
                            It's too tight around your scales,
                            and the coarse fabric the rope is made of kind of [i]hurts[/i] whenever you attempt to escape it.
                            Like rope burn, but a little [b]worse[/b].
                        [/p]
                        [p]
                            Then you notice the rope around your left foot is a little slack.
                            Perhaps you've been going about this all wrong.
                            It isn't your hand you should try to wriggle free, but your foot!
                            You begin to contort your ${footMassage ? `freshly massaged` : ``} scaly trotter,
                            and after some near-dislocating feats of flexibility, you manage to slip your foot free from its bondage!
                        [/p]
                        [p]
                            One foot free, three limbs to go.
                            First, you free your other foot.
                            Untying the knot using your toes is an easy task for a thief as skilled as you are.
                            Then, using your superior agility, you arch your back, curling up and maneuvering your foot to your head,
                            before angling slightly to the side, bringing your clawed toes to bear on the rope holding your hand.
                            After freeing one of your hands, untying the other is child's play.
                        [/p]
                        [p]
                            You rid yourself of the last piece of rope dangling off your wrist, and sneak over to one of the two doors.
                            Opening it, you discover that the room you were in was adjacent to the massage room.
                            Orange is tidying up the place, sweeping the floor, probably removing any trace that were ever even here.
                            As if anyone would come looking for [i]you[/i].
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopesSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopesFail.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorEscapeRopesFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.ropeFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            These ropes aren't so tightly tied.
                            You reckon you can slip free without needing to pry the cord itself apart!
                            Wriggling your wrist about, you begin to try to rid yourself of your bindings.
                        [/p]
                        ${global.ui.condFail(`Agility below 7`)}
                        [p]
                            You try tugging, twisting, pulling, sliding and contorting your hand in all manners possible,
                            but you simply can not escape the thick rope binding your wrists.
                            It's too tight around your scales,
                            and the coarse fabric the rope is made of kind of [i]hurts[/i] whenever you attempt to escape it.
                            Like rope burn, but a little [b]worse[/b].
                        [/p]
                        [p]
                            For five minutes straight, you keep on trying, until your scales run red from chafing.
                            Feeling like you'll draw blood if you continue to wriggle about, you decide to halt your efforts, for now.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopesFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopes.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorEscapeRopes(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorKnockoutAwakenChoice.SLIP;
    var reqController = ctrl.knockout;

    var toFail = new AfternoonThreeParlorEscapeRopesFail(ctrl);
    var toSuccess = new AfternoonThreeParlorEscapeRopesSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.agility() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\escape\AfternoonThreeParlorEscapeRopes.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopesSucceed.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBreakRopesSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preMasseuse;
    var changeController = ctrl.break;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.masseuse;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var armMassage = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var toReturn = `[p]
                            These ropes don't look so tough.
                            You reckon you can break them through sheer strength alone!
                            Tensing up your ${armMassage ? `massage-strengthened` : ``} arm-muscles,
                            you begin to apply a constant, steady tug on your restraints,
                            attempting to tear the ropes apart, to make them unravel.
                        [/p]
                        ${global.ui.condSucc(`Strength greater than 6`)}
                        [p]
                            You can feel the cord beginning to weaken, several fibers breaking off.
                            Straining your arms, you pump your body to its very limit,
                            letting out a low, quite masculine growl as you up your efforts one more time.
                            To your surprise, it is not the rope that breaks first,
                            but the table-leg it's tied to!
                            With a loud [b]crack[/b], the piece of wood breaks off,
                            the entire table falling down onto one side, leaving you hanging vertically off three of your still-bound limbs.
                        [/p]
                        [p]
                            With one hand free, you have no trouble escaping from the rest of your bondage.
                            Undoing the knot around your second hand, you are set free.
                            You rid yourself of the table-leg dangling off your wrist, and sneak over to one of the two doors.
                            Opening it, you discover that the room you were in was adjacent to the massage room.
                            Orange is tidying up the place, sweeping the floor, probably removing any trace that were ever even here.
                            As if anyone would come looking for [i]you[/i].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopesSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopesFail.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBreakRopesFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.ropeFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var armMassage = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var toReturn = `[p]
                            These ropes don't look so tough.
                            You reckon you can break them through sheer strength alone!
                            Tensing up your ${armMassage ? `massage-strengthened` : ``} arm-muscles,
                            you begin to apply a constant, steady tug on your restraints,
                            attempting to tear the ropes apart, to make them unravel.
                        [/p]
                        ${global.ui.condFail(`Strength below 7`)}
                        [p]
                            Try and struggle as you may, the ropes remain unyielding.
                            You exert yourself trying to break them, channeling every bit of energy that you have into your arms,
                            trying so hard you nearly pass out once again.
                            After nearly five minutes of strenuous tugging, you come to an exhausted, sweaty rest.
                            This isn't going to work.
                            ${armMassage ? `Not even the intensive arm-massage you received could have prepared you for a situation like this.
                            ` : `Oh how you wish you'd taken an arm massage instead!`}
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopesFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopes.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBreakRopes(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorKnockoutAwakenChoice.BREAK;
    var reqController = ctrl.knockout;

    var toFail = new AfternoonThreeParlorBreakRopesFail(ctrl);
    var toSuccess = new AfternoonThreeParlorBreakRopesSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.strength() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\break\AfternoonThreeParlorBreakRopes.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\AfternoonThreeParlorKnockoutAwaken.js

/**
 * Knockout section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorKnockoutAwaken(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preKnockout;
    var subController = ctrl.knockout;
    var changeController = ctrl.knockout;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new AfternoonThreeParlorCutRopes(ctrl);
    var secondNext = new AfternoonThreeParlorBreakRopes(ctrl);
    var thirdNext = new AfternoonThreeParlorEscapeRopes(ctrl);

    var firstChoice = AfternoonThreeParlorKnockoutAwakenChoice.CUT;
    var secondChoice = AfternoonThreeParlorKnockoutAwakenChoice.BREAK;
    var thirdChoice = AfternoonThreeParlorKnockoutAwakenChoice.SLIP;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You wake up a bit later, in a much different place than the massage room you went to sleep in.
                            A small, dark and dusty room, with no windows and two closed doors.
                            Your nap was a somewhat pleasant one,
                            but the joys of a small bit of afternoon rest are nothing compared to the panic that settles in when you realize you can't move!
                            Your limbs have been spread-eagle, rope-tied to the legs of the table you're on,
                            which appears to be the same massage table you passed out on,
                            although your comfortable pillow has been taken from you.
                        [/p]
                        [p]
                            You've also somehow been flipped over, now laying on your back.
                            You wonder whether you did that by tossing around in your sleep, or if whoever brought you here did.
                            Come to think of it, your nap was a somewhat [i]unnatural[/i] one.
                            You [i]did[/i] go all night without getting any rest, but you weren't tired enough to just [i]pass out[/i] like that.
                            It must have been that thing the masseuse did with her fingers.
                            You knew you shouldn't have trusted this place!
                        [/p]
                        [p]
                            Whatever Tychus and Orange have planned for you, you doubt it's any good.
                            You need to get out of here.
                            First things first, you have to find a way to get these restraints off!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Cut them</button>`;

        var tooltip = global.parser.parse(`Attempt to sever the ropes using your claws.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Break them</button>`;

        var tooltip = global.parser.parse(`Attempt to snap the rope using sheer force.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Escape them</button>`;

        var tooltip = global.parser.parse(`Attempt to slip from your bindings without breaking them.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_shop");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

var AfternoonThreeParlorKnockoutAwakenChoice = {
    CUT: 1,
    BREAK: 2,
    SLIP: 3
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\knockout\AfternoonThreeParlorKnockoutAwaken.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageKnockout.js

/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageKnockout(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preKnockout;
    var changeController = ctrl.preKnockout;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new AfternoonThreeParlorKnockoutAwaken(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var feet = ctrl.massage.get() === AfternoonThreeParlorMassageChoice.FEET;
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorKnockedOut = true;
        var toReturn = `[p]
                            As you continue to bask in the afterglow of the massage,
                            the masseuse ${feet ? `climbs up on the table, and` : ``} places her hand on the back of your neck.
                            A little extra aftercare?
                            You're not complaining.
                        [/p]
                        ${global.ui.condFail(`Charisma below 7`)}
                        ${global.ui.condFail(`Intelligence below 7`)}
                        [p]
                            She presses two of her fingers against your scales, and begins to apply pressure.
                            You feel your limbs slowly growing more and more heavy.
                            Must be some kind of relaxation technique.
                            It feels kind of nice.
                        [/p]
                        [p]
                            Your eyelids, too, are becoming heavy.
                            You close your eyes, yawning loudly.
                            You're tired, oh so tired.
                            Probably because of the massage.
                            You think you need a nap.
                            Yes, it's time to sleep.
                            Sinking deeper into your pillow, your mind drifts into unconsciousness.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        global.ui.applyTheme("varanar_indoor_knockout");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageKnockout.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFriend.js

/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageFriend(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.friend;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var feet = ctrl.massage.get() === AfternoonThreeParlorMassageChoice.FEET;
        var toReturn = `[p]
                            As you continue to bask in the afterglow of the massage,
                            the masseuse ${feet ? `climbs up on the table, and` : ``} places her hand on the back of your neck.
                            A little extra aftercare?
                            You're not complaining.
                        [/p]
                        ${global.ui.condSucc(`Charisma greater than 6`)}
                        [p]
                            She presses two of her fingers against your scales, but then abruptly pulls back.
                            Thinking something's the matter, you cast a glance back at her.
                            She looks conflicted, and kind of sad, like she's struggling to make a tough decision.
                            As you kindly ask what the matter is, she bursts out into tears.
                        [/p]
                        [p]
                            "[color=orange]I can't do it! I [b]can't[/b]![/color]"
                            Rolling onto your back, and coming to a seat on the table, you tell the masseuse to calm down.
                            What can't she do?
                            Did you do something wrong?
                            "[color=orange]You don't understand!
                            The others...I was the first.
                            He bought me off a slaver.
                            Trained me to do [b]horrible[/b] things.
                            I-I--[/color]"
                            She tears up once more.
                        [/p]
                        [p]
                            You place a hand on Orange's shoulder, and tell her it's okay.
                            She lets out a snicker, and then takes a deep breath.
                            "[color=orange]The other three, they were just like you.
                            They came in here for a massage.
                            Then I knocked them out, like Tychus told me to.
                            But I can't do it.
                            Not anymore.[/color]"
                        [/p]
                        [p]
                            That [b]bastard[/b].
                            You never really liked him.
                            "[color=orange]You have to go.
                            Take your things and leave.
                            Don't worry about me.
                            I died a long time ago, at least on the inside.[/color]"
                        [/p]
                        [p]
                            Before you manage to comfort the kobold further, she hops off the table, and bolts towards the back room.
                            Opening the door, Orange utters a few final words.
                            "[color=orange]Good luck. And thank you.[/color]"
                            Then, she steps inside, closing the door behind her, audibly locking it from the other side.
                            And that is the last you ever see of her.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFriend.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFoe.js

/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageFoe(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preItemGet;
    var changeController = ctrl.foe;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var feet = ctrl.massage.get() === AfternoonThreeParlorMassageChoice.FEET;
        var knowsSeki = global.kobold.history.shared.seki.knowsDying;
        var toReturn = `[p]
                            As you continue to bask in the afterglow of the massage,
                            the masseuse ${feet ? `climbs up on the table, and` : ``} places her hand on the back of your neck.
                            A little extra aftercare?
                            You're not complaining.
                        [/p]
                        ${global.ui.condFail(`Charisma below 7`)}
                        ${global.ui.condSucc(`Intelligence greater than 6`)}
                        [p]
                            She presses two of her fingers against your scales, and begins to apply pressure.
                            Wait a minute.
                            You know this grip.
                            The Serpent King's Sedative,
                            named after its usage in the famous assassination of the previous dragon emperor.
                            Seki${knowsSeki ? `, your old alchemist fence and friend,` : ``} told you about it!
                            He showed you once, even.
                        [/p]
                        [p]
                            Your masseuse is trying to knock you out!
                            You can already feel your limbs growing heavy.
                            But it's not too late!
                            Using your hands, you crudely [b]shove[/b] Orange off the table.
                            She clearly was not expecting any resistance.
                            Taken aback, you easily, manage to force her off.
                        [/p]
                        [p]
                            Instead of taking you by surprise and knocking you out, your masseuse is the one who ends up passing out.
                            She falls backwards off the table with a surprised yelp, landing square on her head with a loud [b]crack[/b].
                            While the table isn't particularly high, her landing was [b]not[/b] a gentle one.
                            The kobold is knocked out cold.
                            Blood oozes from her cracked head.
                            She's still breathing, but she's going to have one [b]hell[/b] of a headache when she wakes up.
                            [i]If[/i] she wakes up.
                        [/p]
                        [p]
                            Good riddance!
                            She tried to disable you.
                            But [b]why[/b]?
                            Something clicks inside of your head.
                            All of the pieces finally fit together.
                            But of course!
                            How could you have missed this?
                            You [b]knew[/b] there was something wrong about this place!
                        [/p]
                        [p]
                            Those kobolds you saw earlier.
                            The red one, the green one and the blue one.
                            They must be [b]slaves[/b].
                            [b]Forced[/b] to work for Tychus.
                            Orange was trying to knock you out so you could be added to the [i]collection[/i].
                        [/p]
                        [p]
                            How [b]terrible[/b]!
                            You have to get out of here.
                            If you can save the others in the process, all the better.
                            But your survival is paramount.
                            If just to tell the tale of what happened here.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFoe.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFail.js

/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassageFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.postMassage;

    var toFail = new AfternoonThreeParlorPostMassageKnockout(ctrl);
    var toSuccess = new AfternoonThreeParlorPostMassageFoe(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.intelligence() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassageFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassage.js

/**
 * Post massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorPostMassage(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.postMassage;

    var toFail = new AfternoonThreeParlorPostMassageFail(ctrl);
    var toSuccess = new AfternoonThreeParlorPostMassageFriend(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.postMassageMisc.charisma() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\post\AfternoonThreeParlorPostMassage.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorBack.js

/**
 * Massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBack(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMassageChoice.BACK;
    var reqController = ctrl.massage;
    var subController = ctrl.postMassage;
    var changeController = ctrl.back;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage = true;
        var startBoy = global.kobold.history.scenes["Introduction"].gender;
        var bargainSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var free = ctrl.entrance.get() === AfternoonThreeParlorEntranceChoice.DECLINE || ctrl.bargain.get() === AfternoonThreeParlorEntranceChoice.DECLINE;
        var toReturn = `[p]
                            A back massage might be nice.
                            You're always carrying around a bunch of stuff and lifting heavy things and crouching to stay hidden.
                            All of this has lead to a constant, dull pain building up in your lower back,
                            which makes sleeping a little less comfortable than it used to be.
                            If Orange could fix or [i]alleviate[/i] those pains a little, you'd love her forevermore,
                            and you'd undoubtedly come back for a second massage after you've scrounged together enough money.
                        [/p]
                        [p]
                            You don't know enough about massaging to know whether or not a simple rubdown [i]can[/i] fix your ailments,
                            but instead of confirming or denying, the orange-scaled kobold simply gestures towards the nearest massage table,
                            and gives you a small wink, as if wordlessly saying to let her work her magic.
                        [/p]
                        [p]
                            You eagerly clamber ontop of the wooden table, and lay yourself down ontop of it, belly-down.
                            This thing was obviously built for species [b]far[/b] bigger than you are.
                            You scoot upwards, until your head ends up resting atop a comfortable pillow at the far end of the wooden surface.
                            Orange climbs up as well, with a bit more tact, style and grace than you.
                            She [i]is[/i] a professional, after all.
                            Or maybe she's just done this enough to know how to get ontop of the table without looking like an idiot.
                        [/p]
                        [p]
                            After coating her hands in massage oil,
                            your masseuse drops to her knees, and begins the massage by placing her hands on your upper back,
                            where your neck and shoulders meet.
                            Before starting to rub you proper, she takes a moment to feel around with her thumbs,
                            looking for knots and kinks she needs to focus on.
                            Your back has [b]lots[/b] of those.
                            Heck, it's practically one big kink wrapped in a knot.
                        [/p]
                        [p]
                            The masseuse does not understand your half-assed attempt at a joke.
                            Unbeknownst to you, she rolls her eyes, concentrating on giving you a proper massage.
                            And that she does.
                            She presses her soft, slickened hands against your back, and begins to rub around in small, circular motions,
                            pleasantly grinding her palms into your scales, reaching every nook and cranny, almost making you [b]squeal[/b] in pure ecstacy.
                            It feels [b]so[/b] good!
                            It's like all your worries and pain are [b]melting[/b] under her soft and gentle, yet surprisingly firm touch.
                        [/p]
                        [p]
                            And she hasn't gotten to the good parts yet.
                            Her hands slowly drift lower and lower, getting to the root of the problem,
                            straightening your back with a firm push before starting to [b]squeeze[/b] your flesh, kneading it in an almost [i]sexual[/i] manner.
                            You let out a relaxed sigh.
                            It feels as though a huge weight is being lifted off your back.
                            For a moment, the treatment stops, but after applying a bit more oil, it suddenly continues,
                            sending another jolt of near-orgasmic goodness coursing down your spine.
                        [/p]
                        [p]
                            If you'd known getting a massage felt [b]this[/b] incredible, you'd have
                            ${bargainSuccess || free ? `gladly paid full price` : `coughed up fourty coins a lot quicker`}.
                            The sensation of relief only grows as the massage continues, your entire backside receiving the same treatment,
                            from right above your butt to your very shoulders.
                            You closer your eyes, and slowly drift away in pleasure.
                        [/p]
                        [p]
                            "[color=orange]Time's up![/color]" comes Orange's heavenly voice, far earlier than you would have expected it.
                            You must have dozed off a little, or maybe time really does go by faster when you're having fun.
                            Either way, your back hasn't felt this good since you were a little ${startBoy ? `boy` : `girl`}.
                            It really does feel like you were born anew.
                        [/p]
                        [p]
                            You run one of your own hands over your oily back.
                            With your muscles loosened, your frame feels more supple, yet also more [b]firm[/b],
                            like you can take a beating and keep on slugging.
                            Like your loosened scales will dampen blows.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorBack.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorFeet.js

/**
 * Massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFeet(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMassageChoice.FEET;
    var reqController = ctrl.massage;
    var subController = ctrl.postMassage;
    var changeController = ctrl.feet;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFootMassage = true;
        var startBoy = global.kobold.history.scenes["Introduction"].gender;
        var bargainSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var free = ctrl.entrance.get() === AfternoonThreeParlorEntranceChoice.DECLINE || ctrl.bargain.get() === AfternoonThreeParlorEntranceChoice.DECLINE;
        var toReturn = `[p]
                            A foot massage is what you want.
                            You've spent your entire life walking around on your bare feet, not to mention climbing stuff.
                            All of this has lead to a constant, dull pain building up in your ankles,
                            which makes running and sprinting a little more [i]intensive[/i] than it used to be.
                            If Orange could fix or [i]alleviate[/i] those pains a little, you'd love her forevermore,
                            and you'd undoubtedly come back for a second massage after you've scrounged together enough money.
                        [/p]
                        [p]
                            You don't know enough about massaging to know whether or not a simple rubdown [i]can[/i] fix your ailments,
                            but instead of confirming or denying, the orange-scaled kobold simply gestures towards the nearest massage table,
                            and gives you a small wink, as if wordlessly saying to let her work her magic.
                        [/p]
                        [p]
                            You eagerly clamber ontop of the wooden table, and lay yourself down ontop of it, belly-down,
                            your feet sticking out from the edge of the table.
                            This thing was obviously built for species [b]far[/b] bigger than you are.
                            You reach upwards, grabbing the present pillow and tugging it towards you so you can comfortably rest your head atop of it.
                        [/p]
                        [p]
                            After coating her hands in massage oil,
                            your masseuse walks over to your feet,
                            and begins the massage by rubbing your soles up and down a little,
                            slathering them in clear, lubricating liquid.
                            Pleasant, but not overly so.
                            Then again, the real massage hasn't even begun.
                        [/p]
                        [p]
                            Speak of the devil, Orange presses both of her thumbs into the sole of your right foot,
                            rubbing them around in circles, pleasantly [b]digging[/b] into your flesh.
                            It feels [b]divine[/b]!
                            Like all the stress and fatigue built up from [b]years[/b] of walking and running is simply [b]melting[/b] away.
                            And the kobold is only getting started.
                        [/p]
                        [p]
                            Grabbing your foot by the ankle, one hand on either side,
                            the masseuse begins to twist it by rubbing up and down in opposite directions.
                            You let out an almost [i]sexual[/i] moan, sheer bliss flooding your mind.
                            Absolutely [b]amazing[/b]!
                        [/p]
                        [p]
                            Your heel is next.
                            She firmly presses her slickened palm against it,
                            rubbing it from side to side for a moment,
                            before stroking alongside the length of your foot,
                            her palm [b]burying[/b] into your scales as it travels all the way to the base of your clawed toes.
                            It feels [b]so[/b] good!
                        [/p]
                        [p]
                            Orange proceeds to grinds her fist into your sole,
                            loosening your muscles, rejuvinating your scales and sending spikes of sheer pleasure coursing up your spine.
                            Each of your toes gets squeezed, tugged, pulled, rubbed, cracked and stretched,
                            the top of your foot gets some care and attention,
                            and the masseuse even thinks to reach in between your toes to complete her work.
                        [/p]
                        [p]
                            Your face sinks deeper into the pillow you're laying on, as Orange moves on to service your [i]other[/i] foot.
                            If you'd known getting a massage felt [b]this[/b] incredible, you'd have
                            ${bargainSuccess || free ? `gladly paid full price` : `coughed up fourty coins a lot quicker`}.
                            Soothed by the near-orgasmic goodness coursing down your spine, you close your eyes, and slowly drift away in pleasure.
                        [/p]
                        [p]
                            "[color=orange]Time's up![/color]" comes Orange's heavenly voice, far earlier than you would have expected it.
                            You must have dozed off a little, or maybe time really does go by faster when you're having fun.
                            Either way, your feet, and particularly your soles haven't felt this good since you were a little ${startBoy ? `boy` : `girl`}.
                            It really does feel like you were born anew.
                        [/p]
                        [p]
                            You flex and curl your toes.
                            With your muscles loosened, and your flesh made more supple, your feel a lot more confident in your ability to run and jump.
                            Plus your softened soles might make you a teensy bit stealthier.
                            The massage has definitely made you more [i]loose[/i], flexible and agile, or at least your feet are.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorFeet.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorArms.js

/**
 * Massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorArms(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorMassageChoice.ARMS;
    var reqController = ctrl.massage;
    var subController = ctrl.postMassage;
    var changeController = ctrl.arms;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.postMassage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage = true;
        var startBoy = global.kobold.history.scenes["Introduction"].gender;
        var bargainSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var free = ctrl.entrance.get() === AfternoonThreeParlorEntranceChoice.DECLINE || ctrl.bargain.get() === AfternoonThreeParlorEntranceChoice.DECLINE;
        var toReturn = `[p]
                            An arm massage might be nice.
                            You're always carrying around a bunch of stuff and lifting heavy things and climbing stuff.
                            All of this has lead to a constant, dull pain building up in your lower arms,
                            which makes doing anything with your hands a little more frustrating than it used to be.
                            If Orange could fix or [i]alleviate[/i] those pains a little, you'd love her forevermore,
                            and you'd undoubtedly come back for a second massage after you've scrounged together enough money.
                        [/p]
                        [p]
                            You don't know enough about massaging to know whether or not a simple rubdown [i]can[/i] fix your ailments,
                            but instead of confirming or denying, the orange-scaled kobold simply gestures towards the nearest massage table,
                            and gives you a small wink, as if wordlessly saying to let her work her magic.
                        [/p]
                        [p]
                            You eagerly clamber ontop of the wooden table, and lay yourself down ontop of it, belly-down.
                            This thing was obviously built for species [b]far[/b] bigger than you are.
                            You scoot upwards, until your head ends up resting atop a soft pillow at the far end of the wooden surface.
                            Once comfortable, you spread eagle, giving your massseuse plenty of access to your arms.
                            Orange climbs up as well, with a bit more tact, style and grace than you.
                            She [i]is[/i] a professional, after all.
                            Or maybe she's just done this enough to know how to get ontop of the table without looking like an idiot.
                        [/p]
                        [p]
                            After coating her hands in massage oil,
                            your masseuse drops to her knees,
                            and begins the massage by smoothly stroking both your arms from your wrist all the way up to your shoulders,
                            slathering your limbs in clear, lubricating liquid.
                            Pleasant, but not overly so.
                            Then again, the real massage hasn't even begun.
                        [/p]
                        [p]
                            Speak of the devil, Orange takes hold of one of your upper arms, and begins to [b]knead[/b] it,
                            pleasantly digging her soft, oil-coated palms into your muscles, loosening and relaxing them.
                            It feels [b]divine[/b]!
                            Like all the stress and fatigue built up from all the work you've ever done is melting away.
                            And the kobold is only getting started.
                        [/p]
                        [p]
                            Your lower arm is next.
                            Orange presses her slickened hands against your scales, and begins to rub around in small, circular motions,
                            grinding her palms into your flesh, almost making you [b]squeal[/b] in pure ecstacy.
                            It feels [b]so[/b] good!
                            It's like all your worries and pain are [b]washed away[/b] under the tide of massage oil slathered about by
                            her soft and gentle, yet surprisingly firm touch.
                        [/p]
                        [p]
                            The dull pain you once felt fades away, replaced by sheer bliss.
                            Your masseuse rises to her feet, placing one foot on your back to hold you in place,
                            while [b]tugging[/b] your arm towards her.
                            A small [i]crack[/i] resonates throughout the room, followed by an almost [i]sexual[/i] moan.
                            Absolutely [b]amazing[/b]!
                        [/p]
                        [p]
                            Your face sinks deeper into the pillow you're laying on, as Orange proceeds to service your [i]other[/i] arm.
                            If you'd known getting a massage felt [b]this[/b] incredible, you'd have
                            ${bargainSuccess || free ? `gladly paid full price` : `coughed up fourty coins a lot quicker`}.
                            Soothed by the near-orgasmic goodness coursing down your spine, you close your eyes, and slowly drift away in pleasure.
                        [/p]
                        [p]
                            "[color=orange]Time's up![/color]" comes Orange's heavenly voice, far earlier than you would have expected it.
                            You must have dozed off a little, or maybe time really does go by faster when you're having fun.
                            Either way, your arms haven't felt this good since you were a little ${startBoy ? `boy` : `girl`}.
                            It really does feel like you were born anew.
                        [/p]
                        [p]
                            You run one of your own hands over your oily bi- and triceps.
                            With your muscles loosened, your arms feels more supple, yet also [b]stronger[/b].
                            Like the massage has renewed your strength,
                            or perhaps brought out some of that [i]inner power[/i] priests and clerics always rant on about.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorArms.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorMassage.js

/**
 * Massage section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorMassage(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preMassage;
    var subController = ctrl.massage;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new AfternoonThreeParlorArms(ctrl);
    var secondNext = new AfternoonThreeParlorBack(ctrl);
    var thirdNext = new AfternoonThreeParlorFeet(ctrl);

    var firstChoice = AfternoonThreeParlorMassageChoice.ARMS;
    var secondChoice = AfternoonThreeParlorMassageChoice.BACK;
    var thirdChoice = AfternoonThreeParlorMassageChoice.FEET;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorMassaged = true;
        var toReturn = ``;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Arms</button>`;

        var tooltip = global.parser.parse(`You've never had your arms rubbed before.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Back</button>`;

        var tooltip = global.parser.parse(`You have [b]a lot[/b] of tension in your lower back.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Feet</button>`;

        var tooltip = global.parser.parse(`A foot massage might be nice.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 135px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You choose:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

var AfternoonThreeParlorMassageChoice = {
    ARMS: 1,
    BACK: 2,
    FEET: 3
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\massage\AfternoonThreeParlorMassage.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightLoot.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFightLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.fight;
    var changeController = ctrl.fightLoot;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            After a minute of indulging in the rare feeling of [b]pride[/b],
                            your mind settles down from the adrenaline rush that fueled your combat prowess.
                            You did it!
                            You beat the insane man that picked a fight with you because you declined a massage!
                            You've done a favor to the city, and no doubt to the kobolds he was [i]employing[/i].
                        [/p]
                        [p]
                            Then again, the [i]law[/i] might not see it that way.
                            You just [b]assaulted[/b] and perhaps even [b]killed[/b] a human.
                            He was a prick, and he totally deserved what happened to him, but he has rights, and you don't.
                            While you would love to stay and spread the good news to your peers,
                            someone might have [b]heard[/b] the struggle that took place.
                        [/p]
                        [p]
                            It'd be best if you took whatever you can and got the hell out of here [i]before[/i] anyone comes to investigate.
                            You bolt around the room, investigating a few points of interest, swiftly and superficially searching some cupboards and racks.
                            A tiny haul.
                            A bit of ${global.kobold.inventory.createItemWithTooltip(new aPlusLotion())} and some coins.
                            Not a lot, but more than nothing!
                            With your loot in hand, you bolt out of the door, and make yourself scarce from the slums.
                            A few minutes later, you find yourself in the mercantile district.
                            You're fairly certain nobody saw you leave the massage parlor, and you don't think anyone followed you, so you must be in the clear!
                        [/p]
                        [p]
                            You did a [b]good[/b] thing today.
                            Well, about as good as killing someone can be.
                            Anyway, the sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Maybe you can [b]smite[/b] some more [b]evildoers[/b] tomorrow!
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightLoot.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightSucceed.js

/**
 * Fight section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFightSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.fight;
    var changeController = ctrl.fight;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.fightFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorFought = true;
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorKilled = true;
        var weapon = ctrl.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var blunt = weapon.blunt();
        var stab = weapon.stab();
        var slash = weapon.slash();
        var canThrow = weapon.throw();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var egg = weapon.id === ItemEnum.ROTTEN_EGG;
        var armorSave = ctrl.misc.totalArmor() > 6;
        var shirt = ctrl.misc.shirt();
        var shirtPopup = global.kobold.inventory.createItemWithTooltip(shirt);
        var shirtArmor = (shirt && shirt.defence()) || 0;
        var pants = ctrl.misc.pants();
        var pantsPopup = global.kobold.inventory.createItemWithTooltip(pants);
        var pantsArmor = (pants && pants.defence()) || 0;
        var pantsBest = pantsArmor > shirtArmor;
        var toReturn = `[p]
                            You never asked for this, but if it's a fight the human wants, a fight he'll get.
                            He may be bigger than you, he may be [b]stronger[/b] than you, and he may have a fancy long-reaching weapon,
                            but you are [b]far[/b] more determined to leave this place alive than the sickly human is to capture you.
                            Heck, he looks like he's halfway to hell already.
                            Time to deliver him to where he belongs.
                        [/p]
                        [p]
                            
                            The leather cord snags you from the side,
                            snapping around your body, the steel tip [b]digging[/b] into your back,
                            burying in between two ligaments of your spine.
                        [/p]`;

        if (weaponSave) {
            toReturn += `${global.ui.condSucc(`Attack + defence greater than 5`)}
                        [p]
                            You brandish your ${weaponPopup}, and drop into a combat stance.
                            The two of you circle around one another like predatory animals.
                            Then, Tychus makes the first move.
                            He swings his whip in a wide horizontal arc.
                            Luckily, you have a strong weapon built for combat.
                            You quickly move to parry the blow, and,
                            to your own surprise, you actually manage to do so,
                            striking the metal tip of the whip away!
                            Wow!
                            Is that how knights do it?
                        [/p]`;
        } else {
            toReturn += `${global.ui.condFail(`Attack + defence below 6`)}
                        [p]
                            You brandish your ${weaponPopup}, and drop into a combat stance.
                            The two of you circle around one another like predatory animals.
                            Then, Tychus makes the first move.
                            He swings his whip in a wide horizontal arc.
                            Despite your best efforts, you [b]fail[/b] to block the attack.
                        [/p]
                        [p]
                            ${global.ui.condSucc(`Armor greater than 6`)}
                        [/p]
                        [p]
                            ${pantsBest ? `The leather cord snags your leg from the side,
                            wrapping around the limb, the steel tip [b]smacking[/b] against your kneecap!
                            Luckily, your ${pantsPopup} are sturdy enough to somewhat shield you from the blow, preventing the worst of the damage.
                            ` : `The leather cord snags you from the side,
                            snapping around your body, the steel tip [b]digging[/b] into your back!
                            Luckily, your ${shirtPopup} is sturdy enough to somewhat shield you from the blow, preventing the worst of the damage.`}
                        [/p]`;
        }

        toReturn += `   [p]
                            Tychus is visibly confused.
                            He probably expected that attack to take you down.
                            He tosses his whip to the side, and prepares his hands to grab you instead.
                            "[color=purple]I'll [b]squeeze[/b] the fight out of you![/color]"
                            He clearly intents to choke or bearhug you, but as the human advances,
                            he stumbles and falls, tripping over his own feet, tumbling face-first to the ground!
                        [/p]
                        [p]
                            Unlucky bastard.
                            You capitalize on his mistake, breaching the gap between the two of you.
                            ${egg ? `You throw your ${weaponPopup}, hitting Tychus straight in the eye!
                            Both the rotten egg and the man's eyeball [b]splatter[/b] open into a foul mixture of eye-goop and reeking,
                            long-overdue eggwhite.` : blunt ? `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]bashing[/b] his skull with all of your might.` : stab ? `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]stabbing[/b] it into his skull with all of your might.` : `Raising your ${weaponPopup}, you bring it to bear upon Tychus' head,
                            [b]slicing[/b] his skull with all of your might, rending his flesh.`}
                            The human [b]screams[/b] in pain!
                            ${egg ? `` : `You retract your weapon, the damage done.`}
                        [/p]
                        [p]
                            With his last ounce of energy, Tychus glares up at you.
                            The hateful look on his face has given way to a fearful, frightened gaze.
                            He stammers out the final few concoctions of his pain-shocked mind.
                            "[color=purple]What...are...you?[/color]"
                            Instead of bothering to reply, you lift up your foot, and [b]stomp[/b] his head into the floor,
                            knocking the man out, hopefully [b]for good[/b].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightFail.js

/**
 * Fight section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFightFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.fightFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var weapon = ctrl.misc.weapon();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var toReturn = `[p]
                            You never asked for this, but if it's a fight the human wants, a fight he'll get.
                            He may be bigger than you, he may be [b]stronger[/b] than you, and he may have a fancy long-reaching weapon,
                            but you are [b]far[/b] more determined to leave this place alive than the sickly human is to capture you.
                            Heck, he looks like he's halfway to hell already.
                            Time to deliver him to where he belongs.
                        [/p]
                        ${global.ui.condFail(`Attack + defence below 6`)}
                        ${global.ui.condFail(`Armor below 7`)}
                        [p]
                            You brandish your ${weaponPopup}, and drop into a combat stance.
                            The two of you circle around one another like predatory animals.
                            Then, Tychus makes the first move.
                            He swings his whip in a wide horizontal arc.
                            Despite your best efforts, you [b]fail[/b] to block the attack.
                            The leather cord snags you from the side,
                            snapping around your body, the steel tip [b]digging[/b] into your back,
                            burying in between two ligaments of your spine.
                        [/p]
                        [p]
                            You [b]scream[/b] out in pain, your spinal cord feeling as though it's completely [b]severed[/b]!
                            You let go of your weapon, dropping to your knees, and then crumpling to the floor.
                            You panic, thinking he [b]crippled[/b] you!
                            [b]You can't move[/b], no matter how much you try!
                        [/p]
                        [p]
                            Luckily, the sensation proves temporary.
                            By the time you regain the ability to rise to your feet, however, the fight is already long over.
                            Tychus has kicked your weapon to the side, and with you still struggling to move, he easily grabs you by your tail.
                            The human coarsely lifts you up off the ground and slings you over his shoulder like a piece of [b]meat[/b].
                            You try to fight back, but you still have a hard time controlling your body.
                            The most you manage to do is struggle and squirm a little, not nearly enough to free yourself.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFightFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFight.js

/**
 * Fight section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorFight(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorSecondDeclineChoice.FIGHT;
    var reqController = ctrl.secondDecline;

    var toFail = new AfternoonThreeParlorFightFail(ctrl);
    var toSuccess = new AfternoonThreeParlorFightSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var weapon = ctrl.misc.weapon();
        return ctrl.misc.totalArmor() > 6 || weapon.offence() + weapon.defence() > 5;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\fight\AfternoonThreeParlorFight.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\duck\AfternoonThreeParlorDuck.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDuck(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorRunChoice.DUCK;
    var reqController = ctrl.run;
    var changeController = ctrl.duckSucceed;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            A jar of oil?
                            You bet the human will aim for your head.
                            You don't need to do any fancy dodging.
                            All you need to do is duck down a little, and he'll miss.
                            You're sure of it!
                            Lowering your head, you continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        [p]
                            The jar, which turns out to be empty, misses your lowered head by mere inches,
                            cracking open against the wall instead, breaking into a million little pieces.
                            Dumb human can't aim!
                            You run through the dust-cloud of debris, sprinting out the door and into the streets, vanishing amidsts the slums.
                            A few minutes of rapid jogging later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            Your left hand hurts like hell.
                            The lacerated cut isn't as bad as you thought it was, but it's pretty darn painful.
                            You think something internally might be broken or shattered, but you retain full control over all of your fingers,
                            so you're not really too concerned about it.
                            The pain'll fade, in time.
                            Or so you hope.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your wounds will heal overnight.
                            And if they don't, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\duck\AfternoonThreeParlorDuck.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDiveSucceed.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDiveSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var changeController = ctrl.diveSucceed;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            A jar of oil?
                            You need to dive to the ground at the exact right time if you want to dodge that!
                            He'll never see that coming!
                            Preparing to dive, you continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condSucc(`Luck greater than 6`)}
                        [p]
                            Exactly as planned, you dive to the floor, your inertia-driven body sliding along the ground,
                            your face coming to a halt directly in front of the wall.
                            The jar, which turns out to be empty, misses your prone frame,
                            cracking open against the wall instead, breaking into a million little pieces.
                            Threw too soon, old man!
                            You scramble to your feet and run through the dust-cloud of debris,
                            sprinting out the door and into the streets, vanishing amidsts the slums.
                            A few minutes of rapid jogging later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            Your left hand hurts like hell.
                            The lacerated cut isn't as bad as you thought it was, but it's pretty darn painful.
                            You think something internally might be broken or shattered, but you retain full control over all of your fingers,
                            so you're not really too concerned about it.
                            The pain'll fade, in time.
                            Or so you hope.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your wounds will heal overnight.
                            And if they don't, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDiveSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDiveFail.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDiveFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.bottleFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            A jar of oil?
                            You need to dive to the ground at the exact right time if you want to dodge that!
                            He'll never see that coming!
                            Preparing to dive, you continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condFail(`Luck below 7`)}
                        [p]
                            You dive to the floor, your inertia-driven body sliding along the ground,
                            your face harshly [b]smacking[/b] against the wall, knocking you senseless.
                            As if to add insult to injury, the jar painfully impacts against your prone back,
                            cracking open, breaking into a million little pieces, hastening your descent.
                            Your sight goes black.
                        [/p]
                        [p]
                            You wake up a few moments later, still on the floor.
                            Your face hurts, your back hurts, your [i]everything[/i] hurts.
                            Then you realize: you were [b]running[/b].
                            Before you manage to scramble to your feet, Tychus has already caught up to you.
                            The human coarsely [b]grabs[/b] you by your tail,
                            lifting you up off the ground and slinging you over his shoulder like a piece of [b]meat[/b].
                            You try to fight back, but getting hit by the jar has left you dazed and stunned.
                            The most you manage to do is struggle and squirm a little, not nearly enough to free yourself.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDiveFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDive.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDive(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorRunChoice.DIVE;
    var reqController = ctrl.run;

    var toFail = new AfternoonThreeParlorDiveFail(ctrl);
    var toSuccess = new AfternoonThreeParlorDiveSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.misc.luck() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dive\AfternoonThreeParlorDive.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodgeSucceed.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDodgeSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var changeController = ctrl.dodgeSucceed;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            A jar of oil?
                            A simple sidestep should suffice to avoid that!
                            You don't need to dive all the way to the ground.
                            All you need to do is move to the side a little, and he'll miss.
                            You're sure of it!
                            Preparing to evade, you continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condSucc(`Agility greater than 6`)}
                        [p]
                            Using your superior agility, you hop to the left mid-stride, dodging without even stopping!
                            The jar, which turns out to be empty, misses you by mere inches,
                            cracking open against the wall instead, breaking into a million little pieces.
                            Seems like you were too fast for the slow brute to handle!
                            You run through the dust-cloud of debris, sprinting out the door and into the streets, vanishing amidsts the slums.
                            A few minutes of rapid jogging later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            Your left hand hurts like hell.
                            The lacerated cut isn't as bad as you thought it was, but it's pretty darn painful.
                            You think something internally might be broken or shattered, but you retain full control over all of your fingers,
                            so you're not really too concerned about it.
                            The pain'll fade, in time.
                            Or so you hope.
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your wounds will heal overnight.
                            And if they don't, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodgeSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodgeFail.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDodgeFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.bottleFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            A jar of oil?
                            A simple sidestep should suffice to avoid that!
                            You don't need to dive all the way to the ground.
                            All you need to do is move to the side a little, and he'll miss.
                            You're sure of it!
                            Preparing to evade, you continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condFail(`Agility below 7`)}
                        [p]
                            You try to sleekly dodge to the side,
                            but in retrospect perhaps you should have [i]stopped[/i] to evade instead of attempting to do so mid-stride.
                            You trip over your feet, falling face-first to the ground.
                            As if to add insult to injury, the jar painfully impacts against your back mid-fall,
                            cracking open, breaking into a million little pieces, hastening your descent.
                            Your sight goes black.
                        [/p]
                        [p]
                            You wake up a few moments later, still on the floor.
                            Your face hurts, your back hurts, your [i]everything[/i] hurts.
                            Then you realize: you were [b]running[/b].
                            Before you manage to scramble to your feet, Tychus has already caught up to you.
                            The human coarsely [b]grabs[/b] you by your tail,
                            lifting you up off the ground and slinging you over his shoulder like a piece of [b]meat[/b].
                            You try to fight back, but getting hit by the jar has left you dazed and stunned.
                            The most you manage to do is struggle and squirm a little, not nearly enough to free yourself.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodgeFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodge.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDodge(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorRunChoice.DODGE;
    var reqController = ctrl.run;

    var toFail = new AfternoonThreeParlorDodgeFail(ctrl);
    var toSuccess = new AfternoonThreeParlorDodgeSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.misc.agility() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\dodge\AfternoonThreeParlorDodge.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTankSucceed.js

/**
 * End section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorTankSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var changeController = ctrl.tankSucceed;

    var nextSceneReady = ctrl.nextSceneReady;
    var nextSceneName = ctrl.nextSceneName;
    var nextFolderName = ctrl.nextFolderName;

    var continueId = arguments.callee.name + "ButtonOne";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var shirt = ctrl.misc.shirt();
        var modReq = shirt ?  7 - Math.floor(shirt.defence() / 3) : 7;
        var toReturn = `[p]
                            A jar of oil?
                            You've been hit with heavier things!
                            You disregard Tychus, and continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condSucc(`Endurance greater than ${modReq - 1}`)}
                        [p]
                            The jar, which turns out to be empty, painfully impacts against your back, cracking open, breaking into a million little pieces.
                            ${shirt ? `Your shirt lessens the blow a little, dampening the blunt force.` : `Your scales, hardened over these past few days,
                            manage to resist most of the impact.`}
                            It hurts, and you've no doubt gained a few cuts and bruises,
                            but you manage to keep going, sprinting out the door and into the streets, vanishing amidsts the slums.
                        [/p]
                        [p]
                            A few minutes of rapid running later, you come to an exhausted standstill, somewhere amidsts the mercantile district of the city.
                            Confident that you've shaken off your pursuer, you sit yourself down against a nearby building, and catch your breath.
                        [/p]
                        [p]
                            Your left hand hurts like hell.
                            The lacerated cut isn't as bad as you thought it was, but it's pretty darn painful.
                            You think something internally might be broken or shattered, but you retain full control over all of your fingers,
                            so you're not really too concerned about it.
                            The pain'll fade, in time.
                            Or so you hope.
                        [/p]
                        [p]
                            Your hand isn't the only thing that hurts.
                            ${shirt ? `Reaching underneath your shirt,` : `Feeling around your back,`}
                            you come across some blood leaking down towards your butt.
                            You can't see exactly how bad the [i]damage[/i] is, but you don't think it's life-threatening.
                            If you really were leaking [i]that[/i] much blood, you'd have fainted by now.
                            Probably just a few scratches.
                            Nothing you can't handle.
                            You're a [b]tough[/b] kobold, not some [b]wimp[/b].
                        [/p]
                        [p]
                            You rise to your feet once more.
                            The sun is slowly starting to set.
                            The afternoon is turning into a dark and cold winter evening.
                            It's time to figure out where you're going to sleep.
                            Your wounds will heal overnight.
                            And if they don't, there's always tomorrow.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.continued();
        var choice = `<div class="choicecontainer">`;
        choice += `<button class="button bigbutton continuebutton" id="${continueId}" ${disabled ? "disabled" : ""}>`;
        choice += `Continue &gt;&gt`;
        choice += `</button>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next scene.
     * @param {string} sceneName Name of the scene to load.
     * @param {string} folder Folder of the scene to load.
     */
    function loadNextPart(sceneName, folder) {
        global.ui.fadeOutContent(function () {
            global.sceneEnd();
            global.sceneLoader.loadScene(sceneName, folder);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.continue();
            loadNextPart(nextSceneName, nextFolderName);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        if (!nextSceneReady) {
            part += `<span class="todo">There is no content beyond this point.</span>`;
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.continued()) {
            return;
        }

        if (nextSceneReady) {
            registerContinueListener();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTankSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTankFail.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorTankFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.preFail;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.bottleFail;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var shirt = ctrl.misc.shirt();
        var modReq = shirt ?  7 - Math.floor(shirt.defence() / 3) : 7;
        var toReturn = `[p]
                            A jar of oil?
                            You've been hit with heavier things!
                            You disregard Tychus, and continue heading for the door.
                            The human, predictably, hurls the heavy pot your way.
                        [/p]
                        ${global.ui.condFail(`Endurance below ${modReq}`)}
                        [p]
                            The jar painfully impacts against your back, cracking open, breaking into a million little pieces.
                            ${shirt ? `Your shirt lessens the blow a little, dampening the blunt force, but not nearly enough.` : `Your scales,
                            hardened over these past few days,
                            aren't nearly tough enough to resist an impact of this magnitude.`}
                            The sheer force of the hit is enough to unbalance you, making you fall flat on your face.
                        [/p]
                        [p]
                            Before you manage to recover, Tychus has already caught up to you.
                            Right as you were about to scramble to your feet once more, the human coarsely [b]grabs[/b] you by your tail,
                            lifting you up off the ground and slinging you over his shoulder like a piece of [b]meat[/b].
                            You try to fight back, but getting hit by the jar has left you dazed and stunned.
                            The most you manage to do is struggle and squirm a little, not nearly enough to free yourself.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTankFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTank.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorTank(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorRunChoice.TANK;
    var reqController = ctrl.run;

    var toFail = new AfternoonThreeParlorTankFail(ctrl);
    var toSuccess = new AfternoonThreeParlorTankSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        var stat = ctrl.misc.endurance();
        var chest = ctrl.misc.shirt();
        if(chest) {
            stat += chest.defence() / 3;
        }
        return stat > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\tank\AfternoonThreeParlorTank.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\AfternoonThreeParlorRun.js

/**
 * Run section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorRun(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorSecondDeclineChoice.RUN;
    var reqController = ctrl.secondDecline;
    var subController = ctrl.run;
    var changeController = ctrl.run;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";
    var fourthId = arguments.callee.name + "ButtonFour";

    var firstNext = new AfternoonThreeParlorTank(ctrl);
    var secondNext = new AfternoonThreeParlorDodge(ctrl);
    var thirdNext = new AfternoonThreeParlorDive(ctrl);
    var fourthNext = new AfternoonThreeParlorDuck(ctrl);

    var firstChoice = AfternoonThreeParlorRunChoice.TANK;
    var secondChoice = AfternoonThreeParlorRunChoice.DODGE;
    var thirdChoice = AfternoonThreeParlorRunChoice.DIVE;
    var fourthChoice = AfternoonThreeParlorRunChoice.DUCK;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorRan = true;
        var toReturn = `[p]
                            You'd love to comply to Tychus' wishes, but like you said:
                            you have somewhere to be.
                            You slowly inch backwards, hoping the deranged massage parlor owner won't make a move until you're already out the door.
                            Alas, he sees right through your plan.
                            The human lashes out, cracking his whip at you, outright [b]lacerating[/b] your left hand with the steel tip.
                            You [b]scream[/b] out in pain, clutching your wrist.
                            Your blood spills to the ground.
                        [/p]
                        [p]
                            A sadistic grin forms on Tychus' face.
                            "[color=purple]I'll teach you to [b]listen[/b], lizard. I'll teach you [b]good[/b].[/color]"
                            Once again, he rears his whip upwards, preparing for a second strike.
                            You tremble in place, covering your face with your arms, knowing you won't be able to outrun the flexible, metal-tipped leather.
                            He lashes out, yet his whip gets caught on one of the beams supporting the roof of the building, wrapping around the wood.
                            The human tries to tug his weapon free, yet it seems stuck fast.
                        [/p]
                        [p]
                            This is your chance!
                            You turn tail and run, bolting for the door.
                            Looking back at the human mid-stride,
                            you notice he's ditched his weapon and is reaching for a heavy-looking jar of massage oil, probably to throw at you.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Tank</button>`;

        var tooltip = global.parser.parse(`Take the hit. Keep on truckin'.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Dodge</button>`;

        var tooltip = global.parser.parse(`Sidestep to avoid the projectile.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 135px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Dive</button>`;

        var tooltip = global.parser.parse(`Throw yourself to the ground.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFourthChoice() {
        var failedCheck = ctrl.misc.intelligence() < 7;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === fourthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fourthId}" ${disabled}>Duck</button>`;

        var tooltip = global.parser.parse(`Lower your frame slightly to avoid getting hit.`);
        if(failedCheck) {
            tooltip = global.parser.parse(`You are not smart enough to realize the projectile won't hit you if you lower your head.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += `<div class="choicerow" style="margin-bottom: 10px;">`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        choice += `<div class="choicerow">`;
        choice += buildThirdChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFourthChoice();
        choice += `</div>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFourthListener() {
        $("#" + fourthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(fourthChoice);
            loadNextPart(fourthNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        ctrl.bgm.loop();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        part += fourthNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();
        fourthNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
        registerFourthListener();
    };
}

var AfternoonThreeParlorRunChoice = {
    TANK: 1,
    DODGE: 2,
    DIVE: 3,
    DUCK: 4
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\run\AfternoonThreeParlorRun.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\decline\AfternoonThreeParlorSecondDecline.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorSecondDecline(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondChoice.DECLINE;
    var reqController = ctrl.secondOffer;
    var subController = ctrl.secondDecline;
    var changeController = ctrl.secondDecline;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeParlorRun(ctrl);
    var secondNext = new AfternoonThreeParlorFight(ctrl);

    var firstChoice = ctrl.secondChoice.DECLINE;
    var secondChoice = ctrl.secondChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You do [b]not[/b] trust this [b]one bit[/b].
                            You shake your head and the human, and tell him that while you appreciate the offer,
                            you kind of need to [i]get going[/i].
                            Maybe you'll be back some other time, but for now, you have some [i]urgent business[/i] you need to attend.
                        [/p]
                        [p]
                            Your lie does not come out as fluently as you'd hoped for.
                            The human seems [b]very[/b] upset.
                            An angry glare forms on his face, his discolored teeth half-bared.
                            "[color=purple]You [b]DARE[/b] spit my generosity back in my face?![/color]"
                            He rises to his feet, tipping over his wooden desk with a loud grunt,
                            pushing the piece of furniture aside, slamming both of the open doors shut in the process.
                            A large wood chip splintering from the cracking surface only barely misses your face, slightly scraping your cheek.
                        [/p]
                        [p]
                            The human reveals that the thing he was clutching underneath the now-tipped table was a large, lengthy, steel-tipped bullwhip.
                            Like the kind a slaver would use, only less meant to [i]discipline[/i], and more geared to [b]hurt[/b], [b]maim[/b] or even [b]kill[/b].
                            "[color=purple]Get the [b]fuck[/b] in the back room [b]right now[/b], kobold.
                            Don't make me [b]hurt[/b] you.[/color]"
                            Tychus loudly cracks the whip, showing that he means business.
                            As if his expression of pure, unbridled rage hadn't already convinced you.
                        [/p]
                        [p]
                            You cautiously take a few steps backwards, adrenaline flooding your system.
                            Like you figured: he's used to getting his way,
                            and you not playing along with his [i]plan[/i] has propelled the human into a blind, wild rampage.
                            You didn't figure him for the [i]violent[/i] type, however.
                            Must be bipolar or something.
                        [/p]
                        [p]
                            While Tychus doesn't look very healthy, he's still twice your size, and his weapon looks like it could seriously hurt you.
                            Still, you get the feeling that you [i]don't[/i] want to take orders from him,
                            given that he went from trying to sell you a massage to wanting to [b]flay[/b] you in a matter of seconds.
                            Your safest bet would be to back off slowly, and hightail out of the building the way you came.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Run</button>`;

        var tooltip = global.parser.parse(`A man his size could easily end you. Run!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 170px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var success = ctrl.misc.weapon();
        var disabled = subController.has() || !success ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Fight</button>`;

        var tooltip = global.parser.parse(`If this human oaf wants a fight, a fight he shall get!`);
        if (!success) {
            tooltip = global.parser.parse(`You have no weapon.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}


var AfternoonThreeParlorSecondDeclineChoice = {
    RUN: 1,
    FIGHT: 2
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\decline\AfternoonThreeParlorSecondDecline.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\accept\AfternoonThreeParlorAccept.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorAccept(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorEntranceChoice.ACCEPT;
    var reqController = false; // Set manually
    var subController = ctrl.preMassage;
    var changeController = ctrl.accept;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.massage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var bargainSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var clothes = !ctrl.misc.nude();
        var toReturn = `[p]
                            While you remain skeptical, some relaxation sounds [b]lovely[/b] to you.
                            ${bargainSuccess ? `Twenty coins seems a reasonable price, especially for an entire [b]hour[/b].
                            ` : `Fourty coins is a pretty steep price, but you'll surely get a [b]quality[/b] massage for your coin.`}
                            And you'll get to be touched by a fellow kobold, which brings a few perverted thoughts to mind.
                        [/p]
                        [p]
                            Sure!
                            You'll take it!
                            You fetch ${bargainSuccess ? `twenty` : `fourty`} coins from your pouch, and toss them atop Tychus' desk.
                            "[color=purple]Excellent choice. You won't regret it.[/color]"
                            The man rises from his feet, and escorts you to the massage room.
                            The orange kobold stands at attention as her [i]boss[/i] walks in the room.
                            "[color=purple]Toss your belongings in the corner.
                            ${clothes ? `You'll need to be completely naked.` : `Just your coin pouch, I guess.`}[/color]"
                        [/p]
                        [p]
                            You comply with the human's instructions, ${clothes ? `undressing and` : ``}
                            tossing your coin pouch out of the way.
                            You eye your equally nude masseuse up and down.
                            A bit shy, you ask Tychus what her name is.
                            "[color=purple]Just call'er by her color.
                            That's what I do, isn't it, orange?[/color]"
                        [/p]
                        [p]
                            "[color=orange]Y-yes, master![/color]" comes the kobold's almost [i]scared[/i] reply.
                            You disregard her frightened tone, figuring she must be as shy as you are.
                            "[color=purple]You two get along. I'll be back in an hour.[/color]"
                            That being said, the human leaves the room, closing the door behind him, returning to his desk.
                        [/p]
                        [p]
                            In order to somewhat break the ice between the two of you, you approach your masseuse, and kindly compliment her lovely voice.
                            She thanks you, mentioning that she hasn't served another kobold in [b]forever[/b].
                            Well, you doubt a lot of kobolds are able to afford the steep price of a massage.
                            She gives you an uneasy smile, before pointing towards the nearest massage table.
                        [/p]
                        [p]
                            "[color=orange]You can have an arm massage, a back massage, or a footrub, but only [b]one[/b] of those.[/color]"
                            You ask why you can't have two, or perhaps a mix of all three.
                            "[color=orange]You paid for one massage, not multiple.
                            Last time I handed out a freebie, the master threatened to [b]beat[/b] me.
                            He has a bit of a [i]hair trigger[/i], in more than one way![/color]"
                            You frown upon hearing that the human might be [i]abusing[/i] your kind,
                            but you suppose it'd have just been some [i]disciplinary[/i] love-taps.
                        [/p]
                        [p]
                            Instead of dwelling on whether or not the workers here are kept in good condition,
                            you decide to focus on your impending massage.
                            Arms, back or feet.
                            A tough choice, since they all kind of ache in their own separate way.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        reqController = ctrl.entrance;
        if (reqController.get() === AfternoonThreeParlorEntranceChoice.BARTER) {
            reqController = ctrl.bargain;
        }

        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\accept\AfternoonThreeParlorAccept.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\accept\AfternoonThreeParlorNoMoneyAccept.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorNoMoneyAccept(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = ctrl.secondChoice.ACCEPT;
    var reqController = ctrl.secondOffer;
    var subController = ctrl.preMassage;
    var changeController = ctrl.secondAccept;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = ctrl.fragments.massage;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var clothes = !ctrl.misc.nude();
        var toReturn = `[p]
                            While you remain skeptical, some relaxation sounds [b]lovely[/b] to you.
                            How can you decline an hour in heaven, without any admission fee even?
                            And you'll get to be touched by a fellow kobold, which brings a few perverted thoughts to mind.
                        [/p]
                        [p]
                            Sure!
                            You'll take it!
                            One free massage, please!
                            "[color=purple]Excellent choice. You won't regret it.[/color]"
                            The man rises from his feet, and escorts you to the massage room.
                            The orange kobold stands at attention as her [i]boss[/i] walks in the room.
                            "[color=purple]Toss your belongings in the corner.
                            ${clothes ? `You'll need to be completely naked.` : `Just your coin pouch, I guess.`}[/color]"
                        [/p]
                        [p]
                            You comply with the human's instructions, ${clothes ? `undressing and` : ``}
                            tossing your coin pouch out of the way.
                            You eye your equally nude masseuse up and down.
                            A bit shy, you ask Tychus what her name is.
                            "[color=purple]Just call'er by her color.
                            That's what I do, isn't it, Orange?[/color]"
                        [/p]
                        [p]
                            "[color=orange]Y-yes, master![/color]" comes the kobold's almost [i]scared[/i] reply.
                            You disregard her frightened tone, figuring she must be as shy as you are.
                            "[color=purple]You two get along. I'll be back in an hour.[/color]"
                            That being said, the human leaves the room, closing the door behind him, returning to his desk.
                        [/p]
                        [p]
                            In order to somewhat break the ice between the two of you, you approach your masseuse, and kindly compliment her lovely voice.
                            She thanks you, mentioning that she hasn't served another kobold in [b]forever[/b].
                            Well, you doubt a lot of kobolds are able to afford the steep price of a massage.
                            Orange gives you an uneasy smile, before pointing towards the nearest massage table.
                        [/p]
                        [p]
                            "[color=orange]You can have an arm massage, a back massage, or a footrub, but only [b]one[/b] of those.[/color]"
                            You ask why you can't have two, or perhaps a mix of all three.
                            "[color=orange]You only get [b]one[/b] for free, not multiple.
                            Last time I handed out double freebies, the master threatened to [b]beat[/b] me.
                            He has a bit of a [i]hair trigger[/i], in more than one way![/color]"
                            You frown upon hearing that the human might be [i]abusing[/i] your kind,
                            but you suppose it'd have just been some [i]disciplinary[/i] love-taps.
                        [/p]
                        [p]
                            Instead of dwelling on whether or not the workers here are kept in good condition,
                            you decide to focus on your impending massage.
                            Arms, back or feet.
                            A tough choice, since they all kind of ache in their own separate way.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\accept\AfternoonThreeParlorNoMoneyAccept.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\decline\AfternoonThreeParlorDecline.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorDecline(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorEntranceChoice.DECLINE;
    var reqController = false; // Set manually
    var subController = ctrl.secondOffer;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = ctrl.fragments.secondDecline;
    var secondNext = ctrl.fragments.noGoldAccept;

    var firstChoice = ctrl.secondChoice.DECLINE;
    var secondChoice = ctrl.secondChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var barterSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var barterAttempted = reqController === ctrl.bargain;
        var toReturn = `[p]
                            ${barterSuccess ? `Something is definitely wrong here. No human would offer a discount to a kobold.
                            Then again, he [i]does[/i] rely on your kind for his entire livelihood.
                            ` : barterAttempted ? `If he's not going to give you a discount, then you're not going to have a massage.
                            Simple as that. Plus this entire thing seems a little fishy to you.
                            ` : `Something is definitely wrong here. No human would treat a kobold with this much respect.`}
                            You shake your head no.
                            No deal.
                        [/p]
                        [p]
                            Tychus grumbles loudly, mumbling something you don't understand.
                            "[color=purple]${barterSuccess ? `I give you a discount, and [b]still[/b] you refuse?
                            ` : barterAttempted ? `Is it because I'm not giving you a discount?
                            ` : `Is it because I'm a human?`}
                            How about this.
                            I'll make you an offer.
                            A once-in-a-lifetime deal.
                            You get a [b]freebie[/b].
                            No payment, one massage.
                            You'll be back for more anyway, after you feel how [b]amazing[/b] it can be.
                            What do you say?[/color]"
                        [/p]
                        [p]
                            Wait, what?
                            [b]Really[/b]?
                            For [b]free[/b]?
                            Why?
                            That makes you even [b]more[/b] suspicious.
                            Why does Tychus want you to get a massage so badly?
                            Is he [b]that[/b] desperate for new customers?
                            While you are [b]very[/b] wary about this whole situation,
                            you actually [b]do[/b] kind of want a massage.
                            You are left conflicted.
                            What should you do?
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Decline</button>`;

        var tooltip = global.parser.parse(`Something's fishy here. You think it's time for you to leave.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 210px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`Well, if it's free, then you don't see why not!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        reqController = ctrl.entrance;
        if (reqController.get() === AfternoonThreeParlorEntranceChoice.BARTER) {
            reqController = ctrl.bargain;
        }

        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\decline\AfternoonThreeParlorDecline.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\nogold\AfternoonThreeParlorNoGold.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorNoGold(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorEntranceChoice.NOGOLD;
    var reqController = false; // Set manually
    var subController = ctrl.secondOffer;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = ctrl.fragments.secondDecline;
    var secondNext = ctrl.fragments.noGoldAccept;

    var firstChoice = ctrl.secondChoice.DECLINE;
    var secondChoice = ctrl.secondChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var barterSuccess = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess;
        var barterAttempted = reqController === ctrl.bargain;
        var couldHaveBoughtIfBarterSucceeded = barterAttempted && ctrl.misc.gold() > 19;
        var toReturn = `[p]
                            Sadly,
                            ${barterSuccess ? `even [i]with[/i] the price halved, you still can not afford a massage.
                            ` : couldHaveBoughtIfBarterSucceeded ? `because Tychus refuses to halve the price, you can not afford a massage.
                            ` : barterAttempted ? `even [i]if[/i] Tychus had halved the price, you still would not have been able to afford a massage.
                            ` : `you can't really [i]afford[/i] a massage.`}
                            ${barterAttempted ? `Once more, you` : `You`} look up at Tychus with a devestated look on your face,
                            putting on your best [i]puppy eyes[/i], your [i]cutesy[/i] look complete with a trembling mouth and watering eyes.
                            You were [b]really[/b] looking forward to a massage, but you can't have one because you don't have enough money!
                            You almost burst out into tears, your sadness half-acted, and partially based in reality.
                        [/p]
                        [p]
                            The human sighs.
                            At first, he resolutely looks away, but as you continue to stare up at him looking like a scolded child denied a treat,
                            he slowly turns to face you once more.
                            Another sigh.
                            ${barterAttempted ? `` : `He rolls his eyes.`}
                            "[color=purple]${barterSuccess ? `Why even barter me down to half if you still can't pay? Dumb kobold. ` : ``}Fine.
                            You get a [b]freebie[/b].
                            Just this [b]once[/b].
                            You'll be back for more anyway, after you feel how [b]amazing[/b] a massage can be.[/color]"
                        [/p]
                        [p]
                            Wait, what?
                            [b]Really[/b]?
                            That was [b]easy[/b].
                            [b]Too[/b] easy, in fact.
                            You wonder if there are [i]ulterior motives[/i] at play here.
                            But, while you [i]are[/i] wary of [b]any[/b] gifts handed to you by a human,
                            you actually [b]do[/b] kind of want a massage.
                            You are left conflicted.
                            What should you do?
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Decline</button>`;

        var tooltip = global.parser.parse(`Something's fishy here. You think it's time for you to leave.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`Well, if it's free, then you don't see why not!`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        reqController = ctrl.entrance;
        if (reqController.get() === AfternoonThreeParlorEntranceChoice.BARTER) {
            reqController = ctrl.bargain;
        }

        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\nogold\AfternoonThreeParlorNoGold.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarterSucceed.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBarterSucceed(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.bargain;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = ctrl.fragments.decline;
    var secondNext = ctrl.fragments.noGold;
    var thirdNext = ctrl.fragments.accept;

    var firstChoice = AfternoonThreeParlorEntranceChoice.DECLINE;
    var secondChoice = AfternoonThreeParlorEntranceChoice.NOGOLD;
    var thirdChoice = AfternoonThreeParlorEntranceChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess = true;
        var chaPass = ctrl.misc.charisma() > 6;
        var toReturn = `[p]
                            You address the human with your concerns.
                            A rubdown [i]would[/i] be nice, but for that much gold?
                            Being a kobold, every coin you earn is a struggle in its own right.
                            You can't part with your money this easily.
                            Such a shame, you [b]really[/b] wanted a massage!
                            You hang your head low, and melancholically think aloud about how nice it would be if the price were cut in half.
                        [/p]
                        ${chaPass ? global.ui.condSucc(`Charisma above 6`) : global.ui.condSucc(`Intelligence above 6`)}
                        [p]
                            Tychus frowns, visibly doubting whether or not he should admit to your pitiful [i]begging[/i].
                            "[color=purple]Ugh.
                            Last time I was nice to [b]your[/b] kind, it cost me my business.
                            Fine.
                            Half off.
                            Twenty coins.[/color]"
                            Hurray!
                            Your shrewd negotiating skills have pulled through once more.
                            You thank the human for his generosity, and assure him he won't regret it.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Decline</button>`;

        var tooltip = global.parser.parse(`You're having second thoughts about this.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var failedCheck = ctrl.misc.gold() > 19;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>No gold</button>`;

        var tooltip = global.parser.parse(`Even with the discount, you still don't have enough money to pay for a massage.`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You [b]do[/b] have enough money.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var failedCheck = ctrl.misc.gold() < 20;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`At a bargain price like this, you'll take it!`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You don't have enough money for this.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarterSucceed.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarterFail.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBarterFail(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.bargain;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = ctrl.fragments.decline;
    var secondNext = ctrl.fragments.noGold;
    var thirdNext = ctrl.fragments.accept;

    var firstChoice = AfternoonThreeParlorEntranceChoice.DECLINE;
    var secondChoice = AfternoonThreeParlorEntranceChoice.NOGOLD;
    var thirdChoice = AfternoonThreeParlorEntranceChoice.ACCEPT;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You address the human with your concerns.
                            A rubdown [i]would[/i] be nice, but for that much gold?
                            Being a kobold, every coin you earn is a struggle in its own right.
                            You can't part with your money this easily.
                            Such a shame, you [b]really[/b] wanted a massage!
                            You hang your head low, and melancholically think aloud about how nice it would be if the price were cut in half.
                        [/p]
                        ${global.ui.condFail(`Charisma and intelligence below 7`)}
                        [p]
                            "[color=purple]Nice try, lizard.
                            Too bad for you, I have a business to run.
                            If word gets out that I offer discounts, I'll be bankrupt again in no-time.[/color]"
                            You promise you won't tell anyone, honestly!
                            "[color=purple]And you expect me to trust a [b]kobold[/b]?
                            Full price, like everyone else.[/color]"
                            You roll your eyes.
                            While not a big deal, you are left slightly disappointed at your own poor haggling skills.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Decline</button>`;

        var tooltip = global.parser.parse(`No discount, no deal.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var failedCheck = ctrl.misc.gold() > 39;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>No gold</button>`;

        var tooltip = global.parser.parse(`Without the discount, you don't have enough money to pay for a massage.`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You [b]do[/b] have enough money.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var failedCheck = ctrl.misc.gold() < 40;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`It may be expensive, but fine, why not?`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You don't have enough money to accept.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarterFail.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarter.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorBarter(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeParlorEntranceChoice.BARTER;
    var reqController = ctrl.entrance;

    var toFail = new AfternoonThreeParlorBarterFail(ctrl);
    var toSuccess = new AfternoonThreeParlorBarterSucceed(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Checks whether the choice succeeded or failed.
     */
    function successCheck() {
        return ctrl.misc.charisma() > 6 || ctrl.misc.intelligence() > 6;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        if (successCheck()) {
            part += toSuccess.build();
        } else {
            part += toFail.build();
        }
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (successCheck()) {
            toSuccess.registerListeners();
        } else {
            toFail.registerListeners();
        }
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\barter\AfternoonThreeParlorBarter.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\AfternoonThreeParlorEntrance.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorEntrance(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.arrival;
    var subController = ctrl.entrance;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";
    var fourthId = arguments.callee.name + "ButtonFour";

    var firstNext = ctrl.fragments.decline;
    var secondNext = ctrl.fragments.noGold;
    var thirdNext = ctrl.fragments.accept;
    var fourthNext = new AfternoonThreeParlorBarter(ctrl);

    var firstChoice = AfternoonThreeParlorEntranceChoice.DECLINE;
    var secondChoice = AfternoonThreeParlorEntranceChoice.NOGOLD;
    var thirdChoice = AfternoonThreeParlorEntranceChoice.ACCEPT;
    var fourthChoice = AfternoonThreeParlorEntranceChoice.BARTER;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        ctrl.misc.weapon(); // Caching
        ctrl.misc.hasAnything(); // Caching
        global.kobold.history.shared.varanar.dayThree.afternoon.massageParlor = true;
        var clothes = !ctrl.misc.nude();
        var gold = ctrl.misc.gold();
        var toReturn = `[st]Massage parlor[/st]
                        [p]
                            The interior is surprisingly ornate for an otherwise dilapidated structure.
                            It's also far bigger than it appeared from the outside.
                            The room you're standing in looks like a [i]lobby[/i] of sorts.
                            Behind a wooden desk sits a large, bearded, bald human, too engrossed in reading a note to notice your arrival.
                            He is clad in poor attire, his clothing riddled with holes and tears.
                            Almost looks like a burlap sack he cut arm and leg-holes into!
                            His skin is [b]extremely[/b] pale, and while you're no expert on humans, he doesn't seem like a very healthy fellow.
                        [/p]
                        [p]
                            From where you're standing, you can see at least two other rooms through open doorways. 
                            There's a work room, with an orange-scaled kobold tidying up a series of massaging tables,
                            and what you presume to be a break room containing three other members of your kind, lounging about.
                            Red, green like yourself and blue.
                            Quite the collection of colors!
                        [/p]
                        [p]
                            After several moments, the man looks up and notices you.
                            He eyes you up and down, silently [i]judging[/i] you before speaking.
                            "[color=purple]Welcome to the Kobold's Den.[/color]" 
                            His voice is deep, his tone is firm, and he carries himself as a man who is used to getting his way,
                            despite his half-sickly appearance.
                            You see him reach for something under the desk, but you can't tell what it is from your vantage point.
                        [/p]
                        [p]
                            "[color=purple]I'm [b]Tychus[/b], the owner of this [i]fine establishment[/i].[/color]"
                            You sense of a hint of sarcasm in his voice.
                            Self-deprecation must be a favored passtime for anyone living in [i]smelltown[/i].
                            You ask him if you were right in assuming this is a massage parlor employing kobolds.
                            A smirk forms on his face.
                            "[color=purple]Was the sign not clear enough for you?[/color]"
                            It was pretty hard to make out, you confess.
                            "[color=purple]It's the best I could afford since my previous business went under.
                            And that also explains why I had to hire [i]your[/i] kind.[/color]"
                            Yikes.
                        [/p]
                        [p]
                            "[color=purple]This is the only massage parlor in town that employs kobolds.
                            It seems that I've finally found something you little lizards are actually [i]good[/i] at, besides whoring yourselves out.
                            Your little hands are [b]far[/b] more nimble than human palms,
                            and they lack fur that gets all sticky when mixed with massage oil.
                            Plus some of my more [i]hardy[/i] customers enjoy a good clawed backscratching.
                            I'd say dragonesses are better masseuses, but I can't afford those.[/color]"
                        [/p]
                        [p]
                            Unsure how to react, you walk up to the man's desk.
                            You [i]guess[/i] he's somehow [i]complimenting[/i] you?
                            Luckily, he himself breaks the awkward silence that falls between the two of you.
                            "[color=purple]Anyway, I don't suppose you're here for a job.
                            So, how about a massage instead?[/color]"
                        [/p]
                        [p]
                            You get the distinct feeling that something isn't right here, but you can't quite put your finger on [i]what[/i] is wrong.
                            Perhaps there's just something unsettling about talking to a human [i]without[/i] him insulting and berating you.
                            Tychus takes your hesitation for confusion.
                            "[color=purple]Let me explain a bit more.
                            You pay me fourty coins, and then you get an hour-long massage from one of my girls.[/color]"
                            You mention that one of his [i]workers[/i] is clearly a [b]boy[/b].
                            "[color=purple][b]All[/b] you kobolds are [b]girls[/b] compared to us humans.[/color]"
                            There's the talking down you're used to.
                        [/p]
                        [p]
                            "[color=purple]You pay up front,
                            ${clothes ? `you take off your clothes,
                            and you head to the back room.
                            It's not like you kobolds are often dressed.` : `you're already naked,
                            as you kobolds often are,
                            so no need to worry about taking off your clothes.
                            You can head straight to the backroom, [b]after[/b] paying.`}
                            So, what do you say?
                            Want to feel good, for [b]once[/b] in your miserable life?
                            Or will you keep feeling like the [b]loser[/b] you are?[/color]"
                        [/p]
                        [p]
                            His sales pitch took a turn for the worse,
                            ending up almost outright [b]insulting[/b] you instead of praising his workers,
                            but you have to admit that Tychus [b]is[/b] kind of [i]right[/i].
                            It [b]would[/b] be nice to relax, if only for an hour.
                            Fourty coins is a steep price, though.
                            ${gold > 39 ? `You can afford it, but it's a [b]lot[/b] of money.` : `You can't even afford it.`}
                            And while Tychus has started to speak a bit more [i]human-like[/i],
                            you still think there's something [i]off[/i] about him.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Decline</button>`;

        var tooltip = global.parser.parse(`No thanks, you don't really want a massage. Especially not at [i]that[/i] rate.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 175px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var failedCheck = ctrl.misc.gold() > 39;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>No gold</button>`;

        var tooltip = global.parser.parse(`You can't really afford that.`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You [b]do[/b] have enough money.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var failedCheck = ctrl.misc.gold() < 40;
        var disabled = subController.has() || failedCheck ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled}>Accept</button>`;

        var tooltip = global.parser.parse(`You could really use a massage, even at an exuberant price.`);
        if (failedCheck) {
            tooltip = global.parser.parse(`You don't have enough money for this.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFourthChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === fourthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fourthId}" ${disabled}>Barter</button>`;

        var tooltip = global.parser.parse(`Attempt to barter the human down to a more reasonable price.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += `<div class="choicerow" style="margin-bottom: 10px;">`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        choice += `<div class="choicerow">`;
        choice += buildThirdChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFourthChoice();
        choice += `</div>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId), $("#" + fourthId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFourthListener() {
        $("#" + fourthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(fourthChoice);
            loadNextPart(fourthNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        global.ui.applyTheme("hamlet_barn");
        ctrl.bgm.loop();
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        if (ctrl.entrance.get() !== AfternoonThreeParlorEntranceChoice.BARTER) {
            part += firstNext.build();
            part += secondNext.build();
            part += thirdNext.build();
        }
        part += fourthNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        if (ctrl.entrance.get() !== AfternoonThreeParlorEntranceChoice.BARTER) {
            firstNext.registerListeners();
            secondNext.registerListeners();
            thirdNext.registerListeners();
        }
        fourthNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
        registerFourthListener();
    };
}

var AfternoonThreeParlorEntranceChoice = {
    DECLINE: 1,
    NOGOLD: 2,
    ACCEPT: 3,
    BARTER: 4
};

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\AfternoonThreeParlorEntrance.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\AfternoonThreeParlorArrival.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlorArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.arrival;
    var changeController = false;

    var autoContinue = false;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new AfternoonThreeParlorEntrance(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You'd rather not risk getting on the Three-Tailed Rat Clan's radar.
                            Even [i]if[/i] being amongst fellow kobolds could lift your spirits a little, it's not worth the risk.
                            Instead of continuing on, you walk down a side street, hoping to find [i]anything[/i] of interest.
                        [/p]
                        [p]
                            Delving deeper into this less savory neighbourhood,
                            you see and [i]smell[/i] that this place is every bit as much of a [i]slum[/i] as the kobold district is.
                            The roads are littered with small bits of refuse and trash, contributing to an unsavory odor that permeates the area.
                            At least there's plenty of [i]light[/i] here, the bright afternoon sun lighting your path, even through the clouds in the sky.
                        [/p]
                        [p]
                            While the odor is disconcerting, your inner thief is greatly pleased by the large amounts of alcoves and tiny alleyways.
                            Perfect places to conceal yourself in, should you need to hide.
                            The streets are pretty empty.
                            There's a few humans and anthros going about their business,
                            but overall, this place is a [b]lot[/b] less packed than the grand avenue.
                            Everyone's probably either indoors or busy working somewhere.
                        [/p]
                        [p]
                            The few businesses in this [i]dismal[/i] backstreet are exactly what you expected to find in such a rundown area.
                            Shady pawn shops, rundown taverns, even a couple of ramshackle inns which you believe to be houses of ill-repute in disguise.
                            You doubt even half these places would serve a kobold like yourself, and you don't feel like stealing things from poor people.
                        [/p]
                        [p]
                            You mindlessly wander on, exploring the slums a little.
                            Besides the stench, this doesn't seem like such a bad place to live.
                            You bet the actual inhabitants would care to argue, though.
                            After a few minutes of strolling,
                            a large wooden sign attached to a rundown brown brick building catches your attention.
                        [/p]
                        [p]
                            [color=brown][big][b]THE KOBOLD'S DEN[/b][/big][/color]
                        [/p]
                        [p]
                            Below this text, the sign features two crudely painted figures, one of which is distinctly lizard-shaped.
                            A kobold, most likely.
                            The other looks to be an anthro of some kind.
                            Maybe a bear?
                            Or a chubby wolf?
                            You can't quite figure out what it's supposed to be, the drawing too poorly made to discern the actual species.
                            Regardless, the anthro is resting on its stomach, atop what appears to be a bed or a table.
                            The kobold appears to be rubbing the furred creature's back.
                        [/p]
                        [p]
                            This must be a massage parlor!
                            And given that they [i]employ[/i] your kind, they'll probably have no quarrel serving you either.
                            Intrigued, you enter the building.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\arrival\AfternoonThreeParlorArrival.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\AfternoonThreeParlorController.js

/**
 * Controller for the Varanar, Afternoon Three scene.
 * @param {string} sceneId The id of the scene.
 * @param {AfternoonThreeParlorController} superCtrl The supercontroller.
 */
function AfternoonThreeParlorController(sceneId, superCtrl) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = superCtrl.nextSceneReady;
    this.nextSceneName = superCtrl.nextSceneName;
    this.nextFolderName = superCtrl.nextSceneName;

    this.misc = superCtrl.misc;
    this.postMassageMisc = new MiscController(id, "postMassageMisc");

    this.arrival = new ChoiceController(id, "parlorArrival");
    this.entrance = new ChoiceController(id, "entrance");
    this.bargain = new ChoiceController(id, "bargain");

    this.accept = new ChangeController(id, "accept", function () {
        if (global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess) {
            global.kobold.inventory.coins = global.kobold.inventory.coins - 20;
        } else {
            global.kobold.inventory.coins = global.kobold.inventory.coins - 40;
        }
        global.kobold.status.mental.increase(3, "Impending massage");
        that.storeInventory();
        that.clearInventory();
    }, function () {
        if (global.kobold.history.shared.varanar.dayThree.afternoon.parlorBargainSuccess) {
            return global.ui.change.buildNotification([
                new Change("Impending massage", "Sanity", +3),
                new Change("Paid", "Gold", -20)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Impending massage", "Sanity", +3),
                new Change("Paid", "Gold", -40)
            ]);
        }
    });

    this.secondOffer = new ChoiceController(id, "secondOffer");

    this.secondAccept = new ChangeController(id, "secondAccept", function () {
        global.kobold.status.mental.increase(3, "Impending massage");
        that.storeInventory();
        that.clearInventory();
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Impending massage", "Sanity", +3)
        ]);
    });

    this.secondDecline = new ChangeController(id, "secondDecline", function () {
        global.kobold.status.physical.decrease(3, "Face scraped");
        global.kobold.status.mental.decrease(3, "Sudden fight");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Face scraped", "Health", -3),
            new Change("Sudden fight", "Sanity", -3)
        ]);
    });

    this.run = new ChangeController(id, "run", function () {
        global.kobold.status.physical.decrease(6, "Hand whipped");
        global.kobold.status.mental.decrease(5, "Pain");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Hand whipped", "Health", -6),
            new Change("Pain", "Sanity", -5)
        ]);
    });

    this.diveSucceed = new ChangeController(id, "diveSucceed", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(12, "Diving and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Diving", "Stamina", -3),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.dodgeSucceed = new ChangeController(id, "dodgeSucceed", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Dodging and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    function calcTankDamage() {
        var chest = that.misc.shirt();
        var dmg = 8;
        if (chest) {
            dmg -= Math.floor(chest.defence() / 3);
        }
        return Math.max(dmg, 5);
    }

    this.tankSucceed = new ChangeController(id, "tankSucceed", function () {
        global.kobold.status.physical.decrease(calcTankDamage(), "Hit by jar");
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Tanking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Hit by jar", "Health", -calcTankDamage()),
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.duck = new ChangeController(id, "duck", function () {
        global.kobold.status.mental.increase(5, "Escaped fight");
        global.kobold.status.stamina.decrease(9, "Ducking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped fight", "Sanity", +5),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.fight = new ChangeController(id, "fight", function () {
        var weapon = that.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (weaponSave) {
            global.kobold.status.mental.increase(8, "Won fight");
            global.kobold.status.stamina.decrease(8, "Combat");
        } else {
            if (egg) {
                global.kobold.status.physical.decrease(2, "Hit during fight");
                global.kobold.status.mental.increase(6, "Won fight");
                global.kobold.status.stamina.decrease(8, "Combat");
                global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
            } else {
                global.kobold.status.physical.decrease(2, "Hit during fight");
                global.kobold.status.mental.increase(6, "Won fight");
                global.kobold.status.stamina.decrease(8, "Combat");
            }
        }
    }, function () {
        var weapon = that.misc.weapon();
        var weaponSave = weapon.offence() + weapon.defence() > 5;
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (weaponSave) {
            return global.ui.change.buildNotification([
                new Change("Parried blow", "Sanity", +3),
                new Change("Won fight", "Sanity", +5),
                new Change("Fighting", "Stamina", -8)
            ]);
        } else {
            if (egg) {
                return global.ui.change.buildNotification([
                    new Change("Minor injury", "Health", -2),
                    new Change("Saved by armor", "Sanity", +1),
                    new Change("Won fight", "Sanity", +5),
                    new Change("Fighting", "Stamina", -8),
                    new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
                ]);
            } else {
                return global.ui.change.buildNotification([
                    new Change("Minor injury", "Health", -2),
                    new Change("Saved by armor", "Sanity", +1),
                    new Change("Won fight", "Sanity", +5),
                    new Change("Fighting", "Stamina", -8)
                ]);
            }
        }
    });

    this.fightLoot = new ChangeController(id, "fightLoot", function () {
        global.kobold.inventory.coins = global.kobold.inventory.coins + 36;
        global.kobold.inventory.addItem(new aPlusLotion());
        global.kobold.inventory.addItem(new aPlusLotion());
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Loot", "Gold", +36),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new aPlusLotion()), +2)
        ]);
    });

    this.preMassage = new ContinueController(id, "preMassage");
    this.massage = new ChoiceController(id, "massage");

    this.arms = new ChangeController(id, "arms", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.STRENGTH, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Arms strengthened", "Strength", +1)
        ]);
    });

    this.back = new ChangeController(id, "back", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Back loosened", "Endurance", +1)
        ]);
    });

    this.feet = new ChangeController(id, "feet", function () {
        global.kobold.status.physical.increase(10, "Massage");
        global.kobold.status.mental.increase(10, "Massage");
        global.kobold.status.stamina.increase(10, "Massage");
        global.kobold.status.exertion.decrease(10, "Massage");
        global.kobold.stats.increaseStat(StatsEnum.AGILITY, 1);
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Body soothed", "Health", +10),
            new Change("Mind eased", "Sanity", +10),
            new Change("Fully relaxed", "Stamina", +10),
            new Change("Tension relieved", "Sexual exhaustion", -10, true),
            new Change("Feet rubbed", "Agility", +1)
        ]);
    });

    this.postMassage = new ContinueController(id, "postMassage");

    this.friend = new ChangeController(id, "friend", function () {
        global.kobold.status.mental.increase(5, "Befriending masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Avoided violence", "Sanity", +5),
            new Change("Distrust confirmed", "Sanity", +3),
            new Change("Resolve strengthened", "Sanity", +4),
            new Change("Implied suicide?", "Sanity", -7)
        ]);
    });

    this.foe = new ChangeController(id, "foe", function () {
        global.kobold.status.physical.decrease(3, "Half-completed sleep-grip");
        global.kobold.status.mental.increase(3, "Knocking out masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Half-completed sleep-grip", "Health", -3),
            new Change("Unintended excessive injury", "Sanity", -3),
            new Change("Did not get knocked out", "Sanity", +2),
            new Change("Distrust confirmed", "Sanity", +3),
            new Change("Resolve strengthened", "Sanity", +4),
            new Change("Is she dead?", "Sanity", -3)
        ]);
    });

    this.preKnockout = new ChangeController(id, "preKnockout", function () {
        global.kobold.status.physical.decrease(5, "Sleep-grip");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Sleep-grip", "Health", -5)
        ]);
    });

    this.knockout = new ChangeController(id, "knockout", function () {
        global.kobold.status.physical.increase(2, "Nice nap");
        global.kobold.status.mental.decrease(5, "Tied up");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Decent nap", "Health", +2),
            new Change("Tied up", "Sanity", -2),
            new Change("Rising panic", "Sanity", -3)
        ]);
    });

    this.cut = new ChangeController(id, "cut", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.break = new ChangeController(id, "break", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.escape = new ChangeController(id, "escape", function () {
        global.kobold.status.mental.increase(2, "Escaped ropes");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Escaped ropes", "Sanity", +4),
            new Change("Still in trouble", "Sanity", -2)
        ]);
    });

    this.preMasseuse = new ContinueController(id, "preMasseuse");

    this.masseuse = new ChoiceController(id, "masseuse");

    this.bluff = new ChangeController(id, "bluff", function () {
        global.kobold.status.mental.increase(3, "Befriended masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Avoided violence", "Sanity", +5),
            new Change("Made a friend", "Sanity", +5),
            new Change("Implied suicide?", "Sanity", -7)
        ]);
    });

    this.trap = new ChangeController(id, "trap", function () {
        global.kobold.status.mental.increase(1, "Trapped masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Outsmarted masseuse", "Sanity", +2),
            new Change("Excessive violence", "Sanity", -3),
            new Change("She deserved it", "Sanity", +2)
        ]);
    });

    this.overpower = new ChangeController(id, "overpower", function () {
        global.kobold.status.mental.increase(1, "Beat masseuse");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Overpowered masseuse", "Sanity", +2),
            new Change("Excessive violence", "Sanity", -3),
            new Change("She deserved it", "Sanity", +2)
        ]);
    });

    this.preItemGet = new ContinueController(id, "preItemGet");

    this.itemGet = new ChangeController(id, "itemGet", function () {
        that.restoreInventory();
    }, function () {
        return "";
    });

    this.itemGetFlee = new ChangeController(id, "itemGetFlee", function () {
        global.kobold.status.mental.increase(6, "Escaped massage parlor");
        global.kobold.status.stamina.decrease(9, "Sneaking and running");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Tense sneaking", "Sanity", -2),
            new Change("Close call", "Sanity", -2),
            new Change("Escaped massage parlor", "Sanity", +10),
            new Change("Running", "Stamina", -9)
        ]);
    });

    this.itemGetSpare = new ChangeController(id, "itemGetSpare", function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
        }
        global.kobold.status.mental.increase(10, "Knocked out Tychus");
    }, function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            return global.ui.change.buildNotification([
                new Change("Knocked out Tychus", "Sanity", +3),
                new Change("Not a murderer", "Sanity", +3),
                new Change("Vented frustration", "Sanity", +2),
                new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Knocked out Tychus", "Sanity", +3),
                new Change("Not a murderer", "Sanity", +3),
                new Change("Vented frustration", "Sanity", +2)
            ]);
        }
    });

    this.itemGetKill = new ChangeController(id, "itemGetKill", function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            global.kobold.inventory.equipment.strip(SlotEnum.WEAPON);
        }
        global.kobold.status.mental.increase(1, "Killed Tychus");
    }, function () {
        var weapon = that.misc.weapon();
        var egg = weapon && weapon.id === ItemEnum.ROTTEN_EGG;
        if (egg) {
            return global.ui.change.buildNotification([
                new Change("Killed Tychus", "Sanity", +3),
                new Change("Murderer", "Sanity", -4),
                new Change("Vented frustration", "Sanity", +2),
                new Change("Broke egg", global.kobold.inventory.createItemWithTooltip(weapon), -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Killed Tychus", "Sanity", +3),
                new Change("Murderer", "Sanity", -4),
                new Change("Vented frustration", "Sanity", +2)
            ]);
        }
    });

    this.itemGetFight = new ChoiceController(id, "itemGetFight");

    this.spareFree = new ChangeController(id, "spareFree", function () {
        global.kobold.status.mental.increase(5, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Liberator", "Sanity", +3),
            new Change("Mixed response", "Sanity", -2),
            new Change("Escaped massage parlor", "Sanity", +5),
            new Change("Helped others escape", "Sanity", +2),
            new Change("Disappearing act", "Sanity", -3)
        ]);
    });

    this.secondLoot = new ChangeController(id, "secondLoot", function () {
        global.kobold.inventory.coins = global.kobold.inventory.coins + 36;
        global.kobold.inventory.addItem(new aPlusLotion());
        global.kobold.inventory.addItem(new aPlusLotion());
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Loot", "Gold", +36),
            new Change("Loot", global.kobold.inventory.createItemWithTooltip(new aPlusLotion()), +2)
        ]);
    });

    this.killFree = new ChangeController(id, "killFree", function () {
        global.kobold.status.mental.increase(3, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Liberator", "Sanity", +3),
            new Change("Upset reaction", "Sanity", -2),
            new Change("Indecent suggestion", "Sanity", +2)
        ]);
    });

    this.killFreeRefuse = new ChangeController(id, "killFreeRefuse", function () {
        global.kobold.status.mental.increase(4, "Freeing kobolds");
    }, function () {
        return global.ui.change.buildNotification([
            new Change("Moral highground", "Sanity", +2),
            new Change("Escaped massage parlor", "Sanity", +5),
            new Change("Did the right thing?", "Sanity", -3)
        ]);
    });

    this.killFreeAccept = new ChangeController(id, "killFreeAccept", function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            global.kobold.stats.decreaseStat(StatsEnum.STRENGTH, 1);
            global.kobold.status.physical.decrease(15, "Dislocated arm");
            global.kobold.status.mental.decrease(10, "Dislocated arm");
        } else if (back) {
            global.kobold.stats.decreaseStat(StatsEnum.ENDURANCE, 1);
            global.kobold.status.physical.decrease(20, "Dislocated vertebra");
            global.kobold.status.mental.decrease(10, "Dislocated vertebra");
        } else {
            global.kobold.stats.decreaseStat(StatsEnum.AGILITY, 1);
            global.kobold.status.physical.decrease(15, "Dislocated foot");
            global.kobold.status.mental.decrease(10, "Dislocated foot");
        }
    }, function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            return global.ui.change.buildNotification([
                new Change("Arm dislocated", "Health", -15),
                new Change("Immense pain", "Sanity", -10),
                new Change("Arm dislocated", "Strength", -1)
            ]);
        } else if (back) {
            return global.ui.change.buildNotification([
                new Change("Vertebra dislocated", "Health", -20),
                new Change("Immense pain", "Sanity", -10),
                new Change("Vertebra dislocated", "Endurance", -1)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Foot dislocated", "Health", -15),
                new Change("Immense pain", "Sanity", -10),
                new Change("Foot dislocated", "Agility", -1)
            ]);
        }

    });

    this.killFreeAcceptLeave = new ChangeController(id, "killFreeAcceptLeave", function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            global.kobold.status.physical.increase(5, "Fixing your arm");
        } else if (back) {
            global.kobold.status.physical.increase(5, "Fixing your back");
        } else {
            global.kobold.status.physical.increase(5, "Fixing your foot");
        }
        global.kobold.status.mental.decrease(-3, "Escaped massage parlor");
    }, function () {
        var arms = global.kobold.history.shared.varanar.dayThree.afternoon.parlorArmMassage;
        var back = global.kobold.history.shared.varanar.dayThree.afternoon.parlorBackMassage;
        if (arms) {
            return global.ui.change.buildNotification([
                new Change("Popped arm back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        } else if (back) {
            return global.ui.change.buildNotification([
                new Change("Popped spine back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        } else {
            return global.ui.change.buildNotification([
                new Change("Popped foot back into place", "Health", +5),
                new Change("More pain", "Sanity", -5),
                new Change("Ungrateful kobolds", "Sanity", -3),
                new Change("Escaped massage parlor", "Sanity", +5)
            ]);
        }

    });

    this.preFail = new ContinueController(id, "preFail");
    this.preContinue = new ContinueController(id, "preContinue");

    this.fail = new ChoiceController(id, "fail");


    this.secondChoice = {
        DECLINE: 1,
        ACCEPT: 2
    };

    this.secondFightChoice = {
        LOOT: 1,
        FREE: 2
    };


    this.fragments = {};
    this.fragments.failChoices = new AfternoonThreeParlorFailChoices(this);
    this.fragments.masseuseFail = new AfternoonThreeParlorMasseuseFail(this);
    this.fragments.ropeFail = new AfternoonThreeParlorRopeFail(this);
    this.fragments.fightFail = new AfternoonThreeParlorFightFailEnd(this);
    this.fragments.bottleFail = new AfternoonThreeParlorBottleFail(this);
    this.fragments.itemGet = new AfternoonThreeParlorItemGet(this);
    this.fragments.masseuse = new AfternoonThreeParlorMasseuse(this);
    this.fragments.postMassage = new AfternoonThreeParlorPostMassage(this);
    this.fragments.massage = new AfternoonThreeParlorMassage(this);
    this.fragments.secondDecline = new AfternoonThreeParlorSecondDecline(this);
    this.fragments.noGoldAccept = new AfternoonThreeParlorNoMoneyAccept(this);
    this.fragments.decline = new AfternoonThreeParlorDecline(this);
    this.fragments.noGold = new AfternoonThreeParlorNoGold(this);
    this.fragments.accept = new AfternoonThreeParlorAccept(this);

    this.bgm = new AudioController(id, "res/audio/varanar/massageparlor.mp3", 120);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Resets the entire scene to the starting point.
     * Note that the main panel will still need to be cleared and reloaded after this.
     */
    this.reset = function () {
        superCtrl.reset();
    };

    /**
     * Clears and reloads the UI.
     */
    this.reload = function () {
        superCtrl.reload();
    };

    var inventoryKey = "parlorInventoryStorage";
    /**
     * Stores the kobold's inventory. Best used before taking it all off.
     */
    this.storeInventory = function () {
        global.kobold.history.setIfNotSet(id, inventoryKey, JSON.parse(JSON.stringify(global.kobold.inventory)));
    };

    /**
     * Clears (i.e. empties) the kobold's inventory.
     */
    this.clearInventory = function () {
        global.kobold.inventory.load(new Inventory());
    };

    /**
     * Restores the kobold's inventory from history. Best used when regaining items.
     */
    this.restoreInventory = function () {
        global.kobold.inventory.load(global.kobold.history.getKey(id, inventoryKey));
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\AfternoonThreeParlorController.js

// scenes\Varanar\DayThree\AfternoonThree\parlor\AfternoonThreeParlor.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeParlorController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeParlor(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = AfternoonThreeArrivalChoice.SLUMS;
    var reqController = ctrl.arrival;
    var subController = false;
    var changeController = false;

    var autoContinue = true;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new AfternoonThreeParlorArrival(ctrl.subControllers.parlor);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = ``;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = subController.continued() ? "disabled" : "";
        var isSelected = subController.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        if (autoContinue) {
            subController && subController.continue();
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += (!autoContinue && buildChoice()) || "";
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        next.registerListeners();

        if (!subController || subController.continued()) {
            return;
        }

        registerNextListener();
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\parlor\AfternoonThreeParlor.js

// scenes\Varanar\DayThree\AfternoonThree\kobold\AfternoonThreeKobold.js

/**
 * Information about the class.
 */
function AfternoonThreeKobold() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
}

// END scenes\Varanar\DayThree\AfternoonThree\kobold\AfternoonThreeKobold.js

// scenes\Varanar\DayThree\AfternoonThree\arrival\AfternoonThreeArrival.js

/**
 * Arrival section of Varanar, Afternoon Three.
 * @param {AfternoonThreeController} controller The Varanar, Afternoon Three controller.
 */
function AfternoonThreeArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.arrival;
    var changeController = false;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";

    var firstNext = new AfternoonThreeKobold(ctrl);
    var secondNext = new AfternoonThreeParlor(ctrl);

    var firstChoice = AfternoonThreeArrivalChoice.KOBOLD;
    var secondChoice = AfternoonThreeArrivalChoice.SLUMS;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The afternoon slowly sets in as you walk towards the kobold district.
                            Buildings gradually decline in quality, from fancy brick masonry with intricate attention to detail,
                            to sturdy wooden houses with solid foundations, transitioning into simple,
                            common homesteads made out of cheaper, more flimsy materials as you near your species' quarters.
                            You can see it in the distance.
                            A clear cutoff point, where cheap but functional homes give way to ramshackle metal [b]shanties[/b] layered atop one another,
                            forming buildings held together with nothing but old ropes and crooked, rusty nails.
                        [/p]
                        [p]
                            Seeing how [i]poor[/i] the kobold district is in,
                            even in comparison to the [i]already[/i] impoverished part of town you're in, fills your head with second thoughts.
                            Maybe going in [i]there[/i] isn't the best idea after all.
                            Besides, there [i]may[/i] be some people of the Three-Tailed Rat Clan that you'd really rather not run into in the kobold district.
                            Should you press on, or should you stick to exploring the human side of things instead?
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled} disabled>Kobold district</button>`;

        var tooltip = global.parser.parse(`You would be more at ease if you could walk amongst your own kind for a bit.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Human slums</button>`;

        var tooltip = global.parser.parse(`Kobolds are every bit as likely to stab you in the back as humans are.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You explore the:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        ctrl.set();
        global.ui.applyTheme("varanar_generic");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        //part += firstNext.build();
        part += secondNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        //firstNext.registerListeners();
        secondNext.registerListeners();

        if (subController.has()) {
            return;
        }

        //registerFirstListener();
        registerSecondListener();
    };
}

var AfternoonThreeArrivalChoice = {
    KOBOLD: 1,
    SLUMS: 2
};

// END scenes\Varanar\DayThree\AfternoonThree\arrival\AfternoonThreeArrival.js

// scenes\Varanar\DayThree\AfternoonThree\AfternoonThreeController.js

/**
 * Controller for the Varanar, Afternoon Three scene.
 * @param {string} sceneId The id of the scene.
 * @param {AfternoonThree} ui The scene itself, so it can be reloaded.
 */
function AfternoonThreeController(sceneId, ui) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;
    SceneController.call(this, id);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";
    this.nextSceneReady = false;
    this.nextSceneName = "SlaveAfternoonTwo";
    this.nextFolderName = "Enslavement/DayTwo";

    this.misc = new MiscController(id, "misc");

    this.arrival = new ChoiceController(id, "arrival");

    this.proposal = new ChangeController(id, "proposal", function () {

    }, function () {
        return global.ui.change.buildNotification([
            new Change("TODO", "CHANGE", +1)
        ]);
    });

    this.fragments = {};

    this.subControllers = {};
    this.subControllers.parlor = new AfternoonThreeParlorController(id, this);

    //this.bgm = new AudioController(id, "res/audio/varanar/noonthreestore.mp3", 100);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};
    var statusKey = "statusReset";
    var statsKey = "statsReset";
    var bodyKey = "bodyReset";
    var inventoryKey = "inventoryReset";

    /**
     * Sets the restore point.
     * Should be called during scene-arrival.
     */
    this.set = function () {
        global.kobold.history.setIfNotSet(id, statusKey, JSON.parse(JSON.stringify(global.kobold.status)));
        global.kobold.history.setIfNotSet(id, statsKey, JSON.parse(JSON.stringify(global.kobold.stats)));
        global.kobold.history.setIfNotSet(id, bodyKey, JSON.parse(JSON.stringify(global.kobold.body)));
        global.kobold.history.setIfNotSet(id, inventoryKey, JSON.parse(JSON.stringify(global.kobold.inventory)));
    };

    /**
     * Resets the entire scene to the starting point.
     * Starting point MUST first have been set with set().
     * Reverts inventory, body, shared history, stats and statuses.
     * Note that the main panel will still need to be cleared and reloaded after this.
     */
    this.reset = function () {
        global.kobold.history.shared.varanar.dayThree.afternoon.load();
        global.kobold.status.load(global.kobold.history.getKey(id, statusKey));
        global.kobold.stats.load(global.kobold.history.getKey(id, statsKey));
        global.kobold.body.load(global.kobold.history.getKey(id, bodyKey));
        global.kobold.inventory.load(global.kobold.history.getKey(id, inventoryKey));
        global.kobold.history.set(id, {});
        that.fakeAutosave();
    };

    /**
     * Reloads the scene.
     * Use after resetting.
     */
    this.reload = function () {
        $("#content").fadeOut(400, function () {
            ui.load();
            $("#content").fadeOut(0);
            $("#content").fadeIn(400);
        });
    };
}

// END scenes\Varanar\DayThree\AfternoonThree\AfternoonThreeController.js

// scenes\Varanar\DayThree\AfternoonThree\AfternoonThree.js

/**
 * Afternoon three of Varanar.
 */
function AfternoonThree() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var sceneId = "AfternoonThree";
    var sceneName = "Varanar, Afternoon Three";
    var sceneSubfolder = "Varanar/DayThree";
    var ctrl = new AfternoonThreeController(sceneId, this);
    var arrival = new AfternoonThreeArrival(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Sets all the required global variables such as scene id, scene name,
     * scene stack and theme.
     */
    function setGlobals() {
        global.currentScene = that.id;
        global.currentSceneName = that.name;
        global.currentSceneFolder = that.subfolder ? that.subfolder : "";
        global.kobold.history.addToSceneStack(that.id);
    }

    /**
     * Builds the tab as a HTML string.
     * @returns {string} The tab, as a HTML string.
     */
    function buildTab() {
        var tab = global.ui.buildGenericHeader();
        tab += global.parser.parse(`[t]Varanar[/t]`);
        tab += global.parser.parse(`[st]Afternoon three[/st]`);
        tab += arrival.build();
        tab += global.ui.buildGenericFooter();
        return tab;
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = sceneId;
    this.name = sceneName;
    this.subfolder = sceneSubfolder;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Loads this scene.
     */
    this.load = function () {
        setGlobals();
        var tab = buildTab();
        global.ui.loadToMain(tab);
        arrival.registerListeners();
        ctrl.autosave();
    };
}

global.sceneLoader.queueRegister(function () {
    var scene = new AfternoonThree();
    global.sceneLoader.registerScene(scene.id, scene);
});

// END scenes\Varanar\DayThree\AfternoonThree\AfternoonThree.js



// END SCENE AfternoonThree

